<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * Holds a collectino of attribute objects and is capable 
 *	displaying those objects in standard formats
 */
class shAttributeCollection extends shCollection
{
	/**
	 * The load method can take the following parameters: 
	 *	NONE 
	 *	Currently, this automatically displays only those 
	 *	attributes that are marked active.
	 */
	public function load (iwParameterCollection $params,$countOnly=false)
	{				
		$this->_params = $params;
		
		$limit = $this->getLimit ($params);				
		$orderby = $this->getOrderBy ($params);
		$direction = $this->getOrderDirection($params);
        
		// No parameters means load all attributes
		$queryCount = 'SELECT COUNT(*) AS total FROM '.$GLOBALS['TABLE_ATTRIBUTES']['name'].' WHERE status = '.ATTRIBUTE_STATUS_ACTIVE;
        app()->flushCacheIfInvalid($queryCount);
		$count = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$queryCount);
		$this->_total = $count[0]['total'];			

		$query = 'SELECT * FROM '.$GLOBALS['TABLE_ATTRIBUTES']['name'].' WHERE status = '.ATTRIBUTE_STATUS_ACTIVE;
		$query .= $orderby.' '.$direction;
		$query .= $limit;
		
        app()->flushCacheIfInvalid($query);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);

		if ($rows !== false)
		{						
							
			foreach ($rows as $row)
			{
				// Now load the data into the object.
				$attrib = new shAttribute;
				$attrib->setDBKeyValueArray ($row);
				$this->addItem ($attrib);
			}
			            
			$this->setIcon (shAttribute::getItemIcon(true));
			$this->setTitle ('All Tags');				
			$this->setSubtitle ('('.$this->count().' tags total)');
		}
		else {
			$this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		}
		
	}
	
	/**
	 * No filters supported
	 */
	public function processFilters ()
	{		
		// Don't do anything 
		return new iwParameterCollection();
	}
	
	/**
	 * No filters supported 
	 */
	public function getFilterHtml ()
	{
		$html = '';
		return $html;
	}

	/**
	 * Returns "shAttribute"
	 */
	public function getClassName()
	{
		return 'shAttribute';
	}
	
	/**
	 * Fills the collection with attributes that belong to the 
	 *	document with the given id.
	 *	@param int $docid The Id of the document.
	 *	@return bool Returns true if loaded successfully, false otherwise.
	 */
	public function loadDocumentTags($docid)
	{
		$doc = new shItemDocument();
		if ($doc->loadId($docid))
		{
			$this->_objects = $doc->getAttributes();
			return true;
		}
		
		return false;
	}
	
	/**
	 * Gets the stored collection of tags in a non-list form.  You
	 *	can optionally have it show the 'Add Tag To Document' form 
	 *	by passing in the document id to which the tags should be
	 * added.
	 *	@param int $docid The id of the document to add new tags to. 
	 *				If 0 then the tag form will not be shown.
	 *	@return string The html for displaying the tags.
	 */
	public function getTagsAsHtml($docid=0)
	{
		$smarty = app()->getSmartyObject();
		
		$tags = array();
		foreach ($this->_objects as $tag)
		{
			$tags[] = array ('name'=>$tag->get(PROP_NAME), 'id'=>$tag->get(PROP_ID));
		}
		$smarty->assign('tags', $tags);
		
		if ($docid > 0 && app()->isAdminLoggedIn())
		{		
			$smarty->assign('show_tagform', true);
			$smarty->assign('document_id', $docid);
		}
		else
		{
			$smarty->assign('show_tagform', false);
			$smarty->assign('document_id', 0);			
		}
				
		return smarty_templ_fetch('controls/control.view.tags.simple.tpl.html',$smarty);		
	}

	/**
	 * Allows sorting on the following fields:
	 *	@arg @c name
	 */
	protected function getSortableFields ()
	{
		
		return array (
				'Name'=>$GLOBALS['TABLE_ATTRIBUTES']['fields']['ATTRIBUTE_NAME'],
				);
	}

	
	/**
	 * Returns the action buttons available for this collection
	 */
 	public function getFormButtons ()
 	{
 		if (app()->isAdminLoggedIn())
		{
			return nx_load_template(array(),'collections/collection.actions.tags.tpl.html');
		}
 	}	
	
}

?>
