<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shGroupCollection extends shCollection
{
	/**
	 * Builds a query based on the given collection of parameters
	 *	@param iwParameterCollection $params The parameters that will be used to generate
	 *					the sql query.
	 *	@param bool $countOnly True if a count query should be returned
	 */
	protected function buildQuery (iwParameterCollection $params, $countOnly)
	{
		$this->_params = $params;
		
		$select = '';
		$query = '';
		
		if ($countOnly)
		{
			$select = 'select COUNT(*) AS total ';
		}
		else
		{
			$select = 'select * ';
		}
		
		// No group filtering
		$query = $select.'FROM sh_groups AS sg ';
		
		if ($params->have (FILTER_COMPANY))
		{
			$query .= ', sh_company_group_map AS scgm';
		}
		
		$query .= ' WHERE status <> '.GROUP_STATUS_DELETED;
						
		// Filter by company
		if ($params->have(FILTER_COMPANY))
		{
			$companies = array ();
			
			$query .= ' AND scgm.group_id = sg.group_id ';
							
			foreach ($params->find_value(FILTER_COMPANY) as $company)
			{
				if ($company!=0)
				{
					$companies[] = '(scgm.company_id = '.$company.') ';
				}
			}
			
			if (count ($companies))
			{
				$query .= 'AND ('.implode (' OR ',$companies).')'; 
			}
		}
				
		$query .= $this->getOrderBy($params);
		$query .= ' '.$this->getOrderDirection($params);
			
        app()->flushCacheIfInvalid($query);	
        
		return $query;
	}
	
	/**
	 * The following parameters are accepted
	 *	@arg FILTER_COMPANY
	 */
	function load (iwParameterCollection $params,$countOnly=false)
	{
				
		$limit = $this->getLimit ($params);				
		$queryCount = $this->buildQuery ($params, true);
		$count = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$queryCount);
		$this->_total = $count[0]['total'];			
		
		$query = $this->buildQuery ($params, false);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		
		if ($rows !== false)	
		{																	
		
			foreach ($rows as $row)
			{
				// Now load the data into the object.
				$group = new shGroup;
				$group->setDBKeyValueArray ($row);
				$this->addItem ($group);
			}

			// Now add the 'ALL' group
			if ($params->have(FILTER_INCLUDE_AGGREGATE))
			{
				$group = new shGroup;
				$group->set(PROP_ID, GROUP_ALL);
				$group->set(PROP_NAME, str('STR_GROUP_ALL'));
				$group->set(PROP_DESCRIPTION, str('STR_GROUP_ALL_DESC'));
				$this->addItem ($group);
			}
			
			$this->setIcon (shGroup::getItemIcon(true));
			$this->setTitle (str('STR_GROUP_ALL'));				
			$this->setSubtitle (sprintf(str('STR_GROUPS_TOTAL'),$this->count()));
		}
		else {
			$this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		}
	}
	
	
	public function processFilters ()
	{
		// Don't do anything 
		return new iwParameterCollection();
	}
	
	function getFilterHtml ()
	{
		$html = '';
		return $html;
	}
		
	function getSortableFields ()
	{
		
		return array (
				str('STR_SORT_NAME')=>$GLOBALS['TABLE_GROUPS']['fields']['GROUP_NAME'],
				);
	}

 	function getFormButtons ()
 	{
		if (app()->isAdminLoggedIn())
		{
 			$buttons = '<table cellpadding="2" cellspacing="0"><tr><td colspan="2"><div style="font-weight:bold;">'.str('STR_SELECTED_ITEMS').':</div></td></tr>';
			$buttons .= '<tr><td><input type="submit" name="delGroup" value="'.str('STR_ACTION_TRASH').'"/></td></tr>';
			$buttons .= '</table>';
 			return $buttons;
		}
 	}	

	function getClassName()
	{
		return 'shGroup';
	}
}

?>
