<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * When loading a collection using the load method, you can use any of the 
 * following filters:
 * FILTER_DOCTYPE 
 * FILTER_USER (use iwParameterArray)
 * FILTER_COMPANY (use iwParameterArray)
 * FILTER_GROUP (use iwParameterArray)
 * FILTER_STATUS
 * FILTER_CATEGORY
 * FILTER_MODIFY_DATE
 * FILTER_KEYWORDS
 * FILTER_ATTRIBUTES
 *
 */
class shItemDocumentCollection extends shItemCollection
{
	protected $_lastQuery = '';

	
	function getAdminActions ()
	{
		return array ();
	}

	function getActions ()
	{
		return array ();
	}

	function getLastQuery()
	{
		return $this->_lastQuery;
	}

	function setItemByIds ($ids)
	{

	}

	function buildQuery (iwParameterCollection $params, $countOnly)
	{
		$select = '';
		$query = '';

		if ($countOnly)
		{
			$select = 'select COUNT(DISTINCT(si.item_id)) AS total ';
		}
		else
		{
			$select = 'select DISTINCT(si.item_id),si.item_title,si.item_description,'.
						'si.item_type_id,si.item_subtype_id,si.item_type_version,'.
						'si.item_original_filename,si.item_data,si.item_created,'.
						'si.item_modified,si.item_status ';
		}

		$from = 'sh_items as si, sh_item_category_map as scm, sh_item_access_map as sam ';
		$query = $select.' FROM '.$from.' WHERE 1 ';

		if ( !$params->have(FILTER_USER) &&
		(!$params->have(FILTER_COMPANY) || !$params->have(FILTER_GROUP)))
		{
			// No user filtering
			$query .= ' AND scm.item_id=si.item_id ';
		}
		else
		{
			$cid = IW_NULL;
			$gid = IW_NULL;				
			if (!$params->have(FILTER_USER))
			{
				// Use the company and group stored in the filter
				$companies = $params->find_value(FILTER_COMPANY);
				$groups = $params->find_value(FILTER_GROUP);
				$cid = $companies[0];
				$gid = isset($groups[0])?$groups[0]:GROUP_ALL;
			}
			else
			{
				$users = $params->find_value(FILTER_USER);
				if (!is_array($users))
				{
					$users = array($users);
				}
				
				$user = null;				
				if (count($users) == 0)
				{
					$user = app()->getLoggedInUserObject();
					if ($user)
					{
						$cid = $user->getCompany()->get(PROP_ID);
						$gid = $user->getGroup()->get(PROP_ID);
					}
				}
				else
				{
					$user = null;
					if (!is_object($users[0]))
					{
						$user = new shUser;
						$user->set(PROP_USERNAME, $users[0]);
						if (!$user->load())
						{
							$user = false;
						}
					}
					else
					{
						$user = $users[0];
					}
					
					if ($user)
					{
						$cid = $user->getCompany()->get(PROP_ID);
						$gid = $user->getGroup()->get(PROP_ID);
					}
					else
					{
						app()->addError (sprintf(app()->loadString('STR_USERNOTFOUND'),$users[0]), ERROR_TYPE_SERIOUS);
					}
				}
			}

			if ($cid == COMPANY_UNREGISTERED)
			{
				// When querying for the "EVERYONE"  company, we don't want to include
				//	those documents that are available to all companies/groups because
				//	that still requires a registered user whereas "EVERYONE" means anyone
				//	(registered or not).
				$query .= '  AND (sam.company_id='.$cid.') '.
						'AND (sam.group_id='.GROUP_ALL.') AND ' .
						'sam.item_id = si.item_id ';
						$query .= ' AND scm.item_id=si.item_id ';

			}
			else if ($cid != IW_NULL || $gid != IW_NULL)
			{
				$query .= '  AND (sam.company_id='.$cid.' OR sam.company_id<=0) '.
						'AND (sam.group_id='.$gid.' OR sam.group_id<=0) AND ' .
						'sam.item_id = si.item_id ';
						$query .= ' AND scm.item_id=si.item_id ';
			}
			else
			{
				// As a security measure we prevent the output from showing anything at all.
				if ($countOnly)
				    $query = 'SELECT COUNT(*) FROM sh_items WHERE 0';
				else
				    $query = 'SELECT * FROM sh_items WHERE 0 ';
			}
		}

		// Filter by document status
		if ($params->have(FILTER_STATUS))
		{
			$requestedStatuses = $params->find_value(FILTER_STATUS);
			$statuses = array ();
			foreach ($requestedStatuses as $status)
			{
				$statuses[] = '(si.item_status = '.$status.') ';
			}
			$query .= 'AND ('.implode (' OR ',$statuses).')';
		}

		// Filter by category
		if ($params->have(FILTER_CATEGORY))
		{
			$requestedCategories = $params->find_value(FILTER_CATEGORY);
			$categories = array ();
			foreach ($requestedCategories as $category)
			{
				$categories[] = '(scm.category_id = '.$category.') ';
			}

			if (count ($categories))
			{
				$query .= 'AND ('.implode (' OR ',$categories).')';
			}
		}

		// Filter by modification date
		if ($params->have(FILTER_MODIFY_DATE))
		{
			$dates = $params->find_value(FILTER_MODIFY_DATE);
			if ($dates['start'])
			{
				$startDate = iwUtilities::databaseDateFormat ($dates['start']);
				$query .= 'AND (item_modified > '.$startDate.')';
			}
			if ($dates['end'])
			{
				$endDate = iwUtilities::databaseDateFormat ($dates['end']);
				$query .= 'AND (item_modified < '.$endDate.')';
			}
		}

		// Filter by keyword (used in search)
		if ($params->have(FILTER_KEYWORDS))
		{
			$statement = array();
			$words = $params->find_value(FILTER_KEYWORDS);
			foreach ($words as $word)
			{
				//$statement[] = '(item_title LIKE "%'.$word.'%" || item_description LIKE "%'.$word.'%" || item_serialized LIKE "%'.$word.'%")';
				$statement[] = '(item_title LIKE "%'.$word.'%" || item_description LIKE "%'.$word.'%")';
			}

			if (count($statement)>0)
			{
				$query .= sprintf ('AND (%s)',implode (' OR ', $statement));
			}
		}

		if (!$countOnly)
		{
			$query .= $this->getOrderBy($params);
			$query .= ' '.$this->getOrderDirection($params);

			// These two filters are done post query so we cannot
			//	limit the result set here.  We have to do it post-processing
			//	as well.  THIS IS A PROBLEM THAT HAS NOT BEEN SOLVED.  As it
			//	stands now, when these filters are specified there is no pagination
			//	supported.
			if (!$params->have(FILTER_ATTRIBUTES) &&
			!$params->have(FILTER_INBIN))
			{
				$query .= ' '.$this->getLimit ($params);
			}
		}
		
		// Note, the FILTER_DOCTYPE has to be applied after the results are returned.
		// Note, the FILTER_INBIN has to be applied after the results are returned.
        app()->flushCacheIfInvalid($query);
        
		return $query;
	}

	function getClassName()
	{
		return 'shItemDocument';
	}

	function loadFromQuery ($query)
	{
		$this->_lastQuery = $query;

		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($rows !== false)
		{
			// Now load the data into the object.
			foreach ($rows as $row)
			{
				// Check the doctype filter
				$obj = new shItemDocument;
				$obj->setDBKeyValueArray ($row);                
				$this->addItem ($obj);
			}

			return true;
		}

		return false;
	}

	public function load (iwParameterCollection $params, $countOnly=false)
	{
		$this->_params = $params;
		$queryCount = $this->buildQuery ($params, true);
		$count = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$queryCount);
		if ($count !== false)
		{
			$this->_total = $count[0]['total'];
			if ($countOnly == true)
			{
				return;
			}
		}
		else
		{
			$this->_total = 0;
			return;
		}

		$query = $this->buildQuery ($params, false);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);

		$this->_lastQuery = $query;

        $username = app()->getLoggedInUsername();
		if ($rows !== false)
		{
			// Now load the data into the object.
			foreach ($rows as $row)
			{
                // Make sure the user can actually view this document.
                if (!shAuthorize::canViewDocument($username,$row['item_id']))
                {
                    continue;
                }
                
				// Check the doctype filter
                
				$obj = new shItemDocument;
				$obj->setDBKeyValueArray ($row);
                                
				if ($params->have(FILTER_DOCTYPE))
				{
					$requestedDocTypes = $params->find_value(FILTER_DOCTYPE);
					if (!in_array ($obj->getDocumentType(),$requestedDocTypes))
					{
						// Go on to the next without adding if the type is not
						//	in the given array.
						$this->_total--;
						continue;
					}
				}

				// Check whether the item has any one of the given attributes
				if ($params->have(FILTER_ATTRIBUTES))
				{
					$found = false;

					$givenAttribNames = $params->find_value(FILTER_ATTRIBUTES);
					$attributes = $obj->getAttributes();
					foreach ($attributes as $attrib)
					{
						foreach ($givenAttribNames as $givenAttrib)
						{
							if (strcasecmp($attrib->get(PROP_NAME), $givenAttrib) == 0)
							{
								$found=true;
								break;
							}
						}

						if ($found)
						{
							break;
						}
					}

					if (!$found)
					{
						$this->_total--;
						continue;
					}
				}

				// Check whether the item is in the bin
				if ($params->have(FILTER_INBIN))
				{
					if ($params->find_value(FILTER_INBIN) != shItemBin::isItemInBin ($obj->get(PROP_ID)))
					{
						$this->_total--;
						continue;
					}
				}

				// Add directly to object list to avoid modifying the _total field indirectly
				$this->_objects[] = $obj;
			}

		}
		else {
			$this->addErrorObject ($rows, ERROR_TYPE_SERIOUS);
		}
	}

	public function processFilters ()
	{

		if (!isset ($_GET[URL_FILTERED]))
		{
			// Clear the filters and all remembered parameters
			//	if the 'filtered' param is not given.
			$this->storeParameters(new iwParameterCollection());
		}

		$params = $this->getStoredParameters();

		$category = @$_POST[FORM_FILTER_CATEGORY];
		if ($category != '')
		{
			$p = $params->add(new iwParameterArray(FILTER_CATEGORY));
			$p->addItem_nokey ($category);
		}

		$keywords = @$_POST[FORM_KEYWORD_FILTER];
		if ($keywords != '')
		{
			$p = $params->add(new iwParameterArray(FILTER_KEYWORDS));

			$keyword_arr = split(' ', $keywords);
			foreach ($keyword_arr as $keyword)
			{
				$p->addItem_nokey ($keyword);
			}
		}

		if (!app()->isAdminLoggedIn())
		{
			$user = app()->getLoggedInUserObject();
			$p = $params->add(new iwParameterArray(FILTER_USER));
			$p->addItem_nokey($user->get(PROP_USERNAME));
		}

		if (!$params->have(FILTER_SORTBY))
		{
			$params->add (new iwParameter(FILTER_SORTBY,'item_title'));
			$params->add (new iwParameter(FILTER_SORTDIR,'asc'));
		}

		$this->storeParameters ($params);

		return $params;
	}

	public function html ($maxPerPage=10,$currentPage=1,$showHeader=true)
	{
		$html = parent::html($maxPerPage,$currentPage,$showHeader);
		$smarty = app()->getSmartyObject();
		$rename = smarty_templ_fetch('forms/form.ajax.rename.tpl.html',$smarty);
		$html = $html.$rename;
		return $html;
	}

	function getFilterHtml ()
	{
		$html = '';

		if (!$this->isViewingBin() && $this->_showFilter)
		{
			$html .= $this->getCategoryFilterControl();
		}

		return $html;
	}

	function isViewingBin ()
	{
		$isViewingBin = (@$_GET[URLARG_ACTION] == 'view_bin');
		return $isViewingBin;
	}

	function getFormButtons ()
	{
		$smarty = app()->getSmartyObject();
		$smarty->assign('viewing_bin', $this->isViewingBin());
		return smarty_templ_fetch('collections/collection.actions.document.tpl.html',$smarty);
	}
	
	function getSortableFields ()
	{

		return array (
				str('STR_DOC_SORT_NAME')=>$GLOBALS['TABLE_ITEM']['fields']['ITEM_NAME'],
				str('STR_DOC_SORT_CREATE')=>$GLOBALS['TABLE_ITEM']['fields']['ITEM_CREATION_DATE'],
				str('STR_DOC_SORT_MODDATE')=>$GLOBALS['TABLE_ITEM']['fields']['ITEM_MODIFIED_DATE']
				);
	}

	function toArray()
	{
		$final = array();
		foreach ($this->_objects as $doc)
		{			
			$arr = array (
					'id'=>$doc->get(PROP_ID),
					'name'=>$doc->get(PROP_NAME),
					'description'=>iwUtilities::makeStringJavascriptSafe($doc->get(PROP_DESCRIPTION)),
					'icon'=>$doc->getItemIcon(false),
					'link'=>"index.php?".URLARG_ACTION."=".URLACTION_VIEWDOC."&".URLARG_ID."=$doc->id",
					'date'=>iwUtilities::dateFormat($doc->get(PROP_MODIFIED_DATE), false));

					$final[] = $arr;
		}
		
		
		return $final;
	}
}

?>
