<?php

define ('FIELD_TYPE_TEXT', 'text');
define ('FIELD_TYPE_TEXTAREA','textarea');
define ('FIELD_TYPE_HTML', 'html');
define ('FIELD_TYPE_PASSWORD','password');
define ('FIELD_TYPE_SELECT','select');
define ('FIELD_TYPE_CHECKBOX','checkbox');
define ('FIELD_TYPE_MULTI_SELECT','multiselect');
define ('FIELD_TYPE_HIDDEN', 'hidden');
define ('FIELD_TYPE_SUBMIT', 'submit');
define ('FIELD_TYPE_UPLOADFILE','uploadfile');
define ('FIELD_TYPE_STATIC','static');

// With the link type, the fieldValue is the text of the link
//	and the fieldOptions holds the link itself 
define ('FIELD_TYPE_LINK','link');

define ('FLD_VALUE_TYPE_STRING','string');
define ('FLD_VALUE_TYPE_NUMBER','number');

class iwFormField
{
	public $label = '';
	public $fieldType = '';
	public $fieldValueType = '';
	public $fieldId = '';
	public $fieldHelp = '';
	public $fieldOptions = null;
	public $fieldValue = null;
	public $fieldName = '';
	public $class = '';
    public $parentForm = null;
	
	public function iwFormField($type='', $value='', $name='', $label='')
	{
		$this->fieldType = $type;	
		$this->fieldValue = $value;
		$this->fieldName = $name;
		$this->label = $label;
	}
		
    public function setParentForm($form)
    {
        $this->parentForm = $form;
    }
    
	public function render()
	{		
		if ($this->fieldType == FIELD_TYPE_HIDDEN)
		{
			return $this->getHtml();
		}		
		else
		{            
			$smarty = app()->getSmartyObject();            
			$smarty->assign('label',$this->label);
			$smarty->assign('field',$this->getHtml());
			$smarty->assign('help', $this->fieldHelp);                        
			$html = smarty_templ_fetch('forms/form.field.generic.tpl.html',$smarty);                        
            return $html;
		}
	}
		
	public function getHtml()
	{
		$val = "";
		
		// Do not remove this or certain scripts may break.
		if ($this->fieldId == '')
		{
			$this->fieldId = $this->fieldName; 
		}
		
		switch ($this->fieldType)
		{
			case FIELD_TYPE_STATIC:
				$val = "<div id='$this->fieldId' class='$this->class'>$this->fieldValue</div>";
				break;
				
			case FIELD_TYPE_LINK:
				$val = "<a id='$this->fieldId' class='$this->class' href='$this->fieldOptions'>$this->fieldValue</a>";
				break;

			case FIELD_TYPE_UPLOADFILE:
				
				$val = "<input name='MAX_FILE_SIZE' type='hidden' id='MAX_FILE_SIZE' value='{DOCUMENT_MAX_FILESIZE}'>";
				$val .= "<input size='70' style='width:100%' id='$this->fieldId' name='$this->fieldName' type='file' />";
				break;
				
			case FIELD_TYPE_PASSWORD:
				$val = "<input type='password' class='$this->class' id='$this->fieldId' name='$this->fieldName' value='$this->fieldValue'/>";
				break;
				
			case FIELD_TYPE_TEXT:
				$val = "<input type='text' class='$this->class' id='$this->fieldId' name='$this->fieldName' value='$this->fieldValue'/>";
				break;
				
			case FIELD_TYPE_HTML:
				$val = "<textarea id='$this->fieldId' class='$this->class' name='$this->fieldName'>$this->fieldValue</textarea>";
                $val .= "
                    <script type='text/javascript'>var editor_{$this->fieldId} = new YAHOO.widget.SimpleEditor('{$this->fieldId}', {
                        height: '300px',
                        width: '90%',
                        handleSubmit:true,
                        dompath: false //Turns on the bar at the bottom
                    });
                    editor_{$this->fieldId}.render();
                    </script>";
				break;
								
			case FIELD_TYPE_TEXTAREA:
				$val = "<textarea id='$this->fieldId' class='$this->class' name='$this->fieldName'>$this->fieldValue</textarea>";
				break;
				
			case FIELD_TYPE_SELECT:
				$val = $this->assembleSelect(false);
				break;
				
			case FIELD_TYPE_CHECKBOX:			
				$checked = $this->fieldValue ? 'checked' : '';
				$val = "<input type='checkbox' name='$this->fieldName' class='$this->class' value='1' $checked>";
				break;
				
			case FIELD_TYPE_MULTI_SELECT:
				$val = $this->assembleSelect(true);
				break;
				
			case FIELD_TYPE_HIDDEN:
				$val = "<input type='hidden' name='$this->fieldName' class='$this->class' value='$this->fieldValue'/>";
				break;
				
			case FIELD_TYPE_SUBMIT:
				$val = "<input type='submit' value='$this->fieldValue' class='$this->class' name='$this->fieldName'/>";
				break;						
		}
		
		return $val;
	}
	
	/**
	 * Creates a select element with the info stored in fieldOptions 
	 * (which is in the form: (value=>name,value=>name, etc).
	 *
	 * @param bool $isMultiselect True to make it a multi select control
	 * @return string The html for the select
	 */
	public function assembleSelect($isMultiSelect)
	{		
		$multi = $isMultiSelect ? 'multiple' : '';
		$val = "<select $multi id='$this->fieldId' name='$this->fieldName'>";
        
        if (is_array($this->fieldOptions))
        {
		    foreach($this->fieldOptions as $optionVal=>$optionText)
		    {
			    $isSelected = ($this->fieldValue == $optionVal)?'selected':'';
			    $val .= "<option value='$optionVal' $isSelected>$optionText</option>";
		    }
		    $val .= "</select>";				
        }
        
		return $val;		
	}
}
?>