<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class iwRssItem 
{
	public $title;
	
	/**
	 * The body of the rss item as it will be displayed in a feed.
	 *
	 * @var string
	 */
	public $description;
	
	public $link;
	public $date;	
	public $creator;
	public $subject;
	
	
	/**
	 * Generates the XML needed to represent this item in an 
	 * RSS feed.
	 *
	 */
	public function generateXml ()
	{
		$xml = "<item rdf:about=\"$this->link\">\r\n";
		$xml .= iwUtilities::getTag ('title', html_entity_decode($this->title), false);		
		$xml .= iwUtilities::getTag ('link', $this->link, false);
		$xml .= iwUtilities::getTag ('description', html_entity_decode($this->description),false);
		$xml .= iwUtilities::getTag ('dc:date', iwUtilities::dateFormatRss($this->date),false);
		$xml .= iwUtilities::getTag ('dc:creator', $this->creator, true);
		$xml .= iwUtilities::getTag ('dc:subject', $this->subject, true);
		$xml .= "</item>\r\n";
		
		return $xml;
	}
}

/**
 * The Rss generator is used to create valid RSS2 feeds based on 
 * content provided.
 *
 */
class iwRssGenerator
{
	protected $feedTitle;
	protected $feedUrl;
	protected $feedDescription;
	protected $feedPublisher;
	protected $feedCreator;
	protected $feedSubject;
	protected $feedImage;
	protected $imageLink;
	
	protected $feedItems = array();
		
	public function __construct()
	{
		$this->feedTitle = 	'';
		$this->feedUrl = '';
		$this->feedDescription = '';
	}
	
	public function __set ($member, $value)
	{
		if (isset ($this->$member))
		{
			$this->$member = $value;
		}
	}
	
	/**
	 * Add a news item to the collection - it will be included
	 * during generation of XML.
	 *
	 * @param iwRssItem $item An rss item that was prepopulated
	 */
	public function addNewsItem ($item)
	{
		$this->feedItems[] = $item;
	}
	
	/**
	 * Add a news item to the list 
	 *
	 * @param string $title The title of the item
	 * @param string $body The description of the item
	 * @param string $url The url to the rss item
	 * @param string $date A date in any string form
	 */
	public function addNewsItemParams ($title, $body, $url, $date)
	{
		$item = new iwRssItem();
		$item->title = $title;
		$item->description = $description;
		$item->link = $url;
		$item->date = $date;
		
		$this->addNewsItem ($item);
	}
	
	/**
	 * Generates valid RSS XML based on items that 
	 *  have been added to the generator using addNewsItem
	 *
	 */
	public function generateXml ()
	{
		// BEGINNING OF CHANNEL DESCRIPTOR
		$xml = '<?xml version="1.0" encoding="UTF-8"?>'."\r\n".
			   '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf'.
			   '-syntax-ns#" xmlns="http://purl.org/rss/1.0/" '.
			   'xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:syn'.
			   '="http://purl.org/rss/1.0/modules/syndication/">'."\r\n";
			   
		$xml .= "<channel rdf:about=\"$this->feedUrl\">\r\n";
		$xml .= iwUtilities::getTag ('title', $this->feedTitle, false);		
		$xml .= iwUtilities::getTag ('link', $this->feedUrl, false);
		$xml .= iwUtilities::getTag ('description', html_entity_decode($this->feedDescription),false);
		
		// OPTIONAL DESCRIPTORS
		$xml .= iwUtilities::getTag ('dc:language', 'en-us', true);
		$xml .= iwUtilities::getTag ('dc:date', iwUtilities::dateFormatRss(time()), true);
		$xml .= iwUtilities::getTag ('dc:publisher', $this->feedPublisher, true);
		$xml .= iwUtilities::getTag ('dc:creator', $this->feedCreator, true);
		$xml .= iwUtilities::getTag ('dc:subject', html_entity_decode($this->feedSubject), true);
		
		// ITEM SEQUENCE
		$xml .= '<items><rdf:Seq>';
		foreach ($this->feedItems as $item)
		{
			$xml .= '<rdf:li rdf:resource="'.$item->link.'" />'."\r\n";
		}
		$xml .= '</rdf:Seq></items>'."\r\n";
		
		// END OF CHANNEL DESCRIPTOR
		$xml .= '<image rdf:resource="'.$this->imageLink.'" />'."\r\n";
		$xml .= "</channel>\r\n";
		
		// IMAGE ITEM
		$imageArea = iwUtilities::getTag ('title',$this->feedTitle, false);
		$imageArea .= iwUtilities::getTag ('url',$this->feedImage, false);
		$imageArea .= iwUtilities::getTag ('link',$this->feedUrl, false);	
		$xml .= iwUtilities::getTag ('image', $imageArea, true);
				
		// ITEM DETAILS
		reset ($this->feedItems);
		foreach ($this->feedItems as $item)
		{
			$xml .= $item->generateXml()."\r\n";
		}		
		
		$xml .= "</rdf:RDF>\r\n";
		
		return $xml;
	}	
}

?>