<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

abstract class iwUser extends iwDbTable
{
	protected $_hiddenHash = IW_HIDDEN_HASH;
	protected $_firstTimeLogin = true;
	
	/** Constructor: Adds required properties*/
	function __construct () 
	{
		parent::__construct ();
	}	
	
	/**  Gets the hash specific to this user
     *	@return string An encrypted hash that's unique to this user
	*/
	function getUserHash () {
		return md5 ($this->get (PROP_USERNAME).$this->_hiddenHash);
	}
	
	/**
	 * Overridden to treat the standard PROP_ID as a
	 * PROP_USERNAME which is the way we uniquely identify
	 * users (not IDs).
	 */	
	function get($key)
	{
		if ($key == PROP_ID)
		{
			return $this->get(PROP_USERNAME);
		}
		
		return parent::get($key);
	}
	
	/**  Gets the user from the DB that has the given session_id 
     *	@param string $session_id The session ID to look up.
     *	@return bool true if the user was found and loaded
	*/
	function setUserFromSessionID ($sessionID) {	

		// Empty the error queue
		$this->clearErrors ();
		$tableConstant = $this->getTableConstant();
		$query = 'SELECT * FROM '.$GLOBALS[$tableConstant]['name'].' WHERE '.$GLOBALS[$tableConstant]['fields']['LAST_SESSION_ID'].'="'.$sessionID.'"';
		
		$fields = $GLOBALS['DBASE']->getAssoc ($query);
		if ( ($fields) && (count ($fields) == 1))	{						
				
			foreach ($fields as $key=>$row)  {			
				
				// Add the key field to the array of fields.
				$fields[$key][$this->getUniqueID (true)] = $key;
				
				// Now load the data into the object.
				return $this->setDBKeyValueArray ($fields[$key]);
			}
		}
		else if (DB::isError ($fields)){
			$this->addErrorObject ($fields, ERROR_TYPE_SERIOUS);
		}
		
		return false;
	}

	/**
	 * Loads data into the object in one call - you pass in the id 
	 * of the object - overridden because the ID field is actually
	 * username, not 'id'.
	 */
	public function loadId($id)
	{		
		if ($id != '')
		{			
			$this->username = $id;
			return $this->load();
		}
		else
		{
			
			return false;
		}
	}
	
	/**  Adds all associated properties
     *	This need only be called once per instantiation of this class
     *	and is handled automatically by the base class as long
     *	as its constructor is called.		
	*/
	function addProperties () {
		$this->_addProperty (PROP_FIRST_NAME, '');
		$this->_addProperty (PROP_LAST_NAME, '');
		$this->_addProperty (PROP_EMAIL, '');
		$this->_addProperty (PROP_USERNAME, '');		
		$this->_addProperty (PROP_PASSWORD, '');
		$this->_addProperty (PROP_DATEJOINED, 0);
		$this->_addProperty (PROP_LASTLOGIN, 0);
		$this->_addProperty (PROP_LAST_SESSIONID, 0);
		$this->_addProperty (PROP_DATE_LAST_ACTIVITY, 0);
		$this->_addProperty (PROP_USERTYPE, USER_TYPE_REGISTERED);
		$this->_addProperty (PROP_STATUS, 0);
		$this->_addProperty (PROP_HASH, '');
		$this->_addProperty (PROP_LOGIN_IP, '');
		$this->_addProperty (PROP_LOGIN_CLIENT_INFO, '');
		$this->_addProperty (PROP_CITY, '');
		$this->_addProperty (PROP_STATE, '');
		$this->_addProperty (PROP_REQUIRE_CONFIRM,0);
	}
	
	/**  Gets all the database field names with associated values
     *	This will return an associative array where the keys
     *	are the table field names and the values are the values
     *	stored in this class.
     *	@param bool $includeDBKey If true, then the key field for the table is returned in the array as well.
     *	@return array An associative array of dbase fields => class values
	*/
	function getDBKeyValueArray ($includeDBKey) {
	
		$tableConstant = $this->getTableConstant();
				
		$fields = array
				(
					$GLOBALS[$tableConstant]['fields']['USERNAME'] => $this->get (PROP_USERNAME),
					$GLOBALS[$tableConstant]['fields']['FIRST_NAME'] => $this->get (PROP_FIRST_NAME),
					$GLOBALS[$tableConstant]['fields']['LAST_NAME'] => $this->get (PROP_LAST_NAME),
					$GLOBALS[$tableConstant]['fields']['PASSWORD'] => $this->get(PROP_PASSWORD),
					$GLOBALS[$tableConstant]['fields']['DATE_JOINED'] => $this->get (PROP_DATEJOINED),
					$GLOBALS[$tableConstant]['fields']['DATE_LAST_LOGIN'] => $this->get (PROP_LASTLOGIN),
					$GLOBALS[$tableConstant]['fields']['LAST_SESSION_ID'] => $this->get (PROP_LAST_SESSIONID),
					$GLOBALS[$tableConstant]['fields']['LAST_ACTIVITY_DATE'] => $this->get (PROP_DATE_LAST_ACTIVITY),
					$GLOBALS[$tableConstant]['fields']['USER_TYPE'] => $this->get ('user_type'),
					$GLOBALS[$tableConstant]['fields']['STATUS'] => $this->get (PROP_STATUS),
					$GLOBALS[$tableConstant]['fields']['EMAIL'] => $this->get (PROP_EMAIL),
					$GLOBALS[$tableConstant]['fields']['HASH'] => $this->get ('hash'),
					$GLOBALS[$tableConstant]['fields']['LOGIN_IP'] => $this->get (PROP_LOGIN_IP),
					$GLOBALS[$tableConstant]['fields']['LOGIN_CLIENT_INFO'] => $this->get (PROP_LOGIN_CLIENT_INFO),
					$GLOBALS[$tableConstant]['fields']['CITY'] => $this->get (PROP_CITY),
					$GLOBALS[$tableConstant]['fields']['STATE'] => $this->get (PROP_STATE)
				);

		return $fields;
	}
	
	/**  Sets all the object properties based on the given database field values
     *	Given is an associative array where the keys are the database field names
     *	and the values are the values of those fields.
     *	@param array $dbFieldsAndValues The associative array of database field names and values.
     *	@return bool True if all the required fields were found and copied over.
	*/
	function setDBKeyValueArray ($dbFieldsAndValues) {
	
		$tableConstant = $this->getTableConstant();
		$this->set (PROP_FIRST_NAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['FIRST_NAME']]);
		$this->set (PROP_LAST_NAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['LAST_NAME']]);
		$this->set (PROP_USERNAME, $dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['USERNAME']]);
		$this->set (PROP_PASSWORD, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['PASSWORD']]);
		$this->set (PROP_DATEJOINED, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['DATE_JOINED']]);
		$this->set (PROP_LASTLOGIN, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['DATE_LAST_LOGIN']]);
		$this->set (PROP_LAST_SESSIONID, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['LAST_SESSION_ID']]);
		$this->set (PROP_DATE_LAST_ACTIVITY, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['LAST_ACTIVITY_DATE']]);
		$this->set ('user_type', @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['USER_TYPE']]);
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['STATUS']]);
		$this->set (PROP_EMAIL, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['EMAIL']]);
		$this->set ('hash', $dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['HASH']]);
		$this->set (PROP_LOGIN_IP, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['LOGIN_IP']]);
		$this->set (PROP_LOGIN_CLIENT_INFO, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['LOGIN_CLIENT_INFO']]);
		$this->set (PROP_CITY, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['CITY']]);
		$this->set (PROP_STATE, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['STATE']]);
		return $this->requiredFieldsValid ();
	}
	
	/**  
	*/
	protected function getDBKeyValueForDelete ($isRestoring=false) {
			$fields = array ($GLOBALS[$this->getTableConstant()]['fields']['STATUS'] => $isRestoring?USER_STATUS_ACTIVE:USER_STATUS_DELETED);								
			return $fields;
	}
	
	/**  Gets the key string for the associative array that contains field information for the table associated with this object.
     *	@see tables.inc.php
     *	@return string The key field string for the assocative array that contains field information for the table associated with this object.
	*/
	function getTableConstant () {
		return 'TABLE_USER';
	}
	
	/**  Checks if the data stored in this object is valid.
     *	All this does is verify that there is valid data in the *required*
     *	fields.
     *	@param bool $checkKey If this is true, then the unique key field for the object is checked for validity as well
     *	@return bool True, if all required fields are valid.
	*/
	function requiredFieldsValid ($checkKey = false) {
	
		$invalidField = false;
		
		if ($this->get(PROP_USERNAME) == '') {
			$this->addError (str('STR_USER_INVALID_USERNAME'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		if ($this->get(PROP_PASSWORD) == '') {
			$this->addError (str('STR_USER_INVALID_PASSWORD'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		if ($this->get(PROP_EMAIL) == '') {
			$this->addError (str('STR_USER_INVALID_EMAIL'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		
		return !$invalidField;
	}
	
	/**  Determines if a user has confirmed his registration
     *	To confirm, a user must get an email and click on a link in that 
     *	email to confirm him or herself.
     *	@return bool Returns true if the user is a confirmed user.
	*/
	function isConfirmed ($user) {		
		return $this->get (PROP_STATUS) != USER_STATUS_UNCONFIRMED;
	}
	
	/**
	 * Determines whether this user has ever logged in before.  This 
	 * will be set to false until the user 
	 *
	 * @return bool Returns true if this is the first time the user has logged in and/or has
	 * 				never logged in before.
	 */
	function isFirstTimeLogin () 
	{
		return $this->_firstTimeLogin;
	}
	
	/**
	 * No longer consider this user a first time login.  Note that if 
	 * the user logs out and back in again, this setting will be lost, but
	 * at that point it shouldn't matter since it won't be a first time
	 * login anymore
	 *
	 */
	function clearFirstTimeLogin ()
	{
		$this->_firstTimeLogin = false;
	}
	
	/**
	 * This will change the password to something random
	 * and return the new password string.  It is up to the caller to
	 * send the password to the user's email address.
	 *	@return mixed The new password if changed, otherwise false.
	 */
	function resetPassword ()
	{
		// start with a blank password
		$password = "";

		// define possible characters
		$possible = "0123456789bcdfghjkmnpqrstvwxyz";

		// set up a counter
		$i = 0;

		// add random characters to $password until $length is reached
		$length = 8;
		while ($i < $length) {

			// pick a random character from the possible ones
			$char = substr($possible, mt_rand(0, strlen($possible)-1), 1);

			// we don't want this character if it's already in the password
			if (!strstr($password, $char)) {
				$password .= $char;
				$i++;
			}

		}

		// Now change the password
		$this->set (PROP_PASSWORD,$this->_encryptPassword($password));
		if ($this->update ())
		{
			return $password;
		}
		
		return false;
	}
	
	/**  Populates this object with data from the user record with the given username
     *	@param string $username The name of the user to lookup in the database
     *	@return bool True, if the user was found and the data loaded, false otherwise.
	*/
	function setUser ($username) {
	
		$this->set (PROP_USERNAME, $username);
		if ($this->load ())
		{
			// Store whether or not the user has logged in before
			if ($this->get('login_ip') == '')
			{
				$this->_firstTimeLogin = true;
			}
			else 
			{
				$this->_firstTimeLogin = false;
			}
			
			return true;
		}
		
		return false;
	}
	
	/**  matches the unencoded password given to the encoded password stored in the database
     *	@pararm string $password The password that's entered by the user.
     *	@return True if the password matches the one in the database.
	*/
	function validatePassword ($password) {
		
		if ($password != '') {
			return ($this->_encryptPassword ($password) == $this->get('password'));
		}
		
		return false;
	}
	
	/**  Verifies that the stored email address is a valid one
     *	@return bool True if the email is valid, false otherwise.
	*/
	function _checkEmailFormat () {
	
		$email = $this->get (PROP_EMAIL);
		if (!ereg('^[-!#$%&\'*+\\./0-9=?A-Z^_`a-z{|}~]+'. '@'. '[-!#$%&\'*+\\/0-9=?A-Z^_`a-z{|}~]+\.' . '[-!#$%&\'*+\\./0-9=?A-Z^_`a-z{|}~]+$', $email)) {
			$this->addError (str('STR_USER_INVALID_EMAIL'), ERROR_TYPE_SERIOUS);
			return false;			
		}
		
		return true;
	}
	
	/**  Verifies that the stored username is a valid and secure one
     *	@return bool True if the username is valid, false otherwise.
	*/
	function _checkUsernameFormat () {
	
		$errorCount = 0;
		$username = $this->get (PROP_USERNAME);
		
	   // no spaces
		if (strrpos($username,' ') > 0) {
			$this->addError (str('STR_USER_NAME_NO_SPACES'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
	
		// must have at least one character
		if (strspn($username,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ") == 0) {
			$this->addError (str('STR_USER_NAME_1LETTER'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
	
		// must contain all legal characters
		if (strspn($username,"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_")!= strlen($username)) {
			$this->addError (sprintf(str('STR_USER_NAME_ILLEGAL_CHAR'),'a-z, A-Z, 0-9, -, _'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
	
		// min and max length
		if (strlen($username) < 5) {
			$this->addError (str('STR_USER_NAME_TOOSMALL'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
		if (strlen($username) > 35) {
			$this->addError (str('STR_USER_NAME_TOOBIG'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
	
		// illegal names
		if (eregi("^((root)|(bin)|(daemon)|(adm)|(lp)|(sync)|(shutdown)|(halt)|(mail)|(news)"
			. "|(uucp)|(operator)|(games)|(mysql)|(httpd)|(nobody)|(dummy)"
			. "|(www)|(cvs)|(shell)|(ftp)|(irc)|(debian)|(ns)|(download))$",$username)) {
			
			$this->addError (str('STR_USER_NAME_RESERVED'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
		if (eregi("^(anoncvs_)",$username)) {
			$this->addError (str('STR_USER_NAME_RESERVED'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
	
		return ($errorCount == 0);
	}
	
	/**  Verifies that the stored password is a valid and secure password
     *	@return bool True if the password is valid, false otherwise.
	*/
	function _checkPasswordFormat () {
		
		$errorCount = 0;
		$password = $this->get ('password');
		if (strlen ($password) <= 6) {
		
			$this->addError (str('STR_USER_PASS_TOOSMALL'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
		
		if (ctype_alpha ($password)) {
			
			$this->addError (str('STR_USER_PASS_ONEDIGIT'), ERROR_TYPE_SERIOUS);
			$errorCount++;
		}
		
		return ($errorCount==0);
	}
	
	/**  Changes this user's email address
     *	This will send a confirmation email to the user's new email address
     *	which the user will have to respond to before the user is reconfirmed.
     *	@param The new email address.
     *	@return bool True if the confirmation email was sent.
	*/
	function newEmailAddress ($email) {
	
		if ($this->_checkEmailFormat ($email)) {
		
			// First, mark this user as unconfirmed again
			$this->set (PROP_STATUS, USER_STATUS_UNCONFIRMED);
			
			// Set up the new email address
			$this->set (PROP_EMAIL, $email);
			
			// Now, try updating the record in the database with the
			//	new data
			if ($this->update ()) {
			
				// Now send the confirmation email address
				return $this->_sendConfirmationEmail ();
			}
		}
		return false;
	}
	
	/**  Registers a new user by adding them to the database and sending a confirmation email
     *	This function will not actually confirm the user. That 
     *	will be done by the confirmUser method of this class.
     *	@return bool True if the registration was successful, false otherwise.
	*/
	function newRegistration () {
	
			
		$this->clearErrors ();
	
		// The new email address
		$email = $this->get (PROP_EMAIL);
		
		// This is the hash that has to be matched in order to 
		//	confirm the email address once the user gets the email.
		$this->set ('hash', md5 ($email.$this->_hiddenHash));
		
		// Mark the user as unconfirmed
		if ($this->get('require_new_user_confirm'))
		{
			$this->set (PROP_STATUS, USER_STATUS_UNCONFIRMED);
		}
		else
		{
			$this->set (PROP_STATUS, USER_STATUS_ACTIVE);
		}
		
		// Store when the registration was done
		$this->set ('date_joined', time ());

		// Do not allow any user type but 'registered' automatically register.
		$this->set ('user_type', USER_TYPE_REGISTERED);

		// Make sure that there are no other users with that email address
		if (!app()->get('allow_duplicate_emails'))
		{
			if (!$this->verifyFieldUnique ($GLOBALS[$this->getTableConstant()]['fields']['EMAIL'], "'$email'")) {
				$this->addError (str('STR_USER_ALLOW_DUP_EMAIL'), ERROR_TYPE_SERIOUS);
				return false;
			}
		}
				
		// Make sure that there are no other users with that username
		//	Even thought the DB will catch this, we might as well look here to 
		//	display a gentler message.
		$username = $this->get (PROP_USERNAME);
		if (!$this->verifyFieldUnique ($GLOBALS[$this->getTableConstant()]['fields']['USERNAME'], "'$username'")) 
		{
			$this->addError (str('STR_USER_DUP_USERNAME'), ERROR_TYPE_SERIOUS);
			return false;
		}
							
		if ($this->add ()) 
		{					
			if ($this->get('require_new_user_confirm'))
			{
				// Send an email to the user.
				if (app()->sendRegistrationEmail ($this)) 
				{
					$this->addNotification (str('STR_USER_CONFIRM_SENT'));			
					return true;
				}
			}
			else
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**  Marks the user as confirmed in the database after verifying the hash and email
     *	@param string $hash  The hash string sent as a parameter in the confirmation link
     *	@param string $email The email sent as a parameter in the confirmation link
     *	@return bool True if the user was confirmed, false otherwise.
	*/
	function confirmUser ($hash, $email) {
	
	   	//verify that they didn't tamper with the email address
		$newHash=md5($email.$this->_hiddenHash);
		if ($newHash && ($newHash==$hash)) {
					
			$tableName = $GLOBALS[$this->getTableConstant()]['name'];
			$hashFieldName = $GLOBALS[$this->getTableConstant()]['fields']['HASH'];
			
			// Find this record in the db
			$query = 'SELECT * FROM '.$tableName.' WHERE '.$hashFieldName.'="'.$hash.'"';
			$results = app()->getDatabase()->getAssoc ($query);
			
			if ( ($results)) {
			
				foreach ($results as $key=>$fields) {
				
					// If we got here then the record was found
					//	so we should update this object with the data
					//	from the record.
					$this->setUser ($key);	
					
					// Now we should set the user object to be confirmed
					$this->set (PROP_STATUS, USER_STATUS_ACTIVE);
					
					// Now we need to update the record
					if ($this->update ()) 
					{									
						return true;
					}
				}
			} 
			else {			
				$this->addErrorObject ($results, ERROR_TYPE_SERIOUS);
			}
		} 
		else 
		{
			$this->addError (str('STR_USER_INVALID_CONFIRM_HASH'), ERROR_TYPE_SERIOUS);
		}	
		return false;
	}
	
	/**  Logs out this logged in user.
     *	All that's done here is that the user's session ID field
     *	is cleared so that it will never match what's returned
     *	by session_id.
     *	@return bool True if the user is logged in.
	*/
	function logout () {
	
		if ($this->isLoggedIn ()) 
		{
			$this->set ('last_session_id', '');
			if ($this->update ()) 
			{			
				return true;
			}
		}
		
		return false;
	}
	
	/**  Determines if this user is currently logged in	
     *	To determine if a user is logged in, the system will compare
     *	the current session ID (in PHP) to the session ID stored
     *	in the database for the user (which is updated each time 
     *	the user is logged in).  The session ID generally is stored
     *	as long as the user's browser is not closed.  The php.ini
     *	file will determine the timeout period for a session.
     *	@return bool True if the user is logged in.
	*/
	function isLoggedIn () {
		
		if ($this->get ('last_session_id') == session_id ()) {
			return true;		
		}
		
		return false;
	}
	
	/**  Logs in a user and updates the dbase to reflect the new state
     *	@param string $password The clear text password entered by the user.
     *	@return bool True if the login was successful.
	*/
	function login ($password) 
	{
				
		// validate the password first.
		if (md5 ($password) == $this->get (PROP_PASSWORD)) 
		{		
			// Make sure that the user's status is active 
			if ($this->get (PROP_STATUS) == USER_STATUS_ACTIVE) 
			{
				// Set session and activity related items.
				$this->set(PROP_LAST_SESSIONID, session_id ());
				$this->set(PROP_DATE_LAST_ACTIVITY, time());
				$this->set(PROP_LASTLOGIN, time());
				$this->set(PROP_LOGIN_IP, $_SERVER['REMOTE_ADDR']);
				$this->set(PROP_LOGIN_CLIENT_INFO, $_SERVER['HTTP_USER_AGENT']);
				
				// Update the database with the new info
				if ($this->update ()) 
				{			
					return true;
				}
				else 
				{
					$this->addError (str('STR_USER_LOGIN_UPDATE_FAILED'));
				}				
			}				
			else {
				switch ($this->get (PROP_STATUS)) {
				case USER_STATUS_UNCONFIRMED:
					$this->addError (str('STR_USER_NOTCONFIRMED'), ERROR_TYPE_SERIOUS);
					break;
				case USER_STATUS_FROZEN:
					$this->addError (str('STR_USER_FROZEN'), ERROR_TYPE_SERIOUS);
					break;
				case USER_STATUS_STALE:
					$this->addError (str('STR_USER_STALE'), ERROR_TYPE_SERIOUS);
					break;
				case USER_STATUS_DELETED:
					$this->addError (str('STR_USER_DELETED'), ERROR_TYPE_SERIOUS);
					break;
				}
			}
		}
		else {
		
			$this->addError (str('STR_USER_INVALID_CRED'), ERROR_TYPE_SERIOUS);
			
			// Security measure.
			sleep (3);
		}
		
		return false;
	}
	
	/**  Encrypts a password using MD5 encryption.
     *	@param string $password The unencrypted password.
     *	@return string The encrypted password
	*/
	function _encryptPassword ($password) {
	
		if ($password != '') 
		{
			return md5 ($password);
		}
		
		return '';
	}
	
	/**  Returns the unique ID for this object as stored in memory (NOT THE DATABASE)
		@param bool $fieldName If true, then this function will return the table's field name for the unique ID instead of the actual unique ID
		@return mixed The unique ID field for this object.  If an array is returned, then more than one field makes up a key.
	*/
	function getUniqueID ($fieldName=false) {
	
		if ($fieldName) {
			return $GLOBALS[$this->getTableConstant()]['fields']['USERNAME'];
		}
		else {
			return '\''.$this->get (PROP_USERNAME).'\'';
		}
	}
	
	/**
	 * Handles submission of story form data
	 * This will take care of retrieving the POST parameters,
	 *	then using the data, if valid, to add to or change
	 *	the databse.
	 * 
	 * @param bool $editForm True if the data being submitted
	 *					is to be used for editing the database, false
	 *					indicates that it's for adding.
	 * @return bool True if the submission was successful, false otherwise.
	 **/
	function handleFormSubmit ($editForm) {
	
		// Make sure that if this is an edit form then
		//	the id field has a valid ID number. The update
		//	will verify that it's in the database.
		if (!$editForm || ($editForm && $this->get (PROP_USERNAME))) {		
				
			// Now set the new values
			
			// First and last name are not required so they can 
			//	be empt6y.
			$this->set (PROP_FIRST_NAME, $this->queryPostValue ('first_name'));
			$this->set (PROP_LAST_NAME, $this->queryPostValue ('last_name'));
			$this->set (PROP_EMAIL, $this->queryPostValue ('email'));				
			$this->set (PROP_USERNAME, $this->queryPostValue ('username'));
			$this->set (PROP_STATE, $this->queryPostValue ('state'));				
			$this->set (PROP_CITY, $this->queryPostValue ('city'));
			$this->set ('email_notify_new_story', $this->queryPostValue ('notify_story') == '' ? false : true);
			$this->set ('email_notify_new_scene_fork', $this->queryPostValue ('notify_scene_fork') == '' ? false : true);				
			$this->set ('email_notify_updates', $this->queryPostValue ('notify_updates') == '' ? false : true);
			$GLOBALS['APP']->rememberValue ('REGISTRATION_USER_OBJ', $this);
			
			// Get the username if this is a registration form.
			if (!$editForm) {
				if (!$this->_checkUsernameFormat()) {
					// invalid username
					return false;
				}
			}
			
			// Validate and set email address
			if (!$this->_checkEmailFormat($this->get (PROP_EMAIL))) {
				return false;
			}			
			
			// Check if the user wanted to enter new passwords.
			//	If they did, then confirm that the passwords
			//	are valid and that they match each other.
			$pass1 = $this->queryPostValue ('password1');
			if ($pass1 != '') {
			
				// Now get the second password and compare
				//	to the first one.	
			    $pass2 = $this->queryPostValue ('password2');
				if ($pass1 == $pass2) {
				 	
					// Validate the password.					
					$this->set ('password', $pass1);
					if ($this->_checkPasswordFormat ()) {
					    
						// Encrypt it.
						$this->set ('password', $this->_encryptPassword ($pass1));
					}
					else {
					
						// There was a problem with the password
						//	format so use the error that was set by
						//	the checker function and 
						return false;
					}
				}
				else {
				 	$this->addError (str('STR_USER_PASS_NOMATCH'), ERROR_TYPE_SERIOUS);
					return false;
				}
			}
			else if (!$editForm) {
				$this->addError (str('STR_USER_NOPASS'), ERROR_TYPE_SERIOUS);
				return false;
		    }
			 			 
			if (!$editForm) { 
				return $this->newRegistration();
			}
			else {
				return $this->update ();
			}
		}
		
		return false;
	}

	/** Initializes smarty variables to display this object
     *	Override for custom object.
	*/
	function toArray() 
	{		
		$array = array();
		$array [PROP_FIRST_NAME] = $this->get (PROP_FIRST_NAME);
		$array [PROP_LAST_NAME] = $this->get (PROP_LAST_NAME);
		$array [PROP_EMAIL] = $this->get (PROP_EMAIL);
		$array [PROP_USERNAME] = $this->get (PROP_USERNAME);
		$array [PROP_STATE] = $this->get (PROP_STATE);
		$array [PROP_CITY] = $this->get (PROP_CITY);
		$array ['logged_in'] = $this->isLoggedIn ();
		$array ['user_type'] = $this->get ('user_type');

		return $array;
	}	

	/**
	 * Puts together a user's actual name.  If first is not available 
	 * or last is not available or both or not available then it will
	 * put together a name with whatever it has (it will always return
	 * the username if everything else fails).
	 * 
	 * @return string The full name of the user.
	 */
	function getFullName ()
	{
		$first = $this->get (PROP_FIRST_NAME);
		$last = $this->get (PROP_LAST_NAME);
		$username = $this->get ('username');
		
		if ($last != "")
		{
			$full = $last;
			
			if ($first != '')
			{
				$full .= ', '.$first;
			}
			
			return $full;
		}
		else if ($first != '')
		{
			$full = $first.'('.$username.')';
			return $full;
		}
		else 
		{
			return $username;
		}		
	}
	
	
	/**
	 * Adds smarty variables to the given smarty object to prepare 
	 *  the scene form for display. 
	 * 
	 * @param Smarty $smarty The smarty object to get the details of the form
	 * @param bool $editForm Whether or not we're constructing an edit form or a creation form.
	 **/
	function prepareFormTemplate (&$smarty, $editForm) {

		// Prepare the action information and whether or not
		//	to display the
		$formProperties = array ();
		$formProperties['edit'] = $editForm;
		$smarty->assign ('iw_form', $formProperties);
		$smarty->assign ('iw_user', $this->toArray());
	}	

	/**
	 * The basic user object doesn't use any icon.  Users of this class can
	 * 	override to return a custom icon.
	 */
	public function getItemIcon ($large=true)
	{
		return '';
	}	

}


?>
