<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFileTypeArchive extends shFileType
{
    protected $mimeTypeExtensionMap = array
        ( 
            'zip'=>array('multipart/x-zip','application/zip'),            
            'gz'=>array('application/x-gzip','multipart/x-gzip'),
            'tgz'=>array('application/x-gtar','application/x-tar','application/x-ustar','application/x-gtar','application/x-tar','application/x-ustar'),
            '7z'=>'application/x-7z-compressed',
            'rar'=>'application/x-rar-compressed',
            'bz2'=>array('application/bzip2','application/bzip'),
            'sit'=>'application/x-stuffit'                
        );      
       
    /**
    * @desc Returns an array of mime types that apply to the given extension
    * @param string $ext the extension to compare against
    * @return array The array of possible mime types.
    */
    public function extensionToMimeType($ext)
    {
       if (isset($this->mimeTypeExtensionMap[$ext]))
       {
           $mimeTypes = $this->mimeTypeExtensionMap[$ext];
           if (!is_array($mimeTypes))
           {
               return array($mimeTypes);
           }
       }
       
       return array();
    }
     
	public function getThumbnailPath(shItemDocument $doc, $sizeWidth, $sizeHeight)
	{           
		if ($sizeWidth >= 32)
		{       
			$icon = app()->getTemplateFile('images/icons/archive_icon_large.gif');
		}       
		else    
		{       
			$icon = app()->getTemplateFile('images/icons/archive_icon_small.gif');
		}       
		return $icon;
	}
	
	public function getPreview(shItemDocument $doc, $localPath)
	{
		// Nothing special needs to be done to display this to the user in the
		//	browser so return an empty string.
		return '';		
	}
	
	public function getMimeType($documentPath)
	{
        if ($documentPath)
        {
            $ext = '';
            list($ext) = pathinfo($documentPath,PATHINFO_EXTENSION);
            $mimeTypes = $this->extensionToMimeType($ext);
            return $mimeTypes;
        }
        else
        {        
            $types = array();
            foreach($this->mimeTypeExtensionMap as $ext=>$t)
            {
                if (is_array($t))
                {
                    $types = array_merge($types,$t);
                }
                else
                {
                    $types[] = $t;
                }                
            }
            
            return array_unique($types);

        }
	}
	
	public function getDescription($documentPath='')
	{
        return 'Archive File';
	}
	
	public function getRelatedUrl()
	{
		return '';
	}	
	
	public function getFilenameExtensions()
	{
		return array ('zip','rar','arj','tgz','bz2','sit','z','7z','tar','gz');
	}
	
	public function getNexusFileType()
	{
		return DOCTYPE_ARCHIVE;
	}
}

?>