<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormCategory extends iwForm
{
	public function getFormAction()
	{
		return URLACTION_MODCAT;	
	}
		
	function getFormType()
	{
		return FORMTYPE_CATEGORY;	
	}
	
	public function getTitle()
	{
		$id = $this->getUrlIdField();
        $action = g(URLARG_ACTION);
		if ($id == '' || ($action != URLACTION_MODCAT))
		{
			return str('STR_CAT_ADD_FORM_TITLE');
		} 
		else
		{
			return str('STR_CAT_EDIT_FORM_TITLE');
		}
	}
			
	public function getFields()
	{
		$fields = array();
		
		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->id,PROP_ID);
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->name,PROP_NAME,str('STR_CAT_NAME'));
		$fields[] = new shFormField(FIELD_TYPE_TEXTAREA,$this->getData()->description,PROP_DESCRIPTION,str('STR_CAT_DESCRIPTION'));
		$fields[] = new shFormField(FIELD_TYPE_CATEGORYSELECTOR,$this->getData()->parent_id,PROP_PARENT,str('STR_CAT_PARENT'));

		// Only show domain selector in non-async mode since the point
		//	of the async mode is to create a category quickly.
		if (!$this->getAsync())
		{			
			$field = new shFormField(FIELD_TYPE_DOMAINSELECTOR,$this->getData(),'domains',str('STR_CAT_DOMAINSWITHACCESS'));
			$field->fieldOptions['form'] = $this->getFormName();
			$fields[] = $field;			
		}
		else
		{
			$field = new shFormField(FIELD_TYPE_SINGLE_DOMAINSELECTOR,array(),'domains',str('STR_CAT_DOMAINSWITHACCESS'));
			$fields[] = $field;
		}
							
		return $fields;	
	}
	
	public function getRedirectLink()
	{
		if (($this->getData() instanceof shCategory) && 
			($this->getData()->id != ''))
		{
			return '?'.URLARG_ACTION.'='.URLACTION_VIEWLIB.
					'&'.URLARG_ID.'='.$this->getData()->id;
		}
		
		return '';		
	}
	
	/**
	 * If the ID parameter is in the url then that group
	 * is loaded and stored in the data field of the class
	 * otherwise a new object is created and set.
	 * @return bool Returns true unless an invalid id is given
	 * 				in the url id parameter. 
	 */
	protected function loadData()
	{
		// Check the cache first.	
		if ($this->recallFormData())
		{
			return true;
		}
		
        $action = g(URLARG_ACTION);
                
		$id = $this->getUrlIdField();
		if (($id != '') && ($id != 0) && ($action == URLACTION_MODCAT))
		{
			$cat = new shCategory();
			if ($cat->loadId($id))
			{
				$this->setData($cat);
				return true;
			}
			else
			{
				$this->setData(null);
				return false;
			}
		}
		
		// adding data
		$this->setData(new shCategory);
		return true;
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{		
		$category = new shCategory;
		$category->set(PROP_NAME,ps(PROP_NAME));
		$category->set(PROP_DESCRIPTION,ps(PROP_DESCRIPTION));
		$category->set(PROP_STATUS,CATEGORY_STATUS_ACTIVE);
		
		$parentId = ps(PROP_PARENT);
		if (is_array($parentId))
		{
			$parentId = $parentId[0];
		}
		
		$category->set(PROP_PARENT,$parentId);
		
		// Get the selected Companies/Groups that have access to this object.
		$selectedCG = isset($_POST['domains'])?$_POST['domains']:array();
		if (!is_array($selectedCG))
		{
			if ($selectedCG != '')
			{
				$selectedCG = array ($selectedCG);
			}
			else
			{
				$selectedCG = array (COMPANY_ALL.','.GROUP_ALL);
			}
		}
		else if (count($selectedCG) == 0)
		{
			$selectedCG = array (COMPANY_ALL.','.GROUP_ALL);
		}
		
		// Replace the current set with what we have here.
		$category->clearCompanyGroupPairs();
		foreach ($selectedCG as $cg)
		{
			list($company,$group) = split(',',$cg);
			$category->addCompanyGroup ($company,$group);			
		}		
		
		$this->setData($category);
		$this->rememberFormData($category);
		
		if ($category->requiredFieldsValid())
		{
            $results = array();
            
            $action = g(URLARG_ACTION);
			if ($this->isEditingObject() && ($action == URLACTION_MODCAT))
			{
                $continue = true;
                $results = nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$category,'continue'=>&$continue));
                if ($continue)
                {                                                       
				    $category->set(PROP_ID,ps(PROP_ID));
				    if ($category->update ())
				    {
					    $this->_results = $category;
					    
					    $this->forgetFormData ();
					    $this->addNotification (str('STR_CAT_UPDATED'));
                        
                        nx_fire_event('EVENT_OBJECT_EDITED',array('object'=>$category));
                        
					    return true;
				    }
                }
			}
			else
			{
                $continue = true;
                nx_fire_event('EVENT_OBJECT_ADDING',array('object'=>$category,'continue'=>&$continue));
                if ($continue)
                {                                                       
				    if ($category->add ())
				    {
					    $this->_results = $category;
					    
					    $this->forgetFormData ();
					    $this->addNotification (str('STR_CAT_ADDED'));
                        
                        nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$category));
                        
					    return true;
				    }
                }
			}
		}
		
		return false;		
	}

	function authorize ()
	{
		$id = g(URLARG_ID);
		if (!$id) 
		{
			$id = ps(PROP_ID);
		}
		
		$user = app()->getLoggedInUserObject();
        
        if ($user)
        {
            return $user->getPermissions()->isAllowed(URLACTION_MODCAT,$id);
        }
        else
        {
            return false;
        }
	}

	/**
	 * Returns the results of the form submission.  The results value is an
	 * an empty array if the form submit failed.  Otherwise, it contains an
	 * associative array:
	 * 
	 * 'name' => group name
	 * 'id' => group id
	 *
	 * @return array The array described above. 
	 */
	function getResults()
	{
		if ($this->_results)
		{
			return array('name'=>$this->_results->get(PROP_NAME), 
					 'id'=>$this->_results->get(PROP_ID));
		}
		
		return array();
	}	
}
?>
