<?php

define ('FIELD_TYPE_GROUPSELECTOR', 'groupsel');

// The fieldValue property for this is the either 
//	null or the category or item that is associated 
//	with the category.
define ('FIELD_TYPE_DOMAINSELECTOR', 'domainsel');
define ('FIELD_TYPE_SINGLE_DOMAINSELECTOR', 'singledomainsel');
define ('FIELD_TYPE_USERSELECTOR', 'usersel');
define ('FIELD_TYPE_CATEGORYSELECTOR', 'categorysel');
define ('FIELD_TYPE_MULTI_CATEGORYSELECTOR', 'multicategorysel');
define ('FIELD_TYPE_EDITABLEIMAGE', 'editableimage');
define ('FIELD_TYPE_ATTRIBUTESELECTOR', 'attribsel');

// The field value is a string containing a comma separated 
//	list of document Ids.
define ('FIELD_TYPE_DOCUMENTSELECTOR', 'docsel');

class shFormField extends iwFormField
{
	public function getHtml()
	{
		$val = '';
		switch ($this->fieldType)
		{
			case FIELD_TYPE_DOCUMENTSELECTOR:
			{
				$smarty = app()->getSmartyObject();
				$smarty->assign('fieldname',$this->fieldName);
				$smarty->assign('fieldid',$this->fieldId?$this->fieldId:$this->fieldName);
				$smarty->assign('document_ids',$this->fieldValue);
				$val = smarty_templ_fetch('controls/control.selector.document.tpl.html',$smarty);					
				break;
			}
			case FIELD_TYPE_SINGLE_DOMAINSELECTOR:
			{                
				$smarty = app()->getSmartyObject();                
				$smarty->assign('sourceList',$this->fieldName);
				$smarty->assign('selected',$this->fieldValue);                
                                
                $domainList = $this->getDomainList();                                
				$smarty->assign('list_data',$domainList);                
				$val = smarty_templ_fetch('controls/control.selector.domain.tpl.html',$smarty);
					
				break;
			}
			case FIELD_TYPE_MULTI_CATEGORYSELECTOR:
			{
				$smarty = app()->getSmartyObject();
				$smarty->assign('sourceList','source_cat_list');
				$smarty->assign('destinationList',$this->fieldName);
				$smarty->assign('selected',$this->fieldValue);
				$smarty->assign('list_data',$this->getCategoryList());
				$smarty->assign('formName',$this->fieldOptions['form']);
				$val = smarty_templ_fetch('controls/control.selector.multiitem.tpl.html',$smarty);
				break;				
			}
			
			case FIELD_TYPE_DOMAINSELECTOR:
			{                
				$smarty = app()->getSmartyObject();
                
                $obj = null;
                if ($this->fieldValue instanceof shCategory ||
                    $this->fieldValue instanceof shItemDocument)
                {
                    $obj = $this->fieldValue;        
                }                
                
				$aggregates = isset($this->fieldOptions['noaggregates'])?$this->fieldOptions['noaggregates']:false;
				$domains = shCompanyGroupPair::getHierarchicalDomainList($obj,$aggregates);				
				$smarty->assign('domains',$domains);
                                
				$smarty->assign('formName',$this->parentForm->getFormName());
                $smarty->assign('formId',$this->parentForm->getFormId());
                $smarty->assign('fieldName',$this->fieldName);
				$val = smarty_templ_fetch('controls/control.selector.domaintree.tpl.html',$smarty);
				break;				
			}

			case FIELD_TYPE_USERSELECTOR:
			{
				$smarty = app()->getSmartyObject();
				$smarty->assign('sourceList',$this->fieldName);
				$smarty->assign('list_data',$this->getUserList());
				$val = smarty_templ_fetch('controls/control.selector.user.tpl.html',$smarty);				
				break;
			}
			case FIELD_TYPE_CATEGORYSELECTOR:
			{
				$cats = $this->getCategoryList();
				foreach ($cats as $cat)
				{
					$this->fieldOptions[$cat['id']] = $cat['name'];
				}
				
				$val = $this->assembleSelect(false);
				break;
			}
			
			case FIELD_TYPE_ATTRIBUTESELECTOR:
			{
				$smarty = app()->getSmartyObject();
				$smarty->assign('fieldname',$this->fieldName);
				$smarty->assign('fieldid',$this->fieldId);
				$smarty->assign('fieldvalue',$this->constructAttributeListFromValue());
				$val = smarty_templ_fetch('controls/control.selector.tag.tpl.html',$smarty);
				break;	
			}
			case FIELD_TYPE_GROUPSELECTOR:
			{
				$smarty = app()->getSmartyObject();
				$smarty->assign('sourceList','source_list');
				$smarty->assign('destinationList',$this->fieldName);
				$smarty->assign('list_data',$this->getGroupList());
				$smarty->assign('formName',$this->fieldOptions['form']);
				$val = smarty_templ_fetch('controls/control.selector.group.tpl.html',$smarty);
				break;				
			}

			case FIELD_TYPE_EDITABLEIMAGE:
			{
				$smarty = app()->getSmartyObject();
				$smarty->assign('logo',$this->fieldValue);
				$smarty->assign('fieldName',$this->fieldName);
				$val = smarty_templ_fetch('controls/control.view.image.tpl.html',$smarty);
				break;				
			}
			default:
				$val = parent::getHtml();
				break;
		}
		
		return $val;
	}
	
	/**
	 * Puts together an array of assoc array that contain a list of 
	 * all the items that belong in the group control.  The form of 
	 * the array is as follows:
	 *  name: the name that appears in the control
	 *  value: the value of a single item
	 *  selected: whether or not to mark it as selected.
	 *
	 * @return array The aforementioned array.
	 */
	public function getGroupList()
	{
		// The 'options' parameter contains the list of group IDs
		//	that should be selected by default.
		
		if (!is_array($this->fieldValue)) 
		{
			$this->fieldValue = array();
		}
		
		$groupsForField = array();
		$groups = new shGroupCollection;
		$params = new iwParameterCollection();
		$params->add (new iwParameter(FILTER,false));
		$groups->load ($params);
		foreach ($groups->items() as $group)
		{
			$currentGroup = $group->get(PROP_NAME);
			$currentGroupId = $group->get(PROP_ID);
						
			$selected = false;
			foreach ($this->fieldValue as $groupInCompany)
			{				
				if ($groupInCompany == $currentGroupId)
				{
					$selected = true;
					break;
				}
			}
			
			$groupsForField[] = array ('name'=>$currentGroup,
									   'value'=>$currentGroupId,
									   'selected'=>$selected);
		}	

		return $groupsForField;
	}
		
	/**
	 * Puts together an array of assoc array that contain a list of 
	 * all the items that belong in the domain control.  The form of 
	 * the array is as follows:
	 *  name: the name that appears in the control
	 *  value: the value of a single item
	 *  selected: whether or not to mark it as selected.
	 *
	 * @return array The aforementioned array.
	 */	
	public function getDomainList()
	{
		$permission = app()->getLoggedInUserObject()->getPermissions();
			
		$domainRestriction = DOMAIN_NONE;
		$allowEveryone = false;
		$permissionGroup = $permission->getPermissionGroup();
		
		if ($permissionGroup == USER_TYPE_ADMIN)
		{
			$domainRestriction = DOMAIN_GLOBAL;
			$allowEveryone = true;
		}
		else if ($permissionGroup == USER_TYPE_DOMAIN_ADMIN)
		{			
			$domainRestriction = DOMAIN_COMPANY;
		}
		else if ($permissionGroup == USER_TYPE_DOMAIN_GROUP_ADMIN)
		{
			$domainRestriction = DOMAIN_GROUP;
		}		
        
		if (isset($this->fieldOptions['noaggregates']) && 
			$this->fieldOptions['noaggregates'])
		{
			$allowEveryone = false;	
		}
		
		$obj = null;
		if ($this->fieldValue instanceof shCategory ||
			$this->fieldValue instanceof shItemDocument)
		{
			$obj = $this->fieldValue;		
		}
		        
        
        $cgpd=$this->prepareDomainList($obj,$allowEveryone,$domainRestriction);
                
		return $cgpd;
	}	
	
	/**
	 * Retrieves the domain list specified by the parameters and coverts into into an array
	 * that can be used by the templating system for display.
	 * @return array An array of arrays that looks like this (name=>'CompanyName:GroupName',value=>'COMPANYID:GROUPID',selected=>[true/false])
	 * @param $item shItem[optional] An item or category that can be used to determine if there is a default selection
	 * @param $includeEveryoneChoice bool[optional] True to include the 'Everyone' or 'Public' option int he returned list.
	 * @param $domainRestriction int [optional] Indicates which domains should be returned (DOMAIN_GLOBAL,DOMAIN_ORG or DOMAIN_GROUP)
	 */
	protected function prepareDomainList($item = false, $includeEveryoneChoice=false, $domainRestriction=DOMAIN_GLOBAL)
	{
		$selectedByDefault = false;
		if ($item)
		{
			$selectedByDefault = $item->getDomains();
		} 
		
		$data = array();
		$domains = shCompanyGroupPair::getAccessibleDomains($includeEveryoneChoice,$domainRestriction);
		foreach ($domains as $domain)
		{
			$selected = false;
			if ($selectedByDefault)
			{
				foreach ($selectedByDefault as $selectedDomain)
				{
					$currentCompanyId = $domain->getGroup()->get(PROP_ID);
					$currentGroupId = $domain->getCompany()->get(PROP_ID);				
					
					if ($currentGroupId == $selectedDomain->getGroup()->get(PROP_ID) &&
						$currentCompanyId == $selectedDomain->getCompany()->get(PROP_ID))
					{
						$selected = true;
						break;
					}
				}
			}
			
			$data[] = array ('name'=>$domain->getCompany()->get(PROP_NAME).':'.$domain->getGroup()->get(PROP_NAME),
							 'value'=>$domain->getCompany()->get(PROP_ID).','.$domain->getGroup()->get(PROP_ID),
							 'selected'=>$selected);			
		}
		        
		return $data;
	}
	
	/**
	 * Puts together an array of assoc array that contain a list of 
	 * all the items that belong in the category control.  The form of 
	 * the array is as follows:
	 *  name: the name that appears in the control
	 *  id: the value of a single item
	 *  selected: whether or not to mark it as selected.
	 *
	 * @return array The aforementioned array.
	 */	
	public function getCategoryList()
	{	
		$selCategories = array();
		
		// Value is an id (use it as is)
		if (is_numeric($this->fieldValue))
		{
			$selCategories[] = $this->fieldValue;	
		}
		
		// Value is a document, get the ids of the parent categories
		else if ($this->fieldValue instanceof shItemDocument)
		{
			$cats = $this->fieldValue->getCategories();
			foreach ($cats as $cat)
			{
				$selCategories[] = $cat->id;	
			}			
		}
		
		// Value is a category, get its parent category
		else if ($this->fieldValue instanceof shCategory)
		{
			$selCategories[] = $this->fieldValue->parent_id;
		}
		
		// Get all the categories and mark whether or not each is selected
		//	There should only be one selection.
		
		$categories = app()->getLoggedInUserObject()->getAuthorizedCategories();

        // Now, if there are no selected categories and there is only
        //  one category in the list, then use that one.        
        if ($this->fieldValue == null && (count($categories) == 1))
        {
            $selCategories[] = $categories[0]->get(PROP_ID);
        }	
        	
		$categoriesArray = array();
		if ($categories)
		{
			$categoriesArray = array();
			foreach($categories as $cat)
			{
				$singleCatArray = $cat->toArray();
				$singleCatArray['value'] = $cat->get(PROP_ID);
				$categoriesArray[] = $singleCatArray;
			}
			
			for ($i=0;$i<count($categoriesArray);$i++)
			{
				$isSelected = false;
				foreach ($selCategories as $selCat)
				{
					if ($selCat == $categoriesArray[$i]['id'])
					{
						$isSelected = true;
					}
				}
				
				$categoriesArray[$i]['selected'] = $isSelected;
			}
		}		
		
		return $categoriesArray;		
	}
	
	/**
	 * Retrieves a list of all the users for which the active user
	 * has access 
	 * @return array Returns an array of associative arrays of user 
	 * 				data. 
	 */
	public function getUserList()
	{
		return nx_get_usernames();
	}
	
	/**
	 * Takes the field value as stored in the object
	 * and constructs a comma separated list of tags 
	 *
	 * @return string The comma separated list of tags.
	 */
	public function constructAttributeListFromValue()
	{
		$value = '';
		$obj = $this->fieldValue;
		if ($obj instanceof shItemDocument)
		{
			$strings = array();
			$attribs = $obj->getAttributes();
			foreach ($attribs as $attrib)
			{
				$strings[] = $attrib->name;		
			}
			
			$value = implode(',',$strings);
		}
		else
		{
			$value = $this->fieldValue;
		}
		
		return $value;
	}
}
?>