<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormMultiDocument extends iwForm
{
	var $_targetIds = array();
	
	public function getTitle()
	{
		return str('STR_MULTIDOC_EDIT_TITLE');
	}
	
	public function getFormAction()
	{
		return FORMACTION_EDITBIN;	
	}
		
	function getFormType()
	{
		return FORMTYPE_MULTIDOC;	
	}
		
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function getFields()
	{				
		$data = array ();
		$item = new shItemDocument;

		// Get related items
		$fields = array();		
		
		$ids = implode(',',$this->getTargetIds());
		
		// No target IDs, something's wrong.
		if (count($ids) == 0)
		{
			return array();	
		}
		
		$fields[] = new iwFormField(FIELD_TYPE_HIDDEN,$ids,'doc_ids');
		
		// Get the names of the documents for display to the user
		$names = array();		
		$query = 'select item_title from sh_items where item_id in ('.$ids.')';
        app()->flushCacheIfInvalid($query);
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		foreach ($results as $row)
		{
			$names[] = $row['item_title'];
		}
		$fields[] = new iwFormField(FIELD_TYPE_STATIC,implode(',',$names),'names',str('STR_MULTIDOC_TARGETS'));
		
		$item = $this->getData();
		if (!$item)
		{
			$item = new shItemDocument;
		}		
		
		$relatedItems = implode(',',$item->getRelatedItems());
		$fields[] = new shFormField(FIELD_TYPE_DOCUMENTSELECTOR,$relatedItems,'related_item_ids',str('STR_MULTIDOC_RELATED'));		
		
		$field = new shFormField(FIELD_TYPE_MULTI_CATEGORYSELECTOR,$item,'category_list',str('STR_MULTIDOC_CATS'));
		$field->fieldOptions['form'] = $this->getFormName();
		$fields[] = $field;
				
		$fields[] = new shFormField(FIELD_TYPE_ATTRIBUTESELECTOR,$item,'attribute_list',str('STR_MULTIDOC_TAGS'));
		 
		$field = new shFormField(FIELD_TYPE_DOMAINSELECTOR,$item,'domains',str('STR_MULTIDOCS_DOMAINS'));
		$field->fieldOptions['form'] = $this->getFormName();
		$fields[] = $field;			
		
		return $fields;	
	}
	
	function setTargetIds ($ids)
	{
		$this->_targetIds = $ids;
	}

	function getTargetIds ()
	{
		return $this->_targetIds;
	}
	
	function getRedirectLink()
	{
		return '';	
	}
	
	/**
	 * Displays a simple list of the documents that were edited 
	 * successfully.
	 *
	 * @return string The html for displaying the list of documents
	 */
	function getResultsHtml()
	{
		$collection = new shItemDocumentCollection();
		
		$documentIds = split(',',ps('doc_ids'));
		foreach($documentIds as $id)
		{
			$doc = new shItemDocument();
			if ($doc->loadId($id))
			{
				$collection->addItem($doc);
			}
		}
		
		$collection->setTitle('Documents Edited');
		$collection->setSubtitle('The following documents were edited successfully');
		$collection->setEnableSorting(false);
		$collection->setShowFilter(false);
		$collection->setAllowSelection(false);
		
		return $collection->html();
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{		
		$items = array();
		$ids = explode(',', ps('doc_ids'));
		
		foreach ($ids as $id)
		{
			$item = new shItemDocument;
			$item->set(PROP_ID,$id);
			if ($item->load())
			{
				$items[] = $item;
			}
		}		
		foreach ($items as $item)
		{						
			
			$item->set(PROP_STATUS,ITEM_STATUS_ACTIVE);
	
			// Get the selected Companies/Groups that have access to this object.
			$selectedCG = ps('domains');
			if (!is_array($selectedCG))
			{
				if ($selectedCG != '')
				{
					$selectedCG = array ($selectedCG);
				}
				else
				{
					$selectedCG = array ();
				}
			}

            echo count($selectedCG);
			if (count($selectedCG) > 0)
			{
				$item->clearCompanyGroupPairs();
				foreach ($selectedCG as $cg)
				{					
					list($company,$group) = split(',',$cg);
					$item->addCompanyGroup ($company,$group);			
				}
			}
			
			// Get the selected categories .			
			$selectedCats = ps('category_list');
			if (!is_array($selectedCats))
			{
				if ($selectedCats != '')
				{
					$selectedCats = array ($selectedCats);
				}
				else
				{
					$selectedCats = array ();
				}
			}
					
			if (count($selectedCats) > 0)
			{	
				// Replace the current set with what we have here.
				$item->clearCategories();
				foreach ($selectedCats as $cat)
				{
					$item->addCategory($cat);			
				}
			}
	
			// Get the selected attributes .
			$taglist = ps('attribute_list');
			if ($taglist != '')
			{				
				$item->clearAttributes();
				$selectedAttribs = explode(',',$taglist);			
				$actions = new shActions();			
				foreach($selectedAttribs as $tag)
				{
					if ($tag != '')
					{
						$tagOb = $actions->doAddTag($tag);
						if ($tagOb)
						{
							$item->addAttribute($tagOb);						
						}
					}
				}
			}						
				
			// Replace the current set of related items with the ones 
			//	submitted with the form (if any)
			$ritems = ps('related_item_ids');			
			if ($ritems != '')
			{
				$item->clearRelatedItems();			
				$ritems = split (',',$ritems);
				foreach ($ritems as $ri)
				{
					$ri = trim ($ri);
					if ($ri != '' && is_numeric ($ri))
					{
						$item->addRelatedItem ($ri);
					}
				}
			}
					
			if ($item->requiredFieldsValid())
			{
				$item->update ();
			}
			else
			{
				$this->setData($item);
				$this->rememberFormData();				
				return false;
			}
		}
		
		$this->addNotification (str('STR_MULTIDOC_UPDATEDALL'));	

		return true;
	}

	/**
	 * Determines if the current user is allowed to edit all the 
	 * documents in the current submission.
	 */
	public function authorize()
	{
		$docids = array();
		if ($this->isSubmittingForm())
		{
			$docids = split(',',ps('doc_ids'));
		}
		else
		{
			$docids = $this->getTargetIds();
		}
		
		foreach ($docids as $id)
		{
			$canEdit = shAuthorize::canEditDocument(app()->getLoggedInUsername(),$id);
			if (!$canEdit)
			{
				return false;
			}
		}
		
		return true;
	}
}
?>
