<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFormPoolItem extends shFormDocument
{
	public function getFormAction()
	{
		return URLACTION_MOD_POOL;	
	}
		
	public function getFormType()
	{
		return FORMTYPE_POOL_ITEM;	
	}

	public function getFormTemplate()
	{
		return iwForm::getFormTemplate();	
	}
	
	public function getTitle()
	{
		$id = $this->getUrlIdField();
		if ($id == '')
		{
			return str('STR_FILEBOX_ADD_FORM_TITLE');
		} 
		else
		{
			return str('STR_FILEBOX_EDIT_FORM_TITLE');
		}		
	}
	
	protected function getContentArea(&$smarty)
	{
        if (g(URL_MULTIUPLOAD) != '')
        {
            return $this->handleMultiUpload();
        }
                
		// Skip base document form class implementation of renderer.
		return iwForm::getContentArea($smarty);	
	}
	
	/**
	 * If the ID parameter is in the url then that group
	 * is loaded and stored in the data field of the class
	 * otherwise a new object is created and set.
	 * @return bool Returns true unless an invalid id is given
	 * 				in the url id parameter. 
	 */
	protected function loadData()
	{
		// Check the cache first.	
		if ($this->recallFormData())
		{
			return true;
		}
		
		$id = $this->getUrlIdField();
		if ($id != '')
		{
			$obj = new shPoolItem();
			if ($obj->loadId($id))
			{
				$this->setData($obj);
				return true;
			}
			else
			{
				$this->setData(null);
				return false;
			}
		}
		
		// adding data
		$this->setData(new shPoolItem);
		return true;
	}
	
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function getFields()
	{

		$fields = array();
		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->id,PROP_ID,'');
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->name,PROP_NAME,str('STR_FILEBOX_NAME'));
		$fields[] = new shFormField(FIELD_TYPE_TEXTAREA,$this->getData()->name,PROP_DESCRIPTION,str('STR_FILEBOX_DESCRIPTION'));
		$fields[] = new shFormField(FIELD_TYPE_UPLOADFILE,'','document',str('STR_FILEBOX_UPLOAD'));
		return $fields;
	}
	
	public function getRedirectLink()
	{
		if (($this->getData() instanceof shPoolItem) && 
			($this->getData()->id != ''))
		{
			return '?'.URLARG_ACTION.'='.URLACTION_VIEWPOOL.
					'&'.URLARG_ID.'='.$this->getData()->id;
		}
		
		return '';		
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit()
	{
				
		$item = new shPoolItem;	
		$user = app()->getLoggedInUserObject();
		
		if ($this->isEditingObject())
		{
			$item->set(PROP_ID,ps(PROP_ID));
			if (!$item->load())
			{
				$this->addError (str('STR_FILEBOX_NOTFOUND'), ERROR_TYPE_SERIOUS);
				return false;
			}			
		}					
		$item->set(PROP_NAME,ps(PROP_NAME));
		$item->set(PROP_DESCRIPTION,ps(PROP_DESCRIPTION));
		$item->set(PROP_STATUS,ITEM_STATUS_ACTIVE);
		$username = $user->get(PROP_USERNAME);
		$item->set(PROP_USERNAME, $username);

		
		$this->setData($item);
		$this->rememberFormData();
				
		$contentUpdated = false; 
		if (isset($_FILES) && is_array($_FILES['document']) && $_FILES['document']['name']) 
		{				
			// Only set if there's something new to upload
			$name = $_FILES['document']['name'];
			$type = typeman()->determineNexusType($name, $_FILES['document']['type']);
		
			if ($type == DOCTYPE_SHARE)
			{
				// This will take care of adding items (one for each file in zip file.
				$uploadsDir = app()->get('upload_dir');
				$tempPath = $uploadsDir.'/'.basename($name,'.share').'.zip';
				if (move_uploaded_file($_FILES['document']['tmp_name'],$tempPath))
				{
					return $this->processMultiAdd ($item,$tempPath);
				}
				else
				{
					return false;
				}
			}
			if ($type != DOCTYPE_UNKNOWN)
			{
				$uploadsDir = app()->get('upload_dir');
				$tempPath = $uploadsDir.'/'.$name;
				
				$maxSizeInMb = $user->get(PROP_ITEMPOOL_SIZE);
				
				// Check for room on disk for this document (if max is 0 that means
				//	filebox is unlimited
				if ($maxSizeInMb > 0)
				{
					$size = $_FILES['document']['size'];				
					if (!shFileAnalysis::isFileBoxSpaceAvailable($size, $maxSizeInMb))
					{
					    $this->addError (str('STR_FILEBOX_NOSPACE'), ERROR_TYPE_SERIOUS);
						return false;
					}				
				}
				
				if (!move_uploaded_file($_FILES['document']['tmp_name'],$tempPath))
				{
					// Failed to copy into good location
					return false;
				}
			
				// Determine the document type based on the extension or the mime type.								
				$item->loadDocument($type,$_FILES['document']['name'],$tempPath);
												
				$contentUpdated = true;
			}
			else
			{
				$this->addError (str('STR_DOC_UNSUP_TYPE'), ERROR_TYPE_SERIOUS);
				return false;
			}
		}
		else
		{
			$this->addError (str('STR_DOC_NOFILE'), ERROR_TYPE_SERIOUS);
			return false;
		}

		$this->rememberFormData($item);
			
		$item->get(PROP_ID);
		if ($item->requiredFieldsValid())
		{
			if ($this->isEditingObject())
			{
                $continue = true;
                nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$item,'continue'=>&$continue));
                if ($continue)
                {                                                                       
				    if ($item->update())
				    {
					    if ($contentUpdated)
					    {
						    $item->storeFileToFinalLocation();	
					    }
					    
					    $this->forgetFormData();			
					    $this->addNotification (str('STR_FILEBOX_UPDATED'));
                        
                        nx_fire_event('EVENT_OBJECT_EDITED',array('object'=>$item));
                        
                        
					    return true;
				    }
                }
			}				
			else
			{
                $continue = true;
                nx_fire_event('EVENT_OBJECT_ADDING',array('object'=>$item,'continue'=>&$continue));
                if ($continue)
                {                                                                       
				    if ($item->add ())
				    {
					    // This will rename the file and delete the original
					    $item->storeFileToFinalLocation ();
				    
					    $this->forgetFormData();
					    $this->addNotification (str('STR_FILEBOX_ADDED'));
                        
                        nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$item));
                        
					    return true;
				    }
                }
			}
		}
		
		return false;		
	}	

	function authorize ()
	{
		$id = g(URLARG_ID);
		if (!$id) 
		{
			$id = ps(PROP_ID);
		}

		$user = app()->getLoggedInUserObject();
		return $user->getPermissions()->isAllowed(URLACTION_MOD_POOL,$id);
	}
	
    /**
     * Handles a request to upload multiple uploads.  
     * @return string The html for multi upload
     */
    function handleMultiUpload()
    {
        $smarty = app()->getSmartyObject();
               
        $uploader = new shUploaderJUpload();
        $uploader->setUploadLocation(DEST_UPLOAD_FILEBOX);
        $smarty->assign('multiupload_code', $uploader->getMarkup());        
        return smarty_templ_fetch('forms/form.add.filebox.document.multiple.html',$smarty);                
    }    
    
}
?>
