<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


class shFormRegisterUser extends iwForm
{
	public function getFormAction()
	{
		return URLACTION_REGISTER;	
	}
		
	function getTitle()
	{
		return str('STR_USER_REGISTER_FORM_TITLE');	
	}
	
	function getFormType()
	{
		return FORMTYPE_ADDUSER;	
	}

	protected function loadData()
	{
		if ($this->recallFormData())
		{
			return true;		
		}
		
		$this->setData(new shUser);
		return true;
	}
	
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function getFields()
	{			
		$company = new shCompany;
		$company->id = settings()->self_reg_company;
		$companyLoaded = $company->load();
		
		$group = new shGroup;
		$group->id = settings()->self_reg_group;
		$groupLoaded = $group->load();
		
		if (!$companyLoaded || !$groupLoaded)
		{
			$this->addError(sprintf(str('USER_SELFREG_NOT_WORKING'),'mailto:'.settings()->admin_email),ERROR_TYPE_SERIOUS);
			return array();	
		}
		
		$fields[] = new shFormField (FIELD_TYPE_STATIC,"$company->name/$group->name",'domain',str('STR_USER_DOMAIN'));
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->username,PROP_USERNAME, str('STR_USER_USERNAME'));
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->first_name,PROP_FIRST_NAME, str('STR_USER_FIRSTNAME'));
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->last_name,PROP_LAST_NAME,str('STR_USER_LASTNAME'));
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->email,PROP_EMAIL,str('STR_USER_EMAIL'));
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->city,PROP_CITY,str('STR_USER_CITY'));
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->state,PROP_STATE,str('STR_USER_STATE'));
		$fields[] = new shFormField(FIELD_TYPE_PASSWORD,'',PROP_PASSWORD,str('STR_USER_NEWPASS'));
		$fields[] = new shFormField(FIELD_TYPE_PASSWORD,'','password_confirm',str('STR_USER_CONFIRM'));		
		
		return $fields;		
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit ()
	{				
		$user = new shUser;		
		$user->set(PROP_USERNAME,ps('username'));
		
		$password = ps('password');
		$passwordConfirm = ps('password_confirm');

		if ($password == '')
		{
			$this->addError (str('STR_USER_MUSTENTERPASSWORD'), ERROR_TYPE_SERIOUS);
			return false;
		}
		
		if (($password != $passwordConfirm))
		{
			$this->addError (str('STR_USER_PASSWORDMISMATCH'), ERROR_TYPE_SERIOUS);
			return false;
		}
						
		$user->set(PROP_EMAIL, ps('email'));
		$user->set(PROP_FIRST_NAME, ps('first_name'));
		$user->set(PROP_LAST_NAME, ps('last_name'));
		$user->set(PROP_CITY, ps('city'));
		$user->set(PROP_STATE, ps('state'));
		$user->set(PROP_PASSWORD,md5($password));
		
		$group = new shGroup;
		$group->set(PROP_ID, (int)settings()->self_reg_group);		
		$company = new shCompany;
		$company->set(PROP_ID, settings()->self_reg_company);		
        
		$user->setDomain($company,$group);
		
		$this->setData($user);
		$this->rememberFormData();
		
		if ($user->requiredFieldsValid())
		{
			$user->set('require_new_user_confirm', true);
			if ($user->newRegistration ())
			{
				return true;
			}
		}
				
		return false;	
	}

	/**
	 * Only allow this if the admin has turned on self-registration
	 * and no one is currently logged in.
	 *
	 * @return bool Return true if authorized.
	 */
	function authorize ()
	{
		return (settings()->allow_self_reg && (!app()->getLoggedInUserObject()));
	}	
}
?>