<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


class shFormSendGroupEmail extends iwForm
{
	public function getFormAction()
	{
		return URLACTION_SENDMAIL_TO_GROUP;	
	}
			
	function getFormType()
	{
		return FORMTYPE_SEND_GROUP_MAIL;	
	}
		
	function getTitle()
	{
		return str('STR_MAIL_TITLE_GROUP_MAIL');
	}

    public function getRedirectLink()
    {
        return 'index.php';
    }

	public function loadData()
	{
		if ($this->recallFormData())
		{
			return true;	
		}
		
		$data = array
			(
				'subject'=>'',
				'body'=>'',
				'com_group_list'=>'1,1'
			);
			
		$this->setData($data);
		
		return true;
	}
	
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	public function getFields()
	{
		$data = $this->getData();
		$fields = array();
		$fields[] = new shFormField(FIELD_TYPE_TEXT,$data['subject'],'subject',str('STR_EMAIL_SUBJECT'));
		$fields[] = new shFormField(FIELD_TYPE_TEXTAREA,$data['body'],'body',str('STR_EMAIL_BODY'));
		$fields[] = new shFormField(FIELD_TYPE_SINGLE_DOMAINSELECTOR,$data['com_group_list'],'com_group_list',str('STR_EMAIL_TARGETGRP'));
				
		return $fields;	
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	public function _submit()
	{		
		// Get companies/groups
		$selectedCG = ps('com_group_list');
		if (!is_array($selectedCG))
		{
			if ($selectedCG != '')
			{
				$selectedCG = array ($selectedCG);
			}				
			else
			{
				$this->addError (src('SRC_MAIL_NOGROUPSPEC'), ERROR_TYPE_SERIOUS);
				return false;
			}
		}

		$subject = ps('subject');
		$body = ps('body');
		
		if ($subject == '')
		{
			$this->addError (str('STR_MAIL_NOSUBJECTSPEC'), ERROR_TYPE_SERIOUS);
			return false;			
		}
		
		if ($body == '')
		{
			$this->addError (str('STR_MAIL_NOBODYSPEC'), ERROR_TYPE_SERIOUS);
			return false;
		}
		
		$notify = new shUserNotification;
		
		foreach ($selectedCG as $cg)
		{
			list($companyId,$groupId) = split(',',$cg);

			if ($companyId > 0 && $groupId > 0)
			{
				$notify->sendMailToCompanyGroup($companyId, $groupId, $subject, $body);
			}						
		}		
				
		return true;		
	}

	public function authorize ()
	{
		$user = app()->getLoggedInUserObject();
		return $user->getPermissions()->isAllowed(URLACTION_SENDMAIL_TO_GROUP,0);
	}		
	
}
?>
