<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


class shFormUsageReport extends iwForm
{
	protected $_resultHtml;
	
	public function getFormAction()
	{
		return URLACTION_VIEWUSAGE;	
	}		
		
	function getFormType()
	{
		return FORMTYPE_USAGE_REPORT;	
	}
		
	function getTitle()
	{
		return str('STR_USAGE_FORM_TITLE');	
	}
	
	public function getRedirectLink()
	{
		// We don't redirect upon submission, we show
		//	data instead.
		return '';	
	}
	
	/**
	 * The _resultHtml field should contain the html
	 * for the report. 
	 */
	public function getResultsHtml()
	{
		return $this->_resultHtml;		
	}
	
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function getContentArea()
	{
				
		$data = array ();		
		
		// First check if there's a cached object
		$report = $this->recallFormData();
		if (!$report || !is_a ($report,'shUsageReport'))
		{
			$report = new shUsageReport;
		}
				
		if ($report)
		{
			$data['start_date'] = $report->get(PROP_START_DATE);
			$data['end_date'] = $report->get(PROP_END_DATE);		
			$data['group_by'] = $report->get(PROP_GROUP_BY);
		}
		
		$smarty = app()->getSmartyObject();
		
		foreach ($data as $prop=>$value)
		{
			$smarty->assign($prop, $value);
		}
		
		$content = smarty_templ_fetch('forms/form.usagereport.tpl.html',$smarty);
				
		return $content;	
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	function _submit()
	{
		$report = new shUsageReport;
		
		$startDate = "";
		$endDate = "";
		
		$report->set(PROP_GROUP_BY,ps('group_by'));		
		
		$range = ps('range');
		if ($range == 0)
		{
			$startDate = ps('start_date');
			$endDate = ps('end_date');
		}
		else if ($range == REPORT_RANGE_THIS_MONTH)
		{
			$startDate = mktime (0,0,0,date('m'),1,date('Y'));
			$endDate = time();
			
			$startDate = iwUtilities::databaseDateFormat($startDate);
			$endDate = iwUtilities::databaseDateFormat($endDate);
			
		}
		else if ($range == REPORT_RANGE_LAST_MONTH)
		{
			$startDate = mktime (0,0,0,date('n')-1,1,date('Y'));
			$endDate = mktime (23,59,59,date('n'),0,date('Y'));

			$startDate = iwUtilities::databaseDateFormat($startDate);
			$endDate = iwUtilities::databaseDateFormat($endDate);
		}
	
		$report->set(PROP_END_DATE,$endDate);
		$report->set(PROP_START_DATE,$startDate);
						
		$this->rememberFormData($report);
		
		if ($report->requiredFieldsValid())
		{
			app()->pushBreadcrumb (str('STR_REPORTS_BREADCRUMB'),'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWUSAGE);
			app()->pushBreadcrumb ($report->getTitle(),'');
			$this->_resultHtml = $report->generateReport();
			return true;
		}
		
		return false;		
	}	
	
	function authorize()
	{
		$user = app()->getLoggedInUserObject();
		return $user->getPermissions()->isAllowed(URLACTION_VIEWUSAGE,'');
	}
	
	/**
	 * When the form is submitted successfully, we want to actually show
	 * the requested report.  So, we don't redirect anywhere
	 */
	function getRedirectLocation ()
	{
		return '';
	}	
}
?>
