<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

define('PASSWORD_CHANGE_ONLY_FIELD','pc');

class shFormUser extends iwForm
{
	public function getFormAction()
	{
		return URLACTION_MODUSER;	
	}	
	
	function getFormType()
	{
		return FORMTYPE_USER;	
	}
	
	public function getTitle()
	{
		if ($this->getPasswordChangeOnly())
		{
			return str('STR_USER_CHANGE_PASSWORD');	
		}		
		else if ($this->isEditingObject())
		{
			return sprintf(str('STR_USER_EDIT_FORM_TITLE'), $this->getData()->username);
		} 
		else
		{
			return str('STR_USER_ADD_FORM_TITLE');			
		}
	}
		
	/**
	 * undocumented function
	 *
	 * @return void
	 * @author Karim Shehadeh
	 **/
	function isSubmittingForm()
	{
		// This form object is also used for processing
		//	batch jobs.
		if (ps('submitUserBatch') != '') 
		{
			return true;
		} 			
		
		return parent::isSubmittingForm();
	}
	
	/**
	 * Uses url/form parameter to determine if we should be displaying
	 * the password change form instead of the user form.
	 * @return bool Returns true if password change only form.
	 */
	function getPasswordChangeOnly()
	{
		return ((g(PASSWORD_CHANGE_ONLY_FIELD) == 1) || 
				(ps(PASSWORD_CHANGE_ONLY_FIELD) == 1));						
	}
	
	public function getRedirectLink()
	{
		$user = $this->getData();
		if ($user instanceof shUser)
		{
			if (ps('submitUserBatch') != '')
			{
				return sprintf('index.php?%s=%s', 
					URLARG_ACTION, URLACTION_USERMANAGER);	
			}
			else
			{
				return sprintf('?%s=%s&%s=%s', URLARG_ACTION,
					URLACTION_VIEWUSER,URLARG_ID,$user->username);
			}	
		}
				
		return parent::getRedirectLink();
	}
		
	/**
	 * If the ID parameter is in the url then that group
	 * is loaded and stored in the data field of the class
	 * otherwise a new object is created and set.
	 * @return bool Returns true unless an invalid id is given
	 * 				in the url id parameter. 
	 */
	protected function loadData()
	{
		// Check the cache first.
		$id = $this->getFormOrUrlId();
		$this->recallFormData();
		
		// Do we have a url or post data id we can use to load
		//	data from.
		if ($id != '')
		{
			// Do we have data in the cache to use
			if ($this->getData() instanceof shUser)
			{
				// Make sure that the cached data matches
				//	the id in the post or url (there might be 
				//	some stale data in the cache if the user
				//	just clicked away from a form after submitting with
				//	an error).
				if ($id == $this->getData()->username)
				{
					// Use the stored cache version
					return true;	
				}
			}			
			
			$user = new shUser();
			if ($user->loadId($id))
			{
				$this->setData($user);
				return true;
			}
			else
			{
				$this->setData(null);
				return false;
			}
		}
		else
		{
			// No id in the url or post data so just use the 
			//	cache if there is something there.
			if ($this->getData() instanceof shUser)
			{			
				return true;
			}
		}
		
		// Nothing in the cache and nothing in the 
		//	url or post data to draw data from so just instantiate
		//	a new user with default data.
		$this->setData(new shUser);
		return true;
	}
		
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function getFields ()
	{				
		$passwordChangeOnly = $this->getPasswordChangeOnly();
							   
		$edit = $this->isEditingObject();
		
		$allow_edit_domain = false;
		$permission = app()->getLoggedInUserObject()->getPermissions();
		if ($permission && 
			(	$permission->allowGlobalAddUser || 
				$permission->allowDomainAddUser ||
				$permission->allowDomainGroupAddUser))
		{
			$allow_edit_domain = true;
		}
		
		$fields = array();

		$fields[] = new shFormField(FIELD_TYPE_HIDDEN,$this->getData()->username,PROP_ID,'');
		$fields[] = new shFormField($edit?FIELD_TYPE_HIDDEN:FIELD_TYPE_TEXT,
						$this->getData()->username,PROP_USERNAME,str('STR_USER_USERNAME'));
		

		if (!$passwordChangeOnly)
		{
			$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->first_name,PROP_FIRST_NAME, str('STR_USER_FIRSTNAME'));
			$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->last_name,PROP_LAST_NAME,str('STR_USER_LASTNAME'));
			$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->email,PROP_EMAIL,str('STR_USER_EMAIL'));
			$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->city,PROP_CITY,str('STR_USER_CITY'));
			$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->state,PROP_STATE,str('STR_USER_STATE'));
			
			$permissionGroup = $permission->getPermissionGroup();
			if ($permissionGroup == USER_TYPE_ADMIN)
			{
				// Only the super administrator can change filebox sizes.
				$fields[] = new shFormField(FIELD_TYPE_TEXT,$this->getData()->item_pool_max_size_mb,PROP_ITEMPOOL_SIZE,str('STR_USER_FILEBOXSIZE'));
			}
			
			
			$userTypeOptions = array();			

            if ($permissionGroup == USER_TYPE_ADMIN)
            {
                $userTypeOptions[USER_TYPE_ADMIN] = str('STR_USER_TYPE_SUPER_ADMIN');
            }

            if ($permissionGroup == USER_TYPE_ADMIN ||
                $permissionGroup == USER_TYPE_DOMAIN_ADMIN)
            {
                $userTypeOptions[USER_TYPE_DOMAIN_ADMIN] = str('STR_USER_TYPE_ORG_ADMIN');                
            }
            
            if ($permissionGroup == USER_TYPE_ADMIN || 
                $permissionGroup == USER_TYPE_DOMAIN_ADMIN || 
                $permissionGroup == USER_TYPE_DOMAIN_GROUP_ADMIN)
            {
                $userTypeOptions[USER_TYPE_DOMAIN_GROUP_ADMIN] = str('STR_USER_TYPE_GROUP_ADMIN');
                $userTypeOptions[USER_TYPE_REGISTERED] = str('STR_USER_TYPE_REG');
            }
            
            if (count($userTypeOptions))
            {
                $field = new shFormField(FIELD_TYPE_SELECT,$this->getData()->getPermissions()->getPermissionGroup(),'usertype',str('STR_USER_TYPE'));                                
                $field->fieldOptions = $userTypeOptions;
                $fields[] = $field;
            }
            
            if ($allow_edit_domain)
            {	
                $selDomain = $this->getData()->getCompany()->id.','.$this->getData()->getGroup()->id;
                $field = new shFormField(FIELD_TYPE_SINGLE_DOMAINSELECTOR,$selDomain,'comp_groups',str('STR_USER_DOMAIN'));
                $field->fieldOptions['noaggregates'] = true;                
                $fields[] = $field;
            }
            else
            {
                $displayText = $this->getData()->getCompany()->name.'/'.$this->getData()->getGroup()->name;
                $fields[] = new shFormField(FIELD_TYPE_STATIC,$displayText,'',str('STR_USER_DOMAIN'));
            }		
		}
		else
		{
			// Indicates that this should be processed as a password only form.
			$fields[] = new shFormField(FIELD_TYPE_HIDDEN,'1',PASSWORD_CHANGE_ONLY_FIELD);
		}
		
		$field = new shFormField(FIELD_TYPE_PASSWORD,'',PROP_PASSWORD,str('STR_USER_NEWPASS'));
		
		if ($this->isEditingObject())
		{
			$field->fieldHelp = str('STR_USER_FORM_PASSWORD_OPTIONAL');			
		}
		$fields[] = $field;		
		$fields[] = new shFormField(FIELD_TYPE_PASSWORD,'','password_confirm',str('STR_USER_CONFIRM'));		

		return $fields;
	}
	
	/**
	 * Adds a script that clears the password fields in 
	 * the form.  Necessary to avoid browsers from populating
	 * password fields automatically
	 */
	protected function getFormSuffixContent()
	{
		$val = '
		<script type="text/javascript">
			function onClearPasswordFields()
			{
				YAHOO.util.Dom.get("'.PROP_PASSWORD.'").value = "";
				YAHOO.util.Dom.get("password_confirm").value = "";
			}			
			YAHOO.util.Event.addListener(window, "load", onClearPasswordFields);
		</script>		
		';
		
		return $val;
	}
	
	function processBatchFile()
	{
		if (isset($_FILES) && 
			is_array($_FILES['batch_file']) && 
			$_FILES['batch_file']['name']) 
		{				
			// Only set if there's something new to upload
			$file = $_FILES['batch_file']['tmp_name'];
			$lines = file ($file);
			if (count($lines) > 0)
			{
				$coll = new shUserCollection;
				$coll->bulkRegistration($lines);
				return true;
			}				
			else
			{
				$this->addError (str('STR_USERBATCH_FILEEMPTY'));
			}		
		}
		return false;	
	}
	
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */    
	protected function _submit ()
	{
		$success = true;
				
		$passwordOnly = (ps(PASSWORD_CHANGE_ONLY_FIELD) != '');
		
		if (ps('submitUserBatch') != '')
		{			
			return $this->processBatchFile();			
		}
				
		$user = new shUser;		
		$user->set(PROP_USERNAME,ps(PROP_USERNAME));
				
		if ($this->isEditingObject())
		{
			// Load the current properties
			if (!$user->load ())
			{
				$this->addError (str('STR_USER_NOTFOUND'), ERROR_TYPE_SERIOUS);
				return false;
			}
		}		
		
		$password = ps(PROP_PASSWORD);
		$passwordConfirm = ps('password_confirm');
		
		if ($password != $passwordConfirm)
		{
			$this->addError (str('STR_USER_PASS_NOMATCH'), ERROR_TYPE_SERIOUS);
			$success = false;
		}
				
		if (!$this->isEditingObject() || $passwordOnly)
		{
			if ($password == '')
			{
				$this->addError (str('STR_USER_INVALID_PASSWORD'), ERROR_TYPE_SERIOUS);
				$success = false;
			}
			else 
			{
				if (md5($password) == $user->get(PROP_PASSWORD))
				{
					$this->addError (str('STR_USER_SAME_PASS'), ERROR_TYPE_SERIOUS);
					$success = false;					
				}
			}
		}

		if (!$passwordOnly)
		{		
			$user->set(PROP_EMAIL, ps(PROP_EMAIL));
			$user->set(PROP_FIRST_NAME, ps(PROP_FIRST_NAME));
			$user->set(PROP_LAST_NAME, ps(PROP_LAST_NAME));
			$user->set(PROP_CITY, ps(PROP_CITY));
			$user->set(PROP_STATE, ps(PROP_STATE));
		}
		
		if ($this->isEditingObject() && $password == '')
		{
			// THIS IS OKAY - WE ASSUME THAT THEY DON'T WANT TO CHANGE THE PASSWORD.
		}
		else
		{
			$user->set(PROP_PASSWORD,md5($password));
		}		
				
		if (!$passwordOnly)
		{
            $company_id = 0;
            $group_id = 0;
            
			// USER TYPE (Admin,registered)
			$loggedInUser = app()->getLoggedInUserObject();
			$loggedInUserPermGroup = $loggedInUser->getPermissions()->getPermissionGroup(); 
						
            $usertype = 0;
            if ($loggedInUserPermGroup == USER_TYPE_REGISTERED)
            {
                $usertype = USER_TYPE_REGISTERED; 
                
                // Registered users cannot change their own domains or 
                //  their user type so we leave the domain as is and set the 
                //  usertype to registered manually.
                             
            }
            else
            {
                $usertype = ps('usertype');
                
                $selectedComGroup = ps('comp_groups');
                if ($selectedComGroup)
                {
                    list($company_id,$group_id) = split(',',$selectedComGroup);
                }            
                
                if (!shCompanyGroupPair::checkDomainExists($company_id,$group_id))
                {                
                    $this->addError(str('STR_USER_GROUPCOMPANY_NOTFOUND'), ERROR_TYPE_SERIOUS);
                    $success = false;
                }
             
                $user->setDomain($company_id,$group_id);   
            }

            
            if ($usertype == USER_TYPE_ADMIN)
            {
            	if ($loggedInUserPermGroup == USER_TYPE_ADMIN)
            	{	
                	$user->setPermissions(makeSuperAdminPermissions());
            	}
            	else
            	{
            		$this->addError(str('STR_USER_UNAUTHORIZED_VALUE'));
            	}
            }
            else if ($usertype == USER_TYPE_REGISTERED)
            {
                $user->setPermissions(makeUserPermissions());
            }
            else if ($usertype == USER_TYPE_DOMAIN_GROUP_ADMIN)
            {
            	if ($loggedInUserPermGroup != USER_TYPE_REGISTERED && 
            		$loggedInUserPermGroup != USER_TYPE_DOMAIN_GROUP_ADMIN)
            	{
            		$user->setPermissions(makeGroupAdminPermissions());
            	}
            	else
            	{
            		$this->addError(str('STR_USER_UNAUTHORIZED_VALUE'));
            	}
            }
            else if ($usertype == USER_TYPE_DOMAIN_ADMIN)
            {
            	if ($loggedInUserPermGroup == USER_TYPE_ADMIN)
            	{
            		$user->setPermissions(makeDomainAdminPermissions());
            	}
            	else
            	{
            		$this->addError(str('STR_USER_UNAUTHORIZED_VALUE'));
            	}
            }
            else
            {
            	$this->addError(str('STR_USER_INVALIDTYPE'));
            	$success = false;
            }
            
			// MAX FILEBOX SIZE (In megabytes)
			if ($loggedInUserPermGroup == USER_TYPE_ADMIN)
			{
				$user->set(PROP_ITEMPOOL_SIZE, ps(PROP_ITEMPOOL_SIZE));
			}
		}

		$this->setData($user);
		$this->rememberFormData();
				
		if (!$success)
		{
			return false;			
		}
				
		if ($user->requiredFieldsValid())
		{
			if ($this->isEditingObject())
			{	
                $continue = true;	
                nx_fire_event('EVENT_OBJECT_EDITING',array('object'=>$user,'continue'=>&$continue));
                if ($continue)
                {                                                                       
				    if ($user->update ())
				    {
					    $this->forgetFormData();
					    $liu = app()->getLoggedInUserObject();
					    
					    // Update the logged in user if it's the same as this one.
					    if ($user->get(PROP_USERNAME) == $liu->get(PROP_USERNAME))
					    {
						    // Make sure that the user  	
						    app()->refreshLoggedInUser();
					    }
					    
					    $this->addNotification (str('STR_USER_UPDATED'));
                        
                        nx_fire_event('EVENT_OBJECT_EDITED',array('object'=>$user)); 
                                   
					    return true;
				    }
                }
			}
			else
			{	
                $continue = true;	
                nx_fire_event('EVENT_OBJECT_ADDING',array('object'=>$user,'continue'=>&$continue));
                if ($continue)
                {                                                                   
				    if ($user->newRegistration ())
				    {				
					    $this->forgetFormData();
					    $this->addNotification (str('STR_USER_ADDED'));
                        
                        nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$user));
                        
					    return true;
				    }   
                }
            }
		}
		
		return false;	
	}
	
	function authorize ()
	{
		$user = app()->getLoggedInUserObject();
		
		if ($user != null)
		{
			$perm = $user->getPermissions();
            $id = $this->getUrlIdField();
            if (!$id) 
            {
                // See if there's a user id in the post data
                $id = ps(PROP_ID);
            }
			$isAuthorized = $perm->isAllowed(URLACTION_MODUSER,$id);
			
			return $isAuthorized;
		}
		
		return false;
	}
	
	function redisplayOnError ()
	{
		if (ps('submitUserBatch') != '')
		{
			return false;
		}
		
		return true;	
	}

	function requiredFieldsValid ($checkKey = false) 
	{
		$invalidField = false;
		
		$itempoolsize = $this->get(PROP_ITEMPOOL_SIZE);
		if (is_num ($itempoolsize)) 
		{
			$this->addError (str('STR_FILEBOX_INVALIDSIZE'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
		
		return (parent::requiredFieldsValid($checkKey) && !$invalidField);
	}

	/**
	 * Checks not only that the id field is non-empty but
	 * also that the id is not already in the database.  user objects
	 * are the only types of objects where the user of the app can pick
	 * the id of the record so we have to handle it differently.
	 *
	 * @return unknown
	 */
	function isEditingObject()
	{
		if (parent::isEditingObject())
		{
			$username = $this->getFormOrUrlId();
			
			$user = new shUser;
			return $user->loadId($username);	
		}
		
		return false;
	}
}
?>