<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


/**
 * A category is a way to organize application items so that 
 * users can easily find them.  A category can have another 
 * category as a parent which allows for nesting of categories.
 */
class shCategory extends iwDbTable
{
	
	/**
	 * An array of company/group pairs that have access
	 * to this category.
	 */	
	protected 	$_companyGroupPairs = array();
	
	/**
	 * Adds the following properties to the object
	 * 	@arg @c id 
	 * 	@arg @c parent_id
	 * 	@arg @c name
	 * 	@arg @c description
	 *	@arg @c status
	 */		  		 	 	 	 	 	
	protected function addProperties()
	{
		parent::addProperties ();
		$this->_addProperty (PROP_ID, 0);
		$this->_addProperty (PROP_PARENT, 0);
		$this->_addProperty (PROP_NAME, '');
		$this->_addProperty (PROP_DESCRIPTION, '');
		$this->_addProperty (PROP_STATUS, 0);
	}

	/** 
	 *	@see See base class	
	 */
	public function getTypeName ($asPlural)
	{
		return $asPlural ? str('STR_CAT_NAME_PLURAL') : str('STR_CAT_NAME');
	}

	/** 
	 *	@see See base class
	 */
	public function getType ()
	{
		return OBJECT_TYPE_CATEGORY;
	}

	/**
	 * When this function has a valid document associated with it
	 *	then the icon that matches the file type will be returned.
	 *	If no item matches the file type then the 'binary' icon
	 * 	will be returned.
	 *	If there is no valid document associated with this item then
	 *	the generic 'document' icon will be returned. 
	 */
	public function getItemIcon ($large=true)
	{		
		if ($large)
		{
			$icon = 'category_icon_large.jpg';
		}
		else
		{
			$icon = 'category_icon_small.jpg';
		}
		$icon = app()->getTemplateFile('images/'.$icon);
		return $icon;	
	}
		
	/**
	 *	The database fields for this table are:
	 *		@arg @c category_id (int)
	 *		@arg @c name (varchar255)
	 *		@arg @c description (text)
	 *		@arg @c parent_id (int)
	 *		@arg @c status (tinyint)
	 *			 	 	 	 	 	 	  
	*/
	public function getDBKeyValueArray ($includeDBKey)
	{

		$tableConstant = $this->getTableConstant();

		$fields = array
		(
		$GLOBALS[$tableConstant]['fields']['CATEGORY_ID'] => $this->get (PROP_ID),
		$GLOBALS[$tableConstant]['fields']['CATEGORY_NAME'] => $this->get (PROP_NAME),
		$GLOBALS[$tableConstant]['fields']['CATEGORY_DESCRIPTION'] => $this->get (PROP_DESCRIPTION),
		$GLOBALS[$tableConstant]['fields']['CATEGORY_PARENT'] => $this->get (PROP_PARENT),
		$GLOBALS[$tableConstant]['fields']['CATEGORY_STATUS'] => $this->get (PROP_STATUS)
		);

		return $fields;
	}

	/**  
	 * @see getDBKeyValueArray for table fields
	 * @see base class for description	  	
	*/
	public function setDBKeyValueArray ($dbFieldsAndValues)
	{
		$tableConstant = $this->getTableConstant();
		$this->set (PROP_ID, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['CATEGORY_ID']]);
		$this->set (PROP_NAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['CATEGORY_NAME']]);
		$this->set (PROP_DESCRIPTION, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['CATEGORY_DESCRIPTION']]);
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['CATEGORY_STATUS']]);
		$this->set (PROP_PARENT, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['CATEGORY_PARENT']]);
		
		$this->loadDomains();
		
		return true;
	}

	/**  
	 *	Returns @c CATEGORY_STATUS_ACTIVE if restoring.  Returns  
	 *	@c CATEGORY_STATUS_DELETED if deleting
	 */
	protected function getDBKeyValueForDelete ($isRestoring=false)
	{
		$fields = array ($GLOBALS[$this->getTableConstant()]['fields']['CATEGORY_STATUS'] => $isRestoring?CATEGORY_STATUS_ACTIVE:CATEGORY_STATUS_DELETED);
		return $fields;
	}

	/**  
	 *	Returns the string @c TABLE_CATEGORIES
	*/
	public function getTableConstant ()
	{
		return 'TABLE_CATEGORIES';
	}

	/**  
	 *	Required fields for this object type are:
	 *	@arg @c name 	 
	*/
	public function requiredFieldsValid ($checkKey = false)
	{
		$invalidField = false;
		if ($this->get(PROP_NAME) == '')
		{
			$this->addError (str('STR_CAT_NONAME'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}

		return !$invalidField;
	}

	/**  
	 * if fieldname is true, this returns the string @c 'category_id'
	 * if fieldname is false, this returns an integer that is the unique id 
	 *		of this object.	  
	*/
	public function getUniqueID ($fieldName=false)
	{
		if ($fieldName)
		{
			return $GLOBALS[$this->getTableConstant()]['fields']['CATEGORY_ID'];
		}
		else
		{
			return $this->get (PROP_ID);
		}
	}

	/**
	 * Gets the number of documents that are members of this category. Remember
	 * that documents can be in multiple categories so the total number of 
	 * documents in all the categories can be larger than the total number of
	 * documents in the library.	 	 	 
	 * @return int The number of documents that are in this category
	 */	 	 	
	public function getDocumentCount ()
	{
		$itemsInCategory = new shItemDocumentCollection();
		$collectionParams = new iwParameterCollection();
		$p = $collectionParams->add(new iwParameterArray(FILTER_CATEGORY));
		$p->addItem_nokey ($this->get(PROP_ID));

		$p = $collectionParams->add(new iwParameterArray(FILTER_STATUS));
		$p->addItem_nokey (ITEM_STATUS_ACTIVE);

		if (!app()->isAdminLoggedIn())
		{
			$p = $collectionParams->add(new iwParameterArray(FILTER_USER));
			$p->addItem_nokey (app()->getLoggedInUserObject()->get(PROP_USERNAME));
		}

		$itemsInCategory->load($collectionParams, true);
		return $itemsInCategory->getTotal();
	}

	/**
	 * Retrieves all the items that are members of this 
	 * 	category in the form of an shItemDocumentCollection.
	 *	@return shItemDocumentCollection The collection object 
	 */
	public function getCategoriesInCategoryCollection($filterOutRestricted=true)
	{
		$catsInCategory = new shCategoryCollection();
		$collectionParams = new iwParameterCollection();

		$p = new iwParameterArray(FILTER_CATEGORY);
		$p->addItem_nokey ($this->get(PROP_ID));
		$p = $collectionParams->add($p);

        $p = new iwParameter(FILTER_SORTBY,'name');
        $collectionParams->add($p);
        $p = new iwParameter(FILTER_SORTDIR,'asc');
        $collectionParams->add($p);

		$p = new iwParameterArray(FILTER_STATUS);
		$p->addItem_nokey (ITEM_STATUS_ACTIVE);
		$p = $collectionParams->add($p);


		if ($filterOutRestricted)
		{
			$user = app()->getLoggedInUserObject();

			// Super admin has no restrictions so don't filter results
			//	on this field.
			if ($user->getPermissions()->getPermissionGroup() != USER_TYPE_ADMIN)
			{
				$p = new iwParameterArray(FILTER_USER);
				$p->addItem_nokey ($user->get(PROP_USERNAME));
				$p = $collectionParams->add($p);
			}
		}

		$catsInCategory->load($collectionParams);
		return $catsInCategory;
	}
	
	/**
	 * Retrieves all the items that are members of this 
	 * 	category in the form of an shItemDocumentCollection.
	 *	@return shItemDocumentCollection The collection object 
	 */
	public function getItemsInCategoryCollection()
	{
		$itemsInCategory = new shItemDocumentCollection();
		$collectionParams = new iwParameterCollection();

		$p = new iwParameterArray(FILTER_CATEGORY);
		$p->addItem_nokey ($this->get(PROP_ID));
		$p = $collectionParams->add($p);

        $p = new iwParameter(FILTER_SORTBY,'item_title');
        $collectionParams->add($p);
        
        $p = new iwParameter(FILTER_SORTDIR,'asc');
        $collectionParams->add($p);
        
		$p = new iwParameterArray(FILTER_STATUS);
		$p->addItem_nokey (ITEM_STATUS_ACTIVE);
		$p = $collectionParams->add($p);


		if (!app()->isAdminLoggedIn())
		{
			$p = new iwParameterArray(FILTER_USER);
			$p->addItem_nokey (app()->getLoggedInUserObject()->get(PROP_USERNAME));
			$p = $collectionParams->add($p);
		}

		$itemsInCategory->load($collectionParams);
		
		return $itemsInCategory;
	}

	/** 
	 * Override to allow for category with ID 0 (root)
	 */
	public function load ()
	{
		if ($this->get(PROP_ID) == 0)
		{
			$this->set(PROP_NAME, 'Root');
			$this->set(PROP_DESCRIPTION, '');

			return true;
		}

		if (parent::load())
		{
			return true;
		}
		
		return false;
	}

	/**
	 * Populates the internal array of shCompanyGroupPair objects with those
	 * domains that are accessible by this category.
	 */
	public function loadDomains()
	{
		$this->_companyGroupPairs = array ();
		$query = 'select company_id,group_id from sh_category_access_map as sam ' .
				'WHERE sam.category_id = '.$this->get(PROP_ID);
				
        app()->flushCacheIfInvalid($query);
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($results !== false)
		{
			$documents = array ();
			foreach ($results as $result)
			{
				$pair = new shCompanyGroupPair;
				$group = new shGroup;
				$group->set(PROP_ID, $result['group_id']);
				$pair->setGroup ($group);

				$company = new shCompany;
				$company->set(PROP_ID, $result['company_id']);
				$pair->setCompany ($company);
				
				$this->_companyGroupPairs[] = $pair;
			}
		}	
		else
		{
			iwUtilities::logDatabaseError();
		}				
	}
	
	public function render($params = false)
	{
		
	}
	
	/**
	 * Gets a smarty array of data that can be used to populate the 
	 * the main library browser in Nexus.  Takes the following
	 * form: 
	 *  
	 *  library
	 * 		parent_category_id
	 *  	items
	 * 			id
	 * 			name
	 * 			actions[]
	 * 			name
	 * 			link
	 * 			special_link
	 * 		categories
	 * 			id
	 * 			name
	 * 			actions[]
	 * 			name
	 * 			link
	 * 			special_link
	 */
	public function generateBrowserData()
	{
		$library = array('categories'=>array(), 'items'=>array());
				
		// Setup the basic category properties for smarty
		$library['parent_category_id'] = $this->get(PROP_ID);		
		$library['parent_category_name'] = $this->get(PROP_NAME);
		$library['parent_category_description'] = $this->get(PROP_DESCRIPTION);
		$library['up_one_category_id'] = $this->get(PROP_PARENT);
		$library['actions'] = $this->getAvailableActions();
		$library['category_icon'] = $this->getItemIcon(true);
		$library['max_document_size'] = iwUtilities::byteFormat(DOCUMENT_MAX_FILESIZE);
								
		// Setup user notifications for updated categories and their children
		$notify = new shUserNotification;		
		$library['category_user_notification_update'] = 
				$notify->isRegisteredForCategoryNotification(
						$this->get(PROP_ID), NOTIFY_CHANGE_TYPE_UPDATE);
						
		$library['category_user_notification_add'] = 
				$notify->isRegisteredForCategoryNotification(
					$this->get(PROP_ID), NOTIFY_CHANGE_TYPE_ADD);		
		
		// Now get a list of all the child categories in this category.  These
		//	are used to display in the view.
		$categoryArray =  $this->getChildCategories();
		$categoryTrail = $this->getCategoryTrail();
		foreach ($categoryTrail as $catTrail)
		{
			app()->pushBreadcrumb ($catTrail->get(PROP_NAME),'index.php?'.
				URLARG_ACTION.'='.URLACTION_BROWSELIBRARY.'&'.
				URLARG_ID.'='.$catTrail->get(PROP_ID));
		}
		
		$documentCollection = $this->getItemsInCategoryCollection();
				
		foreach ($categoryArray as $cat)
		{
			$catInfo = array();
			$childCatCount = $cat->getChildCategoryCount();
			$catInfo['id'] = $cat->get(PROP_ID);
			$catInfo['link'] = 'index.php?a='.URLACTION_BROWSELIBRARY.'&id='.$cat->get(PROP_ID);
			$catInfo['name'] = $cat->get(PROP_NAME);
			$catInfo['description'] = iwUtilities::makeStringJavascriptSafe($cat->get(PROP_DESCRIPTION));
			$catInfo['icon'] = $cat->getItemIcon(true);
			$catInfo['actions'] = $cat->getAvailableActions();
			$catInfo['special_info'] = $cat->getDocumentCount().' docs, '.$childCatCount.' categories';
			
			$library['categories'][] = $catInfo;
		}

		foreach ($documentCollection->items() as $item)
		{			            
            $visible = true;
            $icon = '';
            $params = array('visible'=>&$visible,'doc'=>&$item, 'icon'=>&$icon);
            nx_fire_event('EVENT_BROWSER_PRE_DOCUMENT_ADD',$params);
            
            if ($visible)
            {
			    $info = array();
			    $info['id'] = $item->get(PROP_ID);
                
                $actions = $item->getAvailableActions(array('fromcat'=>$this->get(PROP_ID)));
                $info['actions'] = $actions;
                
                if (shPage::findActionById(ACTION_VIEW,$actions))
                {
			        $info['link'] = 'index.php?a='.URLACTION_VIEWDOC.'&id='.$item->get(PROP_ID).'&'.URL_FROMCAT.'='.$this->get(PROP_ID);
                }
                else
                {
                    $info['link'] = '';
                }
                
			    $info['name'] = $item->get(PROP_NAME);
			    $info['description'] = iwUtilities::makeStringJavascriptSafe($item->get(PROP_DESCRIPTION));                
                
                if ($icon != '')
                {
                    $info['icon'] = $icon;
                }
                else
                {
                    $info['icon'] = $item->getItemIcon(true);
                }
                                			    			    
			    $info['type'] = $item->get(PROP_SUBTYPE_ID);
			    
                nx_fire_event('EVENT_BROWSER_PRE_DOCUMENT_DISPLAY',array('info'=>&$info,'doc'=>&$item));

                $library['items'][] = $info;			
			}
		}
		
		return $library;
	}
		
	/**
	 * The available actions for all users are:
	 * 	@arg View
	 * 	
	 * The available actions for admin users are:	 	 
	 * 	@arg Edit
	 * 	@arg Send To Trash
	 */	 	  	 	
	public function getAvailableActions ($data='')
	{
		$actions =
		array (
		shPage::makeActionArray (str('STR_ACTION_VIEW'),
								app()->getTemplateFile('images/view_small.jpg'),
								'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWLIB.'&'.URLARG_ID.'='.$this->get(PROP_ID),
								'', 
								'View Category',
								ACTION_VIEW));



		$user = app()->getLoggedInUserObject();
		if ($user && shAuthorize::canEditCategory($user->get(PROP_USERNAME),$this->get(PROP_ID)))
		{
			if ($this->get(PROP_ID) != 0)
			{
				$actions[] = 
				shPage::makeActionArray (str('STR_ACTION_RENAME'), 
								 app()->getTemplateFile('images/rename_small.jpg'),
								'javascript:void(0);',
								'javascript:renameObject(\''.TYPE_PREFIX_CAT.'\',\''.$this->get(PROP_ID).'\')', 
								'Rename Category',
                                ACTION_RENAME);
											
				$actions[] =
				shPage::makeActionArray (
					'Edit',	
					app()->getTemplateFile('images/edit_small.jpg'),
					'index.php?'.URLARG_ACTION.'='.URLACTION_MODCAT.'&'.URLARG_ID.'='.$this->get(PROP_ID),
					'', 
					'Edit Category',
                    ACTION_EDIT);
	
				$actions[] =
					shPage::makeActionArray (str('STR_ACTION_TRASH'),
					app()->getTemplateFile('images/trash_small.jpg'),
					 'index.php?'.URLARG_ACTION.'='.URLACTION_DELCAT.'&'.URLARG_ID.'='.$this->get(PROP_ID),
					 '', 
					 'Trash Category',
                     ACTION_TRASH);
	
				$actions[] =
					shPage::makeActionArray (str('STR_ACTION_VIEWHISTORY'),
					app()->getTemplateFile('images/history_small.jpg'),
					'index.php?a=view_mod&filter_object_type='.OBJECT_TYPE_CATEGORY.'&filter_object_id='.$this->get(PROP_ID),
					'', 
					'View History',
                    ACTION_HISTORY);
			}		
			
		}

		return $actions;
	}

	public function renderAsListItem ($selectable, $details, $actions=true)
	{
		$img = $this->getItemIcon(false);

		$description = iwUtilities::trucateWithEllipsis (strip_tags ($this->get(PROP_DESCRIPTION)), 125);
		if (!$description)
		{
			$description = str('STR_NO_DESCRIPTION');
		}

		$link = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWLIB.'&'.URLARG_ID.'='.$this->get(PROP_ID);

		$smarty =  app()->getSmartyObject();
		$smarty->assign ('collection_item_id', $this->get(PROP_ID));
		$smarty->assign ('collection_item_name', $this->get(PROP_NAME));
		$smarty->assign ('collection_item_show_details', $details);
		$smarty->assign ('collection_item_image', $img);
		$smarty->assign ('collection_item_elementid', TYPE_PREFIX_CAT.'_'.$this->get(PROP_ID).'_name');
		$smarty->assign ('collection_item_selectable', $selectable);
		$smarty->assign ('collection_array_name', get_class ($this));
		$smarty->assign ('collection_item_specialinfo', $this->getDocumentCount().' document(s) in category');
		$smarty->assign ('collection_item_description', $description);
        
        if ($actions)
        {
		    $smarty->assign ('collection_item_actions', $this->getAvailableActions());
        }
        
		$smarty->assign ('collection_item_droptype', 'group');
		$smarty->assign ('collection_item_viewlink', $link);
		$smarty->assign ('collection_item_draggable', true);
		$collection_item_html = smarty_templ_fetch ('collections/collection.item.tpl.html',$smarty);

		return $collection_item_html;
	}

	/** 
	 *	The smarty variables that are prepared for this object include:
	 *	@arg @c 'id'
	 *	@arg @c 'name'
	 *	@arg @c 'description'	 	 	 	
	 */
	public function toArray()
	{
		$array = array();
		$array [PROP_ID] = $this->get (PROP_ID);
		$array [PROP_NAME] = $this->get (PROP_NAME);
		$array [PROP_DESCRIPTION] = $this->get (PROP_DESCRIPTION);
		return $array;
	}

	/**
	 * After permanent deletion the category refernce is removed from the 
	 * item>category mapping table as well as from any other categories in
	 * the categories table that reference this category as a parent.  In the
	 * latter case, the parent is set to 0 making the child category a top-level
	 * category.
	 */	 	 	 	 	 			
	protected function onAfterPermanentDelete ()
	{
        $this->_deleteMappings();            
        $this->deleteUpdateNotifications();
        $this->adjustChildItemsAndCategories();
        
        parent::onAfterPermanentDelete();
	}

	/**
	 * Retrieves all direct child categories of this category 
	 * as an array of obects.
	 * @param bool Set to true if it should check if the current user has 
	 *			has access to each category.
	 * @return array The array of category objects (or an empty array if there
	 * 					was a problem).
	 */
	public function getChildCategories ($filterOutRestricted=true)
	{
		$collection = $this->getCategoriesInCategoryCollection($filterOutRestricted);
		return $collection->items();
	}

	public function getChildCategoryCount ()
	{
		$query = 'select count(*) from sh_categories where parent_id='.$this->get(PROP_ID).' AND status='.CATEGORY_STATUS_ACTIVE;
		$result = app()->getDatabase()->GetOne($query);
		if (($result))
		{
			return $result;
		}

		return 0;
	}	
	public function getParentCategory()
	{
		if ($this->get(PROP_PARENT))
		{
			$cat = new shCategory;
			$cat->set(PROP_ID, $this->get(PROP_PARENT));
			if ($cat->load())
			{
				return $cat;
			}
		}
		else
		{
			if ($this->get(PROP_ID) != 0)
			{
				$cat = new shCategory;
				$cat->set(PROP_ID, 0);
				if ($cat->load())
				{
					return $cat;
				}
			}
		}

		return null;
	}

	/**
	 * This retrieves the categories the parent hierarchy leading from this 
	 *  category up to the root. If this category is at the root, then this
	 *  will return an empty array.
	 *  @return array An array of shCategory objects
	 */	 	 	 	 	
	public function getCategoryTrail ()
	{
		$array = array();

		// Add this category to the trail if it's not the root.
		$array[] = $this;

		$parent = $this->getParentCategory();
		while ($parent)
		{
			$array[] = $parent;
			$parent = $parent->getParentCategory();
		}

		return array_reverse($array);
	}

	public function getChildItems()
	{
	}
	

	/**
	 * Gets the array of company/group pairs which have access to this
	 * 	item.
	 * @return array An array of shCompanyGroupPair objects
	 */
	public function getDomains ()
	{
		return $this->_companyGroupPairs;
	}

	public function doesCompanyHaveAccess($cid)
	{
		foreach($this->_companyGroupPairs as $pair)
		{
			if ($cid == $pair->getCompany()->get(PROP_ID))
			{
				return true;
			}
		}
		
		return false;
	}
	
	public function doesDomainHaveAccess($cid,$gid)
	{
		foreach($this->_companyGroupPairs as $pair)
		{			
			if (in_array($pair->getCompany()->id, array($cid, COMPANY_ALL, COMPANY_UNREGISTERED)) &&
				in_array($pair->getGroup()->id, array($gid, GROUP_ALL)))
			{
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Clears all objects from the list of copmany/group pairs
	 */
	public function clearCompanyGroupPairs ()
	{
		$this->_companyGroupPairs = array ();
	}
	
	/** 
	 * Adds a company/group pair object that has 
	 * access to this item. 
	 * @param mixed $company An id or shCompany object
	 * @param mixed $group An id or shGroup object
	 */ 		
	public function addCompanyGroup ($company,$group)
	{		
		$pair = new shCompanyGroupPair;
		if (is_object ($company))
		{			
			$pair->setCompany($company);
		}
		else
		{
			$comp = new shCompany;
			$comp->set(PROP_ID, $company);
			$pair->setCompany($comp);
		}

		if (is_object ($group))
		{			
			$pair->setGroup($group);
		}
		else
		{
			$gr = new shGroup;
			$gr->set(PROP_ID, $group);
			$pair->setGroup($gr);
		}
		
		$this->_companyGroupPairs[] = $pair;
	}	
	
	
	/**
	 * Updates the category to the database, sets the modification
	 * date, then updates the associated data in the necessary tables.
	 *	@arg Company/Group pairs that have access to this category
	 */
	public function update ()
	{		
		$this->set(PROP_MODIFIED_DATE,self::getDatabaseTimeString(time()));					
		if (parent::update())
		{		
			$this->_updateMappings ();
			return true;
		}
		
		return false;
	}
	
	/**
	 * Used to clean up references to this category in tables other than this one.
	 *	Cleans references in the following tables:
	 *		@arg sh_category_access_map
	 */
	protected function _deleteMappings ()
	{
		
		// Remove existing company/group mappings
		$res = app()->getDatabase()->query ("DELETE FROM sh_category_access_map WHERE category_id='".$this->get(PROP_ID)."'");
		if (!$res)
		{
			iwUtilities::logDatabaseError();
			return false;
		}
		
		return true;
	
	}
	
    public function deleteUpdateNotifications()
    {
        $res = db()->Execute('delete from sh_category_notification_map where category_id = '.$this->get(PROP_ID));
        if (!$res)
        {
            iwUtilities::logDatabaseError();
            return false;
        }
        return true;
    }
    
    /**
    * @desc Sets child items and categories such that they are no longer 
    *       parented to this category.  Instead they are parented to the root.
    */
    protected function adjustChildItemsAndCategories()
    {
        $query1 = 'update sh_categories set parent_id=0 where parent_id = '.$this->get(PROP_ID);
        $query2 = 'update sh_item_category_map set category_id = 0 where category_id = '.$this->get(PROP_ID);
        
        $res = db()->Execute($query1);
        if (!$res)
        {
            iwUtilities::logDatabaseError();
        }        
        
        $res = db()->Execute($query2);        
        if (!$res)
        {
            iwUtilities::logDatabaseError();
        }
    }
    
	/**
	 */
	public function _updateMappings ()
	{
				
		if ($this->_deleteMappings ())
		{					
			foreach ($this->_companyGroupPairs as $pair)
			{	
				$group_id = $pair->getGroup()->get(PROP_ID);
				$company_id = $pair->getCompany()->get(PROP_ID);
								
				// Add company and group to respective mapping tables.
				$fields_values = array('category_id'=>$this->get(PROP_ID),
									   'group_id'=>$group_id,
									   'company_id'=>$company_id);
									   
				$ret = app()->getDatabase()->AutoExecute('sh_category_access_map', $fields_values, 'INSERT');
	
				if (!$ret)
				{
					iwUtilities::logDatabaseError();
				}						
			}		
		}		
	}	
	
	/**
	 * Adds the item to the database, sets the creation and modification
	 * dates, then adds the associated data to the necessary tables.
	 *	@arg Company/Group pairs that have access to this item
	 *  @arg Related Item IDs
	 * 	@arg Attributes
	 * 	@arg Categories
	 */
	public function add ()
	{
		if (parent::add())
		{
			$this->_updateMappings ();
			return true;
		}

		return false;
	}	
}
?>
