<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * The group class represents a single group in the database and on a web page
 * It handles populating and retrieving information in/from the database as
 * well as deleting all traces of itself in the case where it is permanently
 * deleted.  The group can show itself as a full page with details or as a list
 * item with some detail.
 */     
class shGroup extends iwDbTable
{
	/**
	 * Adds group-specific properties
	 * @arg @c PROP_ID The ID of the group
	 * @arg @c PROP_NAME The name the group
	 * @arg @c PROP_DESCRIPTION The description of the group (optional)
	 * @arg @c PROP_STATUS Record status (GROUP_STATUS_ACTIVE or GROUP_STATUS_DELETED)
	 * @note This will add the parent class's properties as well
	 */	 	 	 	 	 	 	
	protected function addProperties () 
	{
		parent::addProperties ();
		$this->_addProperty (PROP_ID,0);
		$this->_addProperty (PROP_NAME,'');
		$this->_addProperty (PROP_DESCRIPTION,'');
		$this->_addProperty (PROP_STATUS,0);
        $this->_addProperty (PROP_COMPANY_ID,0);
	}

	/** 
	 */
	public function getTypeName ($asPlural) 
	{		
		return $asPlural ? str('STR_GROUP_NAME_PLURAL') : str('STR_GROUP_NAME');
	}
	
	/** 
	 *	Returns @c OBJECT_TYPE_GROUP
	 */
	public function getType () 
	{
		return OBJECT_TYPE_GROUP;
	}

	/**
	 * @arg @c group_id (int)
	 * @arg @c name (varchar255)
	 * @arg @c description (varchar255)
	 * @arg @c status (tinyint)	 	 
	 */
	public function getDBKeyValueArray ($includeDBKey) 
	{
		$tableConstant = $this->getTableConstant();
		
		$fields = array
				(
					$GLOBALS[$tableConstant]['fields']['GROUP_ID'] => $this->get (PROP_ID),
					$GLOBALS[$tableConstant]['fields']['GROUP_NAME'] => $this->get (PROP_NAME),
					$GLOBALS[$tableConstant]['fields']['GROUP_DESCRIPTION'] => $this->get (PROP_DESCRIPTION),
					$GLOBALS[$tableConstant]['fields']['GROUP_STATUS'] => $this->get (PROP_STATUS),
		);
						
		return $fields;
	}
	
	/**  Sets all the object properties based on the given database field values
     *	Given is an associative array where the keys are the database field names
     *	and the values are the values of those fields.
     *	@param array $dbFieldsAndValues The associative array of database field names and values.
     *	@return bool True if all the required fields were found and copied over.
	*/
	public function setDBKeyValueArray ($dbFieldsAndValues) 
	{
	
		$tableConstant = $this->getTableConstant();
		$this->set (PROP_ID, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['GROUP_ID']]);
		$this->set (PROP_NAME, iwUtilities::unescapeString(@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['GROUP_NAME']]));
		$this->set (PROP_DESCRIPTION, iwUtilities::unescapeString(@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['GROUP_DESCRIPTION']]));
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['GROUP_STATUS']]);
		return true;
	}
	
	/**  
     *	Returns an array with these fields:
     *	@arg @c status => GROUP_STATUS_ACTIVE or GROUP_STATUS_DELETED 	
     *	
	 */
	protected function getDBKeyValueForDelete ($isRestoring=false) 
    {
			$fields = array ($GLOBALS[$this->getTableConstant()]['fields']['GROUP_STATUS'] => $isRestoring?GROUP_STATUS_ACTIVE:GROUP_STATUS_DELETED);								
			return $fields;
	}
	
	/**  
	 * Returns the string @c TABLE_GROUPS
	 */
	protected function getTableConstant () 
	{
		return 'TABLE_GROUPS';
	}

    /**
    * @desc Deletes references to this group in the company/group map    
    */
    protected function deleteCompanyGroupMaps()
    {
        $query = 'delete from sh_company_group_map where group_id='.$this->get(PROP_ID);
        $res = db()->Execute($query);
        if (!$res)
        {
            return false;
        }
        
        return true;
    }
	
    protected function deleteItemReferences()
    {
        $res = db()->Execute('select * from sh_item_access_map where group_id='.$this->get(PROP_ID));
        if (!$res)
        {
            return false;
        }
        else
        {
            db()->Execute('DELETE FROM sh_item_access_map WHERE group_id='.$this->get(PROP_ID));
            return true;
        }        
    }
    
    protected function deleteCategoryReferences()
    {
        $res = db()->Execute('select * from sh_category_access_map where group_id='.$this->get(PROP_ID));
        if (!$res)
        {
            return false;
        }
        else
        {
            $companiesHandled = array();
            
            db()->Execute('DELETE FROM sh_category_access_map WHERE group_id='.$this->get(PROP_ID));

            return true;
        }          
    }
    
    protected function deleteUserReferences()
    {        
        $res = db()->Execute('select * from sh_user_access_map where group_id='.$this->get(PROP_ID));
        if (!$res)
        {
            return false;
        }
        else
        {
            $companiesHandled = array();
            
            // For each result , check the organization and see if that org has another group to 
            //  switch to.  If it does, switch to it.
            foreach($res as $row)
            {
                if (!in_array($row['company_id'],$companiesHandled))
                {
                    $company = new shCompany;
                    $company->loadId($row['company_id']);
                    $groups = $company->getGroups();
                    $handled = false;
                    foreach($groups as $group)
                    {
                        if ($group->get(PROP_ID) != $this->get(PROP_ID))
                        {
                            // Change the group reference to another group in the org.
                            db()->Execute('update sh_user_access_map set group_id='.$group->get(PROP_ID).' where company_id='.$company->get(PROP_ID));
                            $companiesHandled[]=$company->get(PROP_ID);
                            $handled = true;
                            break;
                        }
                    }                
                    
                    if (!$handled)
                    {
                        // there are no other groups to switch to so return to initial default group/company
                        db()->Execute('update sh_user_access_map set company_id=1 and group_id=1');
                    }
                }
            }
            
            return true;
        }         
    }
        
	/**
	 * Retrieves the number of users who are members of this group. 
	 * Regardless of their company affiliation.	 
	 * @return Returns the number of users who are members of this group
	 */	 	 	
	public function getUserCount() 
	{
		$query = 'select count(*) as total from sh_user_access_map where group_id='.
					$this->get(PROP_ID);
        
        app()->flushCacheIfInvalid($query);
		$result = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($result !== false)	
		{																	
			return $result[0]['total'];
		}		
	}
	
	/**  
	 * 	Fields checked: 	
     *	@arg @c name
	 */
	public function requiredFieldsValid ($checkKey = false) 
	{
	
		$invalidField = false;
		
		if ($this->get(PROP_NAME) == '') {
			$this->addError (str('STR_GROUP_NONAME'), ERROR_TYPE_SERIOUS);
			$invalidField = true;
		}
	
		return !$invalidField;
	}
	
	/**  
	 * If fieldName is true, this returns @c 'group_id'
	 * If fieldName is false, this returns an integer	 	
	 */
	function getUniqueID ($fieldName=false) 
	{
		if ($fieldName) {
			return $GLOBALS[$this->getTableConstant()]['fields']['GROUP_ID'];
		}
		else {
			return $this->get (PROP_ID);
		}
	}
	
	/**
	 * This returns the users in the given group regardless of the organization
	 * @return 
	 */
	function getUsers()
	{
		$coll = new shUserCollection;
		$params = new iwParameterCollection;			
		$p = new iwParameterArray(FILTER_GROUP);
		$p->addItem_nokey ($this->get(PROP_ID));				
		$params->add($p);			
		$coll->load($params);
		return $coll->items();			
	}
	
	/**
	 * Displays the group in a full page format.  	 
	 */	 	
	function render ($params=false)
	{
				
		$html = '';
		$item = array ('id'=>$this->get(PROP_ID), 
							'name'=>$this->get(PROP_NAME), 
							'description'=>$this->get(PROP_DESCRIPTION),
							'actions'=>$this->getAvailableActions());

		$users = $this->getUsers();
		$smartyUsers = array();
		foreach ($users as $user)
		{
			$permissionsGroup = $user->getPermissions()->getPermissionGroup();
		
			$smartyUsers[]=
				array(
					'name'=>$user->getFullName(),
					'id'=>$user->get(PROP_ID),
					'is_group_admin'=>($permissionsGroup == USER_TYPE_DOMAIN_GROUP_ADMIN),
					'is_org_admin'=>($permissionsGroup == USER_TYPE_DOMAIN_ADMIN),
					'is_super_admin'=>($permissionsGroup == USER_TYPE_ADMIN)
				);
		}
		$item['users'] = $smartyUsers;
											
		$username = app()->getLoggedInUserObject()->get(PROP_USERNAME);
		app()->notify (EVENT_OBJECT_VIEWED,array('type'=>$this->getType(),'id'=>$this->getUniqueID(),'username'=>$username));
		
		$smarty = app()->getSmartyObject ();
		$smarty->assign ('group',$item);
		return smarty_templ_fetch('views/group.tpl.html',$smarty);												
	}		
	
	/**
	 * Static method which returns a group object with the 
	 * properties necessary to represent All Groups in a company
	 * 
	 * @return shGroup Returns a group object
	 */
	static public function getAllGroup()
	{
		$group = new shGroup;
		$group->set(PROP_ID, GROUP_ALL);
		$group->set(PROP_NAME, "All Groups");
		$group->set(PROP_DESCRIPTION, "Represents all groups in the company");
		return $group;
	}
		
	/**
	 * When a group is deleted, it must update the company-group mapping table
	 * as well as the item-company-group access table to make sure there are
	 * no remnanents left in the database
	 */	 	 	 	
	protected function onAfterPermanentDelete ()
	{
        $this->deleteCompanyGroupMaps();
        $this->deleteCategoryReferences();
        $this->deleteItemReferences();
        $this->deleteUserReferences();
	}	
	
	/**
	 * Group allows the following actions for all users:
	 * 	@arg @c View
	 * 	
	 *  And for administrators:
	 *  
	 *	@arg @c Edit
	 *	@arg @c Send To Trash
	 */
	public function getAvailableActions ($data='')
	{
		$actions =  
			array (
				shPage::makeActionArray (str('STR_ACTION_VIEW'), 
								 app()->getTemplateFile('images/view_small.jpg'),
								'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWGROUP.'&'.URLARG_ID.'='.$this->get(PROP_ID),
								'', 
								str('STR_ACTION_VIEW'))								
			);
				
			
		
		if (app()->isAdminLoggedIn())
		{
			$actions[] = 
			shPage::makeActionArray (str('STR_ACTION_EDIT'), 
							 app()->getTemplateFile('images/edit_small.jpg'),
							'index.php?'.URLARG_ACTION.'='.URLACTION_MODGROUP.'&'.URLARG_ID.'='.$this->get(PROP_ID),
							'', 
							str('STR_ACTION_EDIT'));
						
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_TRASH'), 
							 app()->getTemplateFile('images/trash_small.jpg'),
							 'javascript:areYouSure(\''.str('STR_ASK_DELETE_ONE').'\', \'index.php?'.URLARG_ACTION.'='.URLACTION_DELGROUP.'&'.URLARG_ID.'='.$this->get(PROP_ID).'\')',
							 '', 
							 str('STR_ACTION_TRASH'));
							
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_VIEWHISTORY'), 
							 app()->getTemplateFile('images/history_small.jpg'),
							 'index.php?a=view_mod&filter_object_type='.OBJECT_TYPE_COMPANY.'&filter_object_id='.$this->get(PROP_ID),
							 '', 
							 str('STR_ACTION_VIEWHISTORY'));		
							
		}				
		
		return $actions;
	}
		
	/**
	 * Groups take on the same look as most other components when in a flat
	 * list - that is: icon followed by details about the component.		 	 	 	 
	 */	 	
	public function renderAsListItem ($selectable, $details, $actions=true)
	{
		$img = $this->getItemIcon(false);						
		
		$description = iwUtilities::trucateWithEllipsis (strip_tags ($this->get(PROP_DESCRIPTION)), 125);
		
		if ($description == '')
		{
			$description = str('STR_NO_DESCRIPTION');
		}
		
		$link = 'index.php?a='.URLACTION_VIEWGROUP.'&id='.$this->get(PROP_ID);
				
		$smarty =  app()->getSmartyObject();
		$smarty->assign ('collection_item_id', $this->get(PROP_ID));
		$smarty->assign ('collection_item_name', $this->get(PROP_NAME));
		$smarty->assign ('collection_item_show_details', $details);
		$smarty->assign ('collection_item_image', $img);
		$smarty->assign ('collection_item_selectable', $selectable);
		$smarty->assign ('collection_array_name', get_class ($this));
        $smarty->assign ('collection_item_elementid', '');
		$smarty->assign ('collection_item_specialinfo', $this->getUserCount().' users(s) in this group');
		$smarty->assign ('collection_item_description', $description);
		$smarty->assign ('collection_item_actions', $this->getAvailableActions());
		$smarty->assign ('collection_item_droptype', 'group');
		$smarty->assign ('collection_item_viewlink', $link);		
		$smarty->assign ('collection_item_draggable', true);
		$collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html',$smarty);
								
		return $collection_item_html;		
	}	

	/**
	 * Requires that the active theme contain an image in the icons directory
	 * with the name group_icon_large.jpg	  
	 */
	public function getItemIcon ($large=true)
	{
				
		if ($large)
		{
			$icon = 'group_icon_large.jpg';
		}
		else
		{
			$icon = 'group_icon_small.jpg';
		}
		$icon = app()->getTemplateFile('images/'.$icon);
		return $icon;	
	}
	
	/**
	 * Returns the data portion of this object as an associative array
	 * @return array Returns all the properties of this object
	 */
	public function toArray()
	{
		return $this->_properties;
	}
}

?>
