<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shPoolItem extends shItemDocument
{	    
	protected function addProperties () 
	{
		parent::addProperties ();				        
        $this->_addProperty(PROP_USERNAME,'<none>');
	}
	
	public function inLibrary()
	{
		return false;
	}
		
	/**
	 * Retrieves the icon 
	 * @param bool $large True if the larger icon is to be retrieved.
	 * @return unknown
	 */
	function getItemIcon ($large=true)
	{
		$icon = '';
		
		if ($this->get(PROP_ID) != 0)
		{
			$fileType = $this->getFileTypeObject();		
			if ($fileType instanceof shFileType)
			{
				$icon = $fileType->getThumbnailPath($this, $large?48:16, $large?48:16);
				if ($icon == '')
				{
					$icon = $path;
				}			
			}
		}
		else
		{
			if ($large)
			{
				$icon = app()->get('template_dir').'/images/pool_icon_large.jpg';
			}
			else
			{
				$icon = '';
			}			
			
		}		
		return $icon;	
	}
		
	
	/** Retrieves the classes friendly type name
	 * @param bool $asPlural True to return the plural version, false otherwise.
	 * @return string The name as a string
	 *  @access public
	 */
	function getTypeName ($asPlural) {		
		return $asPlural ? str('STR_FILEBOX_NAME_PLURAL') : str('STR_FILEBOX_NAME');
	}
	
	/** Returns an object type code of OBJECT_TYPE_UNKNOWN if unspecified
	 * @return int The object type value
	 * @access public
	 */
	function getType () {
		return OBJECT_TYPE_POOLITEM;
	}
			
	/**  Gets the key string for the associative array that contains field information for the table associated with this object.
     *	@see tables.inc.php
     *	@return string The key field string for the assocative array that contains field information for the table associated with this object.
	*/
	function getTableConstant () {
		return 'TABLE_ITEM_POOL';
	}
		
	/**  Checks if the data stored in this object is valid.
     *	All this does is verify that there is valid data in the *required*
     *	fields.
     *	@param bool $checkKey If this is true, then the unique key field for the object is checked for validity as well
     *	@return bool True, if all required fields are valid.
	*/
	function requiredFieldsValid ($checkKey = false) {
	
		$success = true;
		
		if ($this->get(PROP_NAME) == '') {
			$this->addError (str('STR_FILEBOX_NONAME'), ERROR_TYPE_SERIOUS);
			$success = false;
		}
				
		return $success;
	}

    public function getInternalFileName()
    {
        $dir = app()->get(PROP_FILEBOX_DIR);
        $filename = $this->get(PROP_ORIGINAL_FILENAME);
        $parts = pathinfo($filename);
        if (isset ($parts['extension']))
        {
            $extension = '.'.$parts['extension'];
        }
        else
        {
            $extension = '';
        }
        
        return '__filebox__'.sprintf ("%04d", $this->get(PROP_ID)).$extension;
    }
        
	/**  Gets all the database field names with associated values
     *	This will return an associative array where the keys
     *	are the table field names and the values are the values
     *	stored in this class.
     *	@param bool $includeDBKey If true, then the key field for the table is returned in the array as well.
     *	@return array An associative array of dbase fields => class values
	*/
	function getDBKeyValueArray ($includeDBKey) {
	
		$tableConstant = $this->getTableConstant();
				
		$fields = array
				(
					$GLOBALS[$tableConstant]['fields']['ITEM_ID'] => $this->get (PROP_ID),
					$GLOBALS[$tableConstant]['fields']['ITEM_NAME'] => $this->get (PROP_NAME),
					$GLOBALS[$tableConstant]['fields']['ITEM_USERNAME'] => $this->get (PROP_USERNAME),
					$GLOBALS[$tableConstant]['fields']['ITEM_DESCRIPTION'] => $this->get (PROP_DESCRIPTION),
					$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_ID'] => $this->get (PROP_TYPE_ID),
					$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_VERSION'] => $this->get (PROP_TYPE_VERSION),
					$GLOBALS[$tableConstant]['fields']['ITEM_ORIGINAL_FILENAME'] => $this->get (PROP_ORIGINAL_FILENAME),
					$GLOBALS[$tableConstant]['fields']['ITEM_DATA'] => $this->get (PROP_DATA),
					$GLOBALS[$tableConstant]['fields']['ITEM_CREATION_DATE'] => $this->get (PROP_CREATION_DATE),
					$GLOBALS[$tableConstant]['fields']['ITEM_STATUS'] => $this->get (PROP_STATUS),
					$GLOBALS[$tableConstant]['fields']['ITEM_SUBTYPE_ID'] => $this->get (PROP_SUBTYPE_ID)
		);
						
		return $fields;
	}
	
	/**  Sets all the object properties based on the given database field values
     *	Given is an associative array where the keys are the database field names
     *	and the values are the values of those fields.
     *	@param array $dbFieldsAndValues The associative array of database field names and values.
     *	@return bool True if all the required fields were found and copied over.
	*/
	function setDBKeyValueArray ($dbFieldsAndValues) 
	{		
		$tableConstant = $this->getTableConstant();
		
		$id = @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_ID']];
		if ($id != 0)
		{
			$this->set (PROP_ID, $id);
		}
		
		$this->set (PROP_NAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_NAME']]);
		$this->set (PROP_USERNAME, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_USERNAME']]);
		$this->set (PROP_DESCRIPTION, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_DESCRIPTION']]);
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_STATUS']]);
		$this->set (PROP_TYPE_ID,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_ID']]);
		$this->set (PROP_SUBTYPE_ID,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_SUBTYPE_ID']]);
		$this->set (PROP_TYPE_VERSION,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_TYPE_VERSION']]);
	 	$this->set (PROP_ORIGINAL_FILENAME,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_ORIGINAL_FILENAME']]);	  
	 	$this->set (PROP_DATA,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_DATA']]);
		$this->set (PROP_CREATION_DATE,@$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_CREATION_DATE']]);			 
		$this->set (PROP_STATUS, @$dbFieldsAndValues[$GLOBALS[$tableConstant]['fields']['ITEM_STATUS']]);			  

		return true;
	}

	function getViewingTemplate()
	{
		return 'poolitem.tpl.html';
	}

	/** Pool items don't have attributes (at least not yet) */
	function allowAttributes () 
	{
		return false;
	}

	/**
	 * Returns an array of usernames of users who have access to this document because
	 * the owner shared the document.
	 * @return array An array of username strings.
	 */
	public function getSharedUsers()
	{
        $usernames = array();
        
        $fileBoxShareDb = $GLOBALS['TABLE_POOL_ACCESS']['name'];
        $query = "select username from sh_pool_user_access
                    where sh_pool_user_access.pool_item_id = {$this->get(PROP_ID)}";        
            
        app()->flushCacheIfInvalid($query);              
        $results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
        if ($results !== false)
        {
            $documents = array ();
            foreach ($results as $result)
            {
                $usernames[] = $result['username'];
            }
        }
                    
        return $usernames;		
	}
	
    /**
     * Renders the document as a single line of text/images.  This is used
     * in areas OTHER THAN the main browser view.
     */
    public function renderAsListItem ($selectable, $details, $actions=true)
    {
        $img = $this->getItemIcon(false);                        
        
        $belongsToMe = ($this->username == app()->getLoggedInUsername());
        $description = iwUtilities::trucateWithEllipsis (strip_tags ($this->get(PROP_DESCRIPTION)), 125);
                
        // show those users who have access to this file
        if ($belongsToMe)
        {
            $users = $this->getSharedUsers();
            $usersHtml = array();
            foreach ($users as $user)
            {
                $usersHtml[] = "<span id='pool_share_$user'>$user [<a onclick=\"javascript:unshareFileboxItemFromUser('$user',$this->id,'sharedUsers')\" href=\"javascript:void(0);\">x</a>]</span>";
            }
            
            if (count($usersHtml) == 0)
            {
                $description .= '<br/><b>Shared with</b>: <span id="noshare">'.str('STR_FILEBOX_NOT_SHARED').'</span>';
            }
            else
            {
                $description .= '<br/><b>Shared with</b>: <span id="sharedUsers">'.implode('&nbsp;',$usersHtml).'</span>';            
            }
        }
        else
        {
            $description .= "<br/>Belongs to {$this->username}";
        }
        
        $link = 'index.php?a=view_'.$this->getUrlActionText().'&id='.$this->get(PROP_ID);
            
        $smarty =  app()->getSmartyObject();
        $smarty->assign ('collection_item_id', $this->get(PROP_ID));
        $smarty->assign ('collection_item_name', $this->get(PROP_NAME));
        $smarty->assign ('collection_item_show_details', $details);
        $smarty->assign ('collection_item_image', $img);
        $smarty->assign ('collection_item_elementid', TYPE_PREFIX_DOC.'_'.$this->get(PROP_ID).'_name');
        $smarty->assign ('collection_item_selectable', $selectable);
        $smarty->assign ('collection_array_name', get_class($this));
        $smarty->assign ('collection_item_specialinfo', $this->get(PROP_ORIGINAL_FILENAME));
        $smarty->assign ('collection_item_description', $description);
        $smarty->assign ('collection_item_actions', $this->getAvailableActions());
        $smarty->assign ('collection_item_droptype', $this->getUrlActionText());
        $smarty->assign ('collection_item_viewlink', $link);        
        $smarty->assign ('collection_item_draggable', $this->isDraggable());            
        $collection_item_html = smarty_templ_fetch('collections/collection.item.tpl.html',$smarty);
                                
        return $collection_item_html;        
    }
        
	function getAvailableActions ($data='')
	{
        $belongsToMe = ($this->get(PROP_USERNAME) == app()->getLoggedInUsername());
        
        if ($belongsToMe)
        {                                
		    $actions[] =  
				    shPage::makeActionArray (str('STR_ACTION_VIEW'), 
								     app()->getTemplateFile('images/view_small.jpg'),
								    'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWPOOL.'&amp;'.URLARG_ID.'='.$this->get(PROP_ID),
								    '', 
								    str('STR_ACTION_VIEW'));
        }
                                
        $actions[] = 
				shPage::makeActionArray (str('STR_ACTION_DOWNLOAD'), 
								app()->getTemplateFile('images/download_small.jpg'),
								'download.php?'.URLARG_ID.'='.$this->get(PROP_ID).'&amp;t='.OBJECT_TYPE_POOLITEM,
								'', 
								str('STR_ACTION_DOWNLOAD'));

        if ($belongsToMe)
        {                                
            $actions[] =                                 
                    shPage::makeActionArray (str('STR_ACTION_SHARE'), 
                                    app()->getTemplateFile('images/share_small.jpg'),
                                    'javascript:void(0)',
                                    'javascript:shareFileboxItemWithUser('.$this->id.', \'sharedUsers\')', 
                                    str('STR_ACTION_SHARE'));
        }		
		
		if (app()->isAdminLoggedIn())
		{
			$actions[] = 
			shPage::makeActionArray (str('STR_ACTION_EDIT'), 
							 app()->getTemplateFile('images/edit_small.jpg'),
							'index.php?'.URLARG_ACTION.'='.URLACTION_MOD_POOL.'&amp;'.URLARG_ID.'='.$this->get(PROP_ID),
							'', 
							'Edit Document');
			
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_DELETE'), 
							 app()->getTemplateFile('images/delete_small.jpg'),
							 'javascript:areYouSure(\''.str('STR_ASK_DELETE_ONE').'\', \'index.php?'.URLARG_ACTION.'='.URLACTION_DELPOOLITEM.'&amp;'.URLARG_ID.'='.$this->get(PROP_ID).'\')',
							 '', 
							 str('STR_ACTION_DELETE'));
							
			$actions[] = 	
			shPage::makeActionArray (str('STR_ACTION_VIEWHISTORY'), 
							 app()->getTemplateFile('images/history_small.jpg'),
							 'index.php?a=view_mod&filter_object_type='.OBJECT_TYPE_POOLITEM.'&amp;filter_object_id='.$this->get(PROP_ID),
							 '', 
							 str('STR_ACTION_VIEWHISTORY'));		
			
		}				
		
		return $actions;
	}
	
	/**
	 * This gets the name of the item type as it appears in urls after 
	 *	the action prefix: view_[text] or mod_[text]
	 *	@return string The action text for this item type
	 */
	public function getUrlActionText()
	{
		return "pool";
	}	
	
	/**
	 * returns false because pool documents can't be kept in the trash
	 *	or in the item bin.
	 */
	public function isDraggable ()
	{
		return false;
	}	

	/**
	 * Determines if the given user has access to this filebox document because it
	 * has been shared
	 * @return bool Returns true if it shared with the given user.
	 * @param $username string The username to checkl
	 */
	public function isSharedWithUser($username)
	{
		$usernames = array();
        
        $query = "select count(username) from sh_pool_user_access 
                    where pool_item_id = {$this->get(PROP_ID)} AND username='$username'";        
       
        app()->flushCacheIfInvalid($query);               
        $results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
        if ($results !== false)
        {
            return ($results[0][0] > 0);
        }
        else
        {
            iwUtilities::logDatabaseError(); 
        }
                        
        return false;
 
	}
	
    /**
    * @desc Removes share of filebox document  with another user in the system
    * @param string $userToRemoveShareFrom The username of the user to unshare 
    * @return bool Returns true if the document was shared, false otherwise.
    */
    public function unshare($userToRemoveShareFrom)
    {
        $query = "delete from sh_pool_user_access where pool_item_id = {$this->get(PROP_ID)} and username='$userToRemoveShareFrom'";
        if (app()->getDatabase()->Execute($query) === false)
        {	
              iwUtilities::logDatabaseError(); 
              return false;
        }
        
		app()->invalidateTable('sh_pool_user_access');
        return true;
    }	
	
	/**
	 * Adds a share to this document for the given user.  Ignored if there is already a share setup
	 * @return bool Returns true if new share added, false otherwise.
	 * @param $username string The user's username to share with.
	 */
	public function share($username)
	{
        if (!$this->isSharedWithUser($username))
        {
            $query = "insert into sh_pool_user_access (pool_item_id,username,access) VALUES ({$this->get(PROP_ID)},'$username',0)";
            if (app()->getDatabase()->Execute($query) === false)
            {
                  iwUtilities::logDatabaseError(); 
                  return false;
            }
            
			app()->invalidateTable('sh_pool_user_access');
            return true;
        }
        else
        {
            return false;
        }		
	}
}
?>
