<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shPost
{
	/**
	 * The unique id of this post
	 */
	public $id;

	/**
	 * The id of the parent post (can be zero)
	 */
	public $parent_post_id;
	
	/**
	 * The title of the post (no more than 255 characters)
	 */
	public $title;
	
	/**
	 * The body of the post.  This will NOT contain any HTML
	 */
	public $body;
	
	/**
	 * The date/time that this was posted in the form of 
	 * 	a standard MySQL database time/date string 
	 */
	public $datePosted;
	
	/**
	 * The user who posted this
	 */
	public $username;
	
	/**
	 * This is an array of shPost objects that were posted as 
	 * 	replies to this post
	 */
	public $replies;
	
	/**
	 * The item(document) associated with this post
	 */
	public $item_id;
	
	/**
	 * Takes the associative array that maps to the posts table
	 *	and  populates this object's data.
	 *	@param array $row Associate array of table data
	 */
	public function load ($row)
	{
		$this->id = $row['post_id'];
		$this->title = $row['title'];
		$this->body = $row['body'];
		$this->datePosted = $row['creation_date'];
		$this->username = $row['username'];
		$this->parent_id = $row['parent_post_id'];
		$this->replies = array();	
		$this->item_id = $row['item_id'];
		
		return true;
	}
	
	public function loadFromId ($id)
	{
		$query = "select * from sh_item_conversations where post_id = $id";
        
        app()->flushCacheIfInvalid($query);
		$rows = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($rows !== false)	
		{																	
			foreach ($rows as $row)
			{
				if ($this->load($row))
				{
					return true;
				}					
			}
		}
		
		return false;
	}		
	
	/**
	 * Adds the stored information as a new post to the database.
	 * @return bool Returns true if added to database, false otherwise
	 */
	public function add ()
	{
		$date = iwUtilities::databaseDateFormat($this->datePosted);
		
		$query = "insert into sh_item_conversations (title,body,username,creation_date, parent_post_id, item_id) ".
					"VALUES ('$this->title', '$this->body', '$this->username', '$date', $this->parent_post_id, $this->item_id)";

		$result = app()->getDatabase()->query($query);		
		if ( ($result))
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Permanently deletes the post 
	 */
	public function delete()
	{
		$query = 'delete from sh_item_conversations where post_id = '.$this->id;
		$result = db()->query($query);
		if ($result)
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Updates the post in the database with the new information stored in this
	 *	object
	 *	@return bool Returns true if the post was updated in the database.
	 */
	public function update ()
	{
		$query = "update sh_item_conversations set 
						title='$this->title', 
						body='$this->body', 
						username='$this->username', 
						creation_date='$this->datePosted', 
						parent_post_id='$this->parent_post_id', 
						item_id='$this->item_id' 
						where post_id=$this->id";
		
		$result = app()->getDatabase()->query($query);		
		if ( ($result))
		{
			return true;
		}
		
		return false;	
	}
    
    /**
    * @desc Returns the properties of this object as an array
    *       of values keyed on the property name.
    * @return array Associatvie array of property=>value pairs.
    */
    public function toArray()
    {
        return get_object_vars($this);
    }
	
}

?>