<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shMainPage extends shPage
{
	function getTemplate ()
	{
		return 'main';
	}

	function display ()
	{	
		if ($this->isLoggingOut ())
		{		
			$this->logout();
			$this->addSmartyVariable(SMARTY_LOGGEDIN, false);
			$this->addSmartyVariable(SMARTY_ISADMIN, false);
		}
	
		// Note that the form action stuff
		//	has its own autherization stuff (since it also handles login/logout 
		//	forms) so we don't put this inside the 'authorize' call below. 
		$formResult = $this->doFormAction(true);
		
		// We process the statusbar (which contains user information) after
		//	we've done any logging in or out.
		$this->processStatusBar ();

		$this->addSmartyVariable(SMARTY_ISFRONTPAGE, false);
		$this->addSmartyVariable(SMARTY_JAVASCRIPT_ONLOAD, '');	
		
		if ($this->authorize())
		{
			// Only display the content if we're not displaying a form
			if ($formResult != kFormDisplayed && 
				$formResult != kFormHandledContent)
			{
				if (!$this->doUrlAction ())
				{		                    
					// The requested action could not be performed 
					//	so jsut show the main page.					
					$this->displayFrontPage();                    
				}
			}
		}
		else
		{
			if (app()->getLoggedInUserObject() != null)
			{
				// User is logged in but not with the right credentials so display an error message
				$this->addError (str('STR_PAGE_NO_PERMISSION'), ERROR_TYPE_SERIOUS);
			}
			else
			{                
                $continue = true;                
                $action = $this->getUrlArgument(URLARG_ACTION);
                $id = $this->getUrlArgument(URLARG_ID);
                $results = nx_fire_event('EVENT_EXECUTE_ACTION_PRE',array('action'=>$action,'id'=>$id,'page'=>&$this,'continue'=>&$continue));
                if ($continue)
                {               
				    if ($formResult != kFormDisplayed)
				    {                    
					    // Not logged in and no other forms
					    //	showing so just display the login form
					    $form = new shFormLogin;
					    $this->addContent($form->render());
					    $this->addSmartyVariable (SMARTY_JAVASCRIPT_ONLOAD, 'document.getElementById(\''.$form->getDefaultField().'\').focus();');
				    }
                }
			}
		}

		$this->addPageTemplateVars();
		
		// Make sure and save the location *after* we've processed
		//	the content since we could get into an infinite loop
		//	if the processing yields a redirect. And we don't save forms
		//	as redirect targets because we could get into an infinite redirect
		//	loop.
		if ($formResult != kFormDisplayed)
		{			
			app()->saveLocation();
		}

		parent::display ();
	}

	/**
	 * Adds the content to the page object's smarty object that will display the 
	 * content area of the main page whether a user or 
	 *
	 */
	function displayFrontpage ()
	{		
		$smarty = app()->getSmartyObject();
		$user = app()->getLoggedInUserObject();
				
		// Determine what to display based on what company the user is an employee of.		
		$company = $user->getCompany();
        $frontContent = '';
        $companyLogo = '';
		if ($company)
		{						
			if ($company->get(PROP_INTRO_PAGE)!= '')
			{
                $frontContent = '<div class="intro_text">'.$company->get(PROP_INTRO_PAGE).'</div>';
			}
			else
			{
				$frontContent = '';
			}
			
			if ($company->get(PROP_LOGO) != '')
			{
				$companyLogo = '<div class="company_logo"><img src="download.php?type='.OBJECT_TYPE_COMPANY.'&id='.$company->get(PROP_ID).'" title="Company Logo"/></div>';
			}
			else
			{
				$companyLogo = '';
			}
		}
        
        $actionHtml = $this->getActionHtml(nx_get_user_actions());        
        
        $html = $companyLogo.$frontContent;                    
        if ($html != '')
        {
            $html = '<div class="front_content">'.$html.'</div>';
            $html .= '<div style="clear:both"></div>';
            $this->addContent($html);
        }
                
        $this->addContent($actionHtml);

        // Recent history html
		$history = new shModCollection();
		$params = new iwParameterCollection();
		$params->add(new iwParameter(FILTER_USER,array(app()->getLoggedInUsername())));
		$params->add(new iwParameter(FILTER_SORTBY,$GLOBALS['TABLE_MOD']['fields']['MOD_DATE']));
		$params->add(new iwParameter(FILTER_SORTDIR,'desc'));
		$params->add(new iwParameter(FILTER_START,1));
		$params->add(new iwParameter(FILTER_PERPAGE,10));
		$history->load($params);
		
		$historyBlock1 = '<div class="history_block">';
		$historyBlock1 .= '<div class="history_block_title">'.str('STR_FRONT_RECENT_ACTIVITY').'</div>';
		$historyBlock1 .= '<ul>';
		foreach ($history->items() as $mod)
		{
			$icon = $mod->getItemIcon(false);
			$historyBlock1 .= '<li><img src="'.$icon.'" titile="Action Icon"/>&nbsp;'.$mod->getActionDescription().' on '.iwUtilities::dateTimeFormat ($mod->get('mod_date'), false).'</li>';
		}
		if (app()->isAdminLoggedIn())
		{
			$historyBlock1 .= '<li><a href="index.php?'.URLARG_ACTION.'='.URLACTION_VIEWMOD.'">'.str('STR_VIEW_FULL_HISTORY').'</a></li>';
		}
		$historyBlock1 .= '</ul></div>';
		
		$this->addContent($historyBlock1);
	}
	
    function getActionHtml($actions)
    {               
        $html = '';
        
        foreach ($actions as $action)
        {
            if (isset($action->parent) && ($action->parent == true))
            {                
                $html .= "<div class='action_block'>\r\n";
                // This is a parent action (contains other actions)
                $html .= "<div class='action_block_title'>{$action->title}</div>\r\n";                
                $html .= $this->getActionHtml($action->children);            
                $html .= "</div>\r\n";                
            }
            else
            {
                $html .= "<div class='action_block_action'>";
                
                $icon = '';
                if (isset($action->icon) && $action->icon != '')
                {   
                    $img = new iwImage($action->icon);                    
                    if ($img->isValid())
                    {
                        $padding = $img->getWidth() + 4;
                        $icon = "style='display:block; padding-left: {$padding}px; background-image:url({$action->icon});background-repeat:no-repeat;'";
                    }
                    else
                    {                    
                       $this->addError(str('STR_IMAGE_EXTENSION_NOT_INSTALLED'));                           
                    }
                }
                                
                $html .= "<a $icon href='{$action->link}'>{$action->title}</a>";
                $html .= "</div>\r\n";                
            }
        }                       
        
        return $html;
    }
    
	/**
	 * Compiles a list of strings that are warnings that an administrator
	 * should be notified of concerning various problems found on the system
	 * that may affect smooth operation of the site.
	 * @return array Returns an array of string that can be displayed to the user 
	 */
	function getAdministrativeWarnings()
	{
		$msg = array();
		
		// Maximum file  size
		$postMaxSize = ini_get('post_max_size');
		$phpMax = iwUtilities::phpIniSettingToBytes($postMaxSize);
		$nexusMax = DOCUMENT_MAX_FILESIZE;
		if ($phpMax < $nexusMax)
		{
			$msg[] = sprintf(str('STR_ADMIN_UPLOAD_WARNING_1'),
						'post_max_size',
						iwUtilities::byteFormat($phpMax),
						iwUtilities::byteFormat(DOCUMENT_MAX_FILESIZE));  
		}
		
	
		$postMaxSize = ini_get('upload_max_filesize');
		$phpMax = iwUtilities::phpIniSettingToBytes($postMaxSize);
		if ($phpMax < $nexusMax)
		{
			$msg[] = sprintf(str('STR_ADMIN_UPLOAD_WARNING_1'),
						'upload_max_filesize',
						iwUtilities::byteFormat($phpMax),
						iwUtilities::byteFormat(DOCUMENT_MAX_FILESIZE));  			
		}
					
		$time = ini_get('max_execution_time');
		if (($time < 120) && ($nexusMax > 104857600))
		{
			$msg[] = sprintf(str('STR_ADMIN_UPLOAD_WARNING_2'),
						$time,iwUtilities::byteFormat(DOCUMENT_MAX_FILESIZE));
		}	
	}
}

?>
