<?php

/**
 * A utility class used to perform commonly used actions
 *	that are complex and should be centralized to facilitate
 *	code reuse.  
 */
class shActions
{
	protected $_page = null;
	protected $action_output = '';

	public function __construct()
	{
		$this->_page = app()->getActivePage();
	}
	
	public function getOutput()
	{
		return $this->action_output;
	}
	
	public function doDeleteObject($objectType, $id, $perm) 
	{
		if (class_exists($objectType))
		{
			$ob = new $objectType;
			$ob->set(PROP_ID,$id);
			if ($ob->load() && $this->verifyDelete($ob))
			{
				$ob->enableTrueDeletion($perm);
				if ($ob->delete())
				{
                    if (!$perm)
                    {
                        app()->addGlobalNotification (str('STR_ITEM_TRASHED'));
                    }
                    else
                    {
                        app()->addGlobalNotification (str('STR_ITEM_PERMDELETED'));
                    }					                    
					return true;
				}	
			}
		}
		
		return false;
	}
	
	public function doRestoreObject($objectType,$id)
	{
		if (class_exists($objectType))
		{
			$ob = new $objectType;
			$ob->set(PROP_ID,$id);
			if ($ob->load())
			{
				if ($ob->restore())
				{
					app()->addGlobalNotification (str('STR_ITEM_RESTORED'));
					return true;
				}				
				else
				{
					app()->addError (str('STR_RESTORE_OBJECT_NOTFOUND'), ERROR_TYPE_SERIOUS);
				}
			}
			else
			{
				app()->addError (str('STR_RESTORE_OBJECT_NOTFOUND'), ERROR_TYPE_SERIOUS);				
			}
		}	
	}
	
	public function doListObjects($objectType,$data='')
	{
		$user = app()->getLoggedInUserObject ();
		$permissions = $user->getPermissions();
		
		$collection = null;		
		$defaultParams = new iwParameterCollection();
		
		switch ($objectType)
		{
			case OBJECT_TYPE_SPECIAL_TRASH:
				app()->pushBreadcrumb (str('STR_VIEW_TRASH'),app()->_getURLWithParameters(array()));
				$collection = new shTrashCollection();
				$collection->setTitle(str('STR_VIEW_TRASH'));
				break;
				
			case OBJECT_TYPE_BRIEFCASE:
				
				$collection = new shItemDocumentCollection;
				
				app()->pushBreadcrumb (str('STR_VIEW_BRIEFCASE'),app()->_getURLWithParameters(array()));
				
				$defaultParams->add (new iwParameter (FILTER,true));
				
				$p = new iwParameterArray (FILTER_INBIN);
				$p->addItem('bin',true);
				$defaultParams->add ($p);

				$p = new iwParameterArray (FILTER_STATUS);
				$p->addItem_nokey (ITEM_STATUS_ACTIVE);
				$defaultParams->add ($p);
				
				$collection->setTitle(str('STR_VIEW_BRIEFCASE'));
				break;
				
			case OBJECT_TYPE_ITEM:

				$collection = new shItemDocumentCollection;
				
				app()->pushBreadcrumb (str('STR_VIEW_LIBRARY'),app()->_getURLWithParameters(array()));
									
				$defaultParams->add (new iwParameter (FILTER,true));

				$activeFilter = new iwParameterArray (FILTER_STATUS);
				$activeFilter->addItem_nokey (ITEM_STATUS_ACTIVE);

				$companyFilter = new iwParameterArray (FILTER_COMPANY);
				$companyFilter->addItem_nokey ($user->getCompany()->get(PROP_ID));

				$groupFilter = new iwParameterArray (FILTER_GROUP);
				$groupFilter->addItem_nokey ($user->getGroup()->get(PROP_ID));
				
				if ($permissions->allowGlobalEditDocument)
				{
					$defaultParams->add ($activeFilter);	
				}
				else if ($permissions->allowDomainEditDocument)
				{
					$defaultParams->add($activeFilter);
					$defaultParams->add($companyFilter);
				}
				else 
				{
					$defaultParams->add($activeFilter);
					$defaultParams->add($companyFilter);
					$defaultParams->add($groupFilter);
				}
				
				$collection->setTitle(str('STR_VIEW_ALLDOCS'));

				break;
			case OBJECT_TYPE_POOLITEM:
            
				$collection = new shPoolItemCollection;
				app()->pushBreadcrumb (str('STR_VIEW_FILEBOX'),app()->_getURLWithParameters(array()));
				
				$defaultParams->add (new iwParameter (FILTER,true));
				
				$p = new iwParameterArray (FILTER_STATUS);
				$p->addItem_nokey (ITEM_STATUS_ACTIVE);
                
                if (g('shared') == 1)
                {
                    $defaultParams->add(new iwParameter(FILTER_SHARED,true));   
                }
                
				$defaultParams->add ($p);
				$collection->setTitle(str('STR_VIEW_FILEBOX'));
				
				break;
			case OBJECT_TYPE_CATEGORY:
				app()->pushBreadcrumb (str('STR_VIEW_CATEGORIES'),app()->_getURLWithParameters(array()));
				$collection = new shCategoryCollection;				
				$defaultParams->add (new iwParameter (FILTER,false));
				$collection->setTitle(str('STR_VIEW_CATEGORIES'));
				break;
			case OBJECT_TYPE_COMPANY:
				app()->pushBreadcrumb (str('STR_VIEW_ORGS'),app()->_getURLWithParameters(array()));
				$collection = new shCompanyCollection;
				$defaultParams->add (new iwParameter (FILTER,false));
				$collection->setTitle(str('STR_VIEW_ORGS'));
				break;
			case OBJECT_TYPE_GROUP:
				app()->pushBreadcrumb (str('STR_VIEW_GROUPS'),app()->_getURLWithParameters(array()));
				$collection = new shGroupCollection;
				$defaultParams->add (new iwParameter (FILTER,false));
				$collection->setTitle(str('STR_VIEW_GROUPS'));
				break;
			case OBJECT_TYPE_USER:
				app()->pushBreadcrumb (str('STR_VIEW_USERS'),app()->_getURLWithParameters(array()));
				$collection = new shUserCollection;
				$defaultParams->add (new iwParameter (FILTER,false));
				$collection->setTitle(str('STR_VIEW_USERS'));
				break;
			case OBJECT_TYPE_LOG:
				app()->pushBreadcrumb (str('STR_VIEW_HISTORY'),app()->_getURLWithParameters(array()));
				$collection = new shModCollection;
				$defaultParams->add (new iwParameter (FILTER,false));
				$collection->setTitle(str('STR_VIEW_HISTORY'));
				break;
			case OBJECT_TYPE_ATTRIBUTE:
				app()->pushBreadcrumb (str('STR_VIEW_TAGS'),app()->_getURLWithParameters(array()));
				$collection = new shAttributeCollection;
				$defaultParams->add (new iwParameter (FILTER,false));
				$collection->setTitle(str('STR_VIEW_TAGS'));
				break;
			default:
				app()->addError(str('STR_VIEW_INVALID'),ERROR_TYPE_SERIOUS);
				return;
		}

		$this->displayGenericCollection($collection,$defaultParams);		
	}
	
	public function doViewObject($class,$id)
	{		
		if (class_exists($class))
		{
			$ob = new $class;			
			$ob->set(PROP_ID, $id);
			if ($ob->load())
			{			                	
				$this->addViewBreadcrumb($ob);	
				$this->action_output = $ob->render();
				
				return true;
			}
		}		
		
		return false;
	}
	
	public function addViewBreadcrumb($ob)
	{
		if ($ob instanceof shItemDocument)
		{
			$catid = $this->_page->getUrlArgument(URL_FROMCAT);
			if ($catid != '')
			{
				$category = new shCategory;
				$category->set(PROP_ID, $catid);
				if ($category->load())
				{
					$categoryTrail = $category->getCategoryTrail();
					foreach ($categoryTrail as $catTrail)
					{
						app()->pushBreadcrumb ($catTrail->get(PROP_NAME),'index.php?'.
							URLARG_ACTION.'='.URLACTION_BROWSELIBRARY.'&'.
							URLARG_ID.'='.$catTrail->get(PROP_ID));
					}					
				}
			}
			
			app()->pushBreadcrumb ($ob->get(PROP_NAME),app()->_getURLWithParameters(array()));			
		}
		else
		{
            $action = str('STR_ACTION_VIEW');
			app()->pushBreadcrumb ($action.' '.$ob->getTypeName(false).': '.$ob->get(PROP_NAME),app()->_getURLWithParameters(array()));
		}
	}
	
	public function verifyDelete($ob)
	{
		if ($ob instanceof shCompany)
		{
			$users = $ob->getUsers();
			if (count($users) > 0)		
			{
                $userCollection = new shUserCollection;
                $userCollection->setItems($users);

				app()->addError(str('STR_COMPANY_DELFAILED_HASUSERS'),ERROR_TYPE_SERIOUS);				
				$userCollection->setTitle(sprintf(str('STR_AJAX_USERS_BELONGING_TO'),$ob->get(PROP_NAME)));
				$userCollection->setSubtitle('');
				$userCollection->setShowFilter(false);
				$userCollection->setEnableSorting(false);
                
				$this->action_output = $userCollection->html();
				
				return false;
			}
		}	
		else if ($ob instanceof shGroup)
		{
			$users = $ob->getUsers();
			
			if (count($users) > 0)		
			{
				$collection = new shUserCollection;
				$collection->setItems($users);
				
				app()->addError(str('STR_GROUP_DELFAILED_HASUSERS'),ERROR_TYPE_SERIOUS);				
				$collection->setTitle(sprintf(str('STR_AJAX_USERS_BELONGING_TO'),$ob->get(PROP_NAME)));
				$collection->setSubtitle(str('STR_AJAX_CHANGE_ORG'));
				$collection->setShowFilter(false);
				$collection->setEnableSorting(false);
				$this->action_output = $collection->html();
				
				return false;
			}				
		}
        else if ($ob instanceof shUser)
        {
            $currentUsername = app()->getLoggedInUsername();
            if ($currentUsername == $ob->get(PROP_USERNAME))
            {
                app()->addError(str('STR_USER_DELFAILED_CANNOTDELETESELF'));
                app()->redirectToLastLocation();
                return false;
            }
        }
		
		return true;
	}
	
	public function doPurgeHistory ($date)
	{
		if ($date != '')
		{
			if (shModRecord::purgeHistory($date))
			{
				app()->addGlobalNotification(str('STR_MOD_PURGE_SUCESS'));
			}				
			else
			{
				app()->addError(str('STR_MOD_PURGE_FAILED',ERROR_TYPE_SERIOUS));
			}
		}
		else
		{
			app()->addError(str('STR_MOD_NOPURGEDATE'),ERROR_TYPE_SERIOUS);
		}					
	}
	
	protected function displayGenericCollection(iwObjectCollection $collection, 
												iwParameterCollection $params)
	{		
		$page = $this->_page->getUrlArgument(URL_PAGE,1);
		$perPage = 10;		
		$start = (($page-1) * $perPage);
		
		// Make sure that the collection has a chance to change the default
		//	parameters for the collection listing.
		$params->mergeParameters($collection->processFilters());							   
		
		$params->add (new iwParameter(URL_START,$start));
		$params->add (new iwParameter(URL_PERPAGE,$perPage));
				
		// Determine if the url indicates what the sort should be.
		$sortby = $this->_page->getUrlArgument(URL_SORTBY);
		$sortdir = $this->_page->getUrlArgument(URL_SORTDIR,'desc');
		if ($sortby)
		{
			$params->add (new iwParameter(FILTER_SORTBY,$sortby));
			$params->add (new iwParameter(FILTER_SORTDIR,$sortdir));
		}		

		$collection->load ($params);
		$this->action_output = $collection->html($perPage,$page);
	}
		
	/**
	 * Adds a tag to the database or if it already exists,
	 * returns the existing tag object.
	 *
	 * @param string $tagname The tag name
	 * @return shAttribute The added (or existing) attribute
	 */
	public function doAddTag($tagname)
	{
		// Check if the attribute already exists
		$query = 'select * from sh_attributes where attribute_name="'.$tagname.'"';		
		$tagCollection = new shAttributeCollection;
		$tagCollection->loadFromQuery ($query);
		$tag = null;
		if ($tagCollection->count() == 0)
		{
			$tag = new shAttribute;
			$tag->set(PROP_NAME, $tagname);
			$tag->set(PROP_DESCRIPTION, 'Created on '.iwUtilities::dateTimeFormat(time(), true));
			$tag->set(PROP_ATTRIBUTE_TYPE,0);
			
			$tag->add();
		}
		else 
		{
			$tag = $tagCollection->getAt (0);
		}		
		
		return $tag;
	}
	
	/**
	 * Applies a given tag object to a document
	 * with the given id 
	 *
	 * @param int $docId The id of the document to apply to
	 * @param shAttribute $tagObj The attribute object
	 * @return bool Returns true if added, false otherwise.
	 */
	public function doApplyTagToDocument($docId,$tagObj)
	{
		$item = new shItemDocument;
		$item->set (PROP_ID, $docId);				
		if ($item->load())
		{			
			if (!$item->hasAttribute($tagObj->get(PROP_NAME)))	
			{
				$item->addAttribute ($tagObj);
				return $item->update();						
			}
		}	

		return false;
	}
}
?>