<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shAjaxServer
{
    public function populateCompanyGroups ($formVars)
    {
        $response = new xajaxResponse();
            
        $companyId = $formVars[FORM_FILTER_COMPANY];
        // first remove all existing options
        $sScript = "var sel = YAHOO.util.Dom.get('".FORM_FILTER_GROUP."');";
        $sScript .= "sel.options.length = 0;";

        // now get all new options
        $groups = nx_get_groups_in_company($companyId);
        $sScript .= "sel.options[0] = new Option('All Groups', '0');";
        $index = 1;
        foreach ($groups as $group)
        {
            $sScript .= "sel.options[$index] = new Option('{$group[PROP_NAME]}', '{$group[PROP_ID]}');";
            $index++;
        }
        $response->script($sScript);
                
        $response->printOutput();
    }

    public function addAjaxError (&$response, $err)
    {
        $msg = '<div class="error">'.$err.'</div>';
        $response->assign('errors','innerHTML',$msg);            
    }

    public function addAjaxMessage(&$response, $msg)
    {
        $all = '<div class="message">'.$msg.'</div>';
        $response->assign('messages','innerHTML',$all);            
    }

    public function emptyTrash ()
    {
        $response = new xajaxResponse();

        if (nx_empty_trash())
        {
            $this->addAjaxMessage ($response, str('STR_AJAX_TRASH_EMPTIED'));
        }
        else
        {
            $this->addAjaxError ($response, str('STR_AJAX_TRASH_EMPTYFAIL'));    
        }
        $response->printOutput();
    }

    public function addSingleDocToBin ($docid)
    {
        $response = new xajaxResponse();
        
        if (nx_add_document_to_briefcase($docid))
        {
            $this->addAjaxMessage($response, str('STR_AJAX_ADDBRIEFCASE'));
            
            // Remove the add to doc link (if it's there)
            $addtodoc = 'docbin_add_'.$docid;
            $response->remove($addtodoc);
            
            // Update the count
            $response->assign('docbin_count','innerHTML',shItemBin::getCount());        
        }
        else 
        {
            $this->addAjaxError($response,str('STR_AJAX_ADDBRIEFCASE_FAIL'));
        }    
        
        $response->printOutput();
    }

    public function removeAttributeFromDocument ($docid, $attrib, $updateTagList)
    {
        $response = new xajaxResponse();
        
        if (nx_remove_document_tag($docid, $attrib))
        {
            // Now update the page with the new tag(s)    
            $tags = new shAttributeCollection();
            $tags->loadDocumentTags($docid);
            $html = $tags->getTagsAsHtml($docid);
                    
            $response->assign ('documentTags', 'innerHTML', $html);
        }
        else
        {    
            $this->addAjaxError($response,"Unable to remove tag $attrib from document");        
        }
        
        $response->printOutput();
    }

    public function addAttributeToDocument ($docid, $attrib, $updateTagList)
    {
        $response = new xajaxResponse();
        if (nx_add_tag_list_to_document($docid,$attrib))
        {
            // Now update the page with the new tag(s)    
            $tags = new shAttributeCollection();
            $tags->loadDocumentTags($docid);
            $html = $tags->getTagsAsHtml($docid);
                    
            $response->assign ('documentTags', 'innerHTML', $html);        
        }
        else
        {
            $this->addAjaxError($response,"Unable to add the given tags to the document");
        }
        
        $response->printOutput();
    }

    public function typePrefixToNexusType($prefix)
    {
        switch ($prefix)
        {
            case TYPE_PREFIX_DOC:
                return OBJECT_TYPE_ITEM;
            case TYPE_PREFIX_CAT:
                return OBJECT_TYPE_CATEGORY;
            default:
                return OBJECT_TYPE_UNKNOWN;
        }
    }

    public function addCatToCategory ($sourceCatId, $catId, $removeFromCatId)
    {
        $response = new xajaxResponse();
        if (nx_set_category_parent($catId,$sourceCatId))
        {                
            // Hide the item (since it has been moved)
            $response->script("onItemRemovedFromView('cat_$sourceCatId')");        
        }
        else
        {
            $this->addAjaxError ($response, sprintf(str('STR_AJAX_CATMOVE_FAILED'), $sourceCatId, $catId));
        }    
        $response->printOutput();
    }

    public function addDocToCategory ($docId, $catId, $removeFromCatId)
    {
        $response = new xajaxResponse();   
        if (nx_add_document_category($docId, $catId, $removeFromCatId))
        {        
            // Hide the item (since it has been moved)
            $response->script("onItemRemovedFromView('doc_$docId')");        
        }
        else
        {
            $this->addAjaxError ($response, str('STR_AJAX_DOCMOVE_FAILED'));
        }    
        $response->printOutput();
    }

    /**
     * Sends the given items to the trash. The array contains 
     * strings of the following form:
     *
     * @param unknown_type $arrayOfIds
     * @param unknown_type $type
     * @return unknown
     */
    public function addItemsToTrash($arrayOfIds)
    {
        $response = new xajaxResponse();    
        $countAdded = 0;
        $countAlreadyThere = 0;
        if (is_array ($arrayOfIds))
        {
            $count = 0;
            foreach ($arrayOfIds as $idAndType)
            {
                list($type,$id) = explode('_',$idAndType);
                $type = $this->typePrefixToNexusType($type);
                
                if (nx_move_to_trash($id,$type))
                {
                    // Hide the item mared as 'doc'(if it's there)
                    $response->script("onItemSentToTrash('$idAndType')");
                    
                    $count++;
                }        
            }
            
            // Update the count
            $trash = new shTrashCollection;
            $response->assign('trash_count','innerHTML',$trash->getCount());                                                
            
            $this->addAjaxMessage ($response, sprintf (str('STR_AJAX_TRASHED'), $count));                
            
        }    
        $response->printOutput();
        
    }

    public function addToDocumentBin ($formVars)
    {
        $response = new xajaxResponse();
        
        // Adding items to bin
        $itemIds = $formVars['shItemDocument'];
        $countAdded = 0;
        $countAlreadyThere = 0;
        if (is_array ($itemIds))
        {
            foreach ($itemIds as $id)
            {
                if (nx_add_document_to_briefcase($id))
                {
                    // Remove the add to doc link (if it's there)
                    $addtodoc = 'docbin_add_'.$id;
                    $response->remove($addtodoc);
                    
                    $countAdded++;
                }
                else 
                {
                    $countAlreadyThere++;
                }
            }
        }    
        else 
        {
            $this->addAjaxMessage($response,str('STR_AJAX_BRIEFCASE_NOSEL'));
        }
        
        if ($countAdded > 0)
        {
            $this->addAjaxMessage($response,sprintf(str('STR_AJAX_BRIEFCASE_ADDED'), $countAdded));
            
            // Update the count
            $response->assign('docbin_count','innerHTML',shItemBin::getCount());
        }
        else 
        {
            $response->assign('messages','innerHTML','');
        }
        
        if ($countAlreadyThere > 0)
        {
            $this->addAjaxError($response,sprintf(str('STR_AJAX_BRIEFCASE_EXIST'),$countAlreadyThere));
        }    
        else 
        {
            $response->assign('errors','innerHTML','');
        }
        
        $response->printOutput();
    }

    public function registerForCategoryUpdateNotification ($catId)
    {
        $response = new xajaxResponse();
        
        if (nx_register_category_notification($catId, NOTIFY_CHANGE_TYPE_UPDATE))
        {        
            $this->addAjaxMessage ($response, str('STR_AJAX_NOTIFY_UPDATED'));                    
        }        
        else
        {
            $this->addAjaxError ($response, str('STR_AJAX_NOTIFY_UPDATEFAIL'));        
        }
        
        $response->printOutput();
    }

    public function registerForCategoryAddNotification ($catId)
    {
        $response = new xajaxResponse();

        if (nx_register_category_notification($catId, NOTIFY_CHANGE_TYPE_ADD))
        {        
            $this->addAjaxMessage ($response, str('STR_AJAX_NOTIFY_UPDATED'));                    
        }        
        else
        {
            $this->addAjaxError ($response, str('STR_AJAX_NOTIFY_UPDATEFAIL'));        
        }
        
        $response->printOutput();
    }

    public function submitForm($successCallback,$formData)
    {
        $response = new xajaxResponse();
        
        $page = new shPage;
        $page->setFormArguments($formData);
        $_POST = $formData;
        $page->setArguments($_GET);
        
        if ($page->authorize())
        {            
            $form = $page->getForm();
            if ($form)
            {        
                $jsonEncodedData = '';    
                $form->submitFormAsync();
                $results = $form->getResults();
                $json = new Services_JSON();
                $jsonEncodedData = $json->encode($results);
                            
                $messages = app()->getNotifications();
                foreach ($messages as $message)
                {
                    $this->addAjaxMessage ($response, $message);
                }
                
                $errors = app()->getErrors();
                foreach ($errors as $error)
                {
                    if ($error instanceof iwError)
                    {
                        $msg = $error->getMessage();
                    }
                    else
                    {
                        $msg = $error;
                    }
                    
                    $this->addAjaxError ($response, $msg);
                }
                
                // This will prevent the messages from appearing the next time a nexus
                //    page is displayed.  Unnecessary since xajax will insert the message
                //    in the current page.
                app()->clearErrors();
                app()->clearNotifications();
                
                $response->script("if (window.$successCallback) $successCallback('$jsonEncodedData');");
            }
            else
            {
                $this->addAjaxError($response, "The given form is not valid");
            }
        }
        else
        {
            $this->addAjaxError($response, str('STR_NOPERMISSION'));
        }    
        
        $response->printOutput();
    }

	public function deleteConversationPost ($postId)
	{
		$response = new xajaxResponse();
		if (nx_delete_conversation_post($postId))
		{
			$response->script('onPostDeleted()');
		}
		else
		{
			$this->addAjaxError($response, str('STR_POST_DELETE_FAILED'));
		}
		
		$response->printOutput();
	}
	
    public function populateConversationList ($containerDivId, $documentId)
    {
        $conversationAsArray = nx_get_conversation($documentId);
            
        $smarty = app()->getSmartyObject();
        $smarty->assign('conversations',$conversationAsArray);
        $html = smarty_templ_fetch('controls/control.view.comments.tpl.html',$smarty);

        $response = new xajaxResponse();
        $response->assign($containerDivId, 'innerHTML', $html);

        $response->printOutput();
    }

    public function updateBrowseAreaContents ($categoryId, $containerId)
    {
        $response = new xajaxResponse();
        $page = new shPage;
        
        $category = new shCategory;
        $category->set(PROP_ID, $categoryId);
        if ($category->load())
        {
            $library = $category->generateBrowserData();

            $smarty = app()->getSmartyObject();
            $smarty->assign ('library', $library);        
            $smarty->assign ('containerId', $containerId);
        
            // Clear out the area first in preparation for new data.
            $response->assign($containerId, 'innerHTML', '');
            $js = 'gNexusObjectRegistry.clear();';
            $js = $smarty->fetch ('scripts/control.library.browser.tpl.js');
            $response->script($js);
        }
        else
        {
            $this->addAjaxError($response,str('STR_AJAX_REFRESH_FAILED'));
        }
        
        $response->printOutput();
    }

    public function addCategoryToBriefcase($categoryId)
    {
        $response = new xajaxResponse();
        
        $count = nx_add_documents_in_category_to_briefcase($categoryId);
        if ($count > 0)
        {
            $this->addAjaxMessage($response, sprintf(str('STR_AJAX_BRIEFCASE_ADDED'),$count));        
            $response->assign('docbin_count','innerHTML',nx_get_number_of_docs_in_briefcase());
        }
        else
        {
            $this->addAjaxError($response,str('STR_AJAX_BC_CATNOTFOUND'));
        }
        
        $response->printOutput();
    }

    public function renameCategory($catId, $newName)
    {
        $response = new xajaxResponse();
        if (nx_rename_category($catId,$newName))
        {
            $response->assign('cat_'.$catId.'_name','innerHTML',$newName);
            $response->script("onItemRenamed('".TYPE_PREFIX_CAT."','$catId','$newName')");
        }
        else
        {
            $this->addAjaxError($response,str('STR_AJAX_RENAME_CAT_FAILED'));
        }
            
        $response->printOutput();
    }

    function renameDocument($docId, $newName)
    {
        $response = new xajaxResponse();
        if (nx_rename_document($docId,$newName))
        {
            $response->assign(TYPE_PREFIX_DOC.'_'.$docId.'_name','innerHTML',$newName);
            $response->script("onItemRenamed('".TYPE_PREFIX_DOC."','$docId','$newName')");
        }
        else
        {
            $this->addAjaxError($response,str('STR_AJAX_RENAME_DOC_FAILED'));
        }
        
        
        $response->printOutput();
    }

    /**
    * @desc Gets a list of users and calls a callback with the 
    * results as a json encoded string.  User specifies the callback
    * @param string $callback The callback to call when successful.
    */
    public function getUserList($callback)
    {
        $response = new xajaxResponse();
        
        if (nx_is_user_logged_in())
        {
            $response = new xajaxResponse();
            $users = nx_get_usernames();
            
            $json = new Services_JSON();
            $jsonUsers = $json->encode($users);
            
            $response->script("$callback('$jsonUsers')");
        }
        
        $response->printOutput();
    }

    public function getDocumentList($containerId, $selectedCallbackString)
    {
        $response = new xajaxResponse();

        $docs = nx_get_documents();
        
        $list = '';
        $list .= '<div class="asyncList">';
        foreach ($docs as $item)
        {
            $callbackEval = $selectedCallbackString.'(\''.$item[PROP_ID].'\')';
            $list .= '
                <div class="asyncItem">
                    <a href="javascript:void(0);" onclick="javascript:'.$callbackEval.'">'.$item[PROP_NAME].' ('.$item[PROP_ID].')
                    <span class="asyncDescription">'.$item[PROP_DESCRIPTION].'</span>
                    </a>                
                </div>';
        }
        $list .= '</div>';
        
        $response->assign($containerId, 'innerHTML', $list);

        $response->printOutput();
    }

    /**
    * @desc Attempts to share the given document with the 
    * given user.  If successful, the javascript function 
    * onFileBoxItemShared is called, otherwise, an error 
    * message is displayed.
    * @param int $docId The id of the filebox document
    * @param string $username The username of the user to share with
    */
    public function shareFileBoxDocument($docId, $username)
    {    
        $response = new xajaxResponse();
        if (nx_share_filebox_document($docId,$username))
        {
            $response->script("onFileBoxItemShared($docId,'$username');");
        }
        else
        {
            $this->addAjaxError($response,str('STR_AJAX_SHARE_DOC_FAILED'));        
        }
        
        $response->printOutput();
    }
        
    /**
    * @desc Attempts to unshare the given document from the 
    * given user.  If successful, the javascript function 
    * onFileBoxItemUnshared is called, otherwise, an error 
    * message is displayed.
    * @param int $docId The id of the filebox document
    * @param string $username The username of the user to unshare from
    */
    public function unshareFileBoxDocument($username,$docId)
    {    
        $response = new xajaxResponse();
        if (nx_unshare_filebox_document($docId,$username))
        {
            $response->script("onFileBoxItemUnshared($docId,'$username');");
        }                         
        else
        {
            $this->addAjaxError($response,str('STR_AJAX_UNSHARE_DOC_FAILED'));        
        }
        
        $response->printOutput();
    }    
}
?>
