<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


/**
 * Use this utility class to check the database for authorization
 * 	across all protectable items.  It also has methods that allow for
 * 	quick access to related entities.  For example, to find out which
 *	groups are in a given company, simply call getCompanyGroups.
 */
class shAuthorize
{			
	/**
	 * Determines whether or not the given user can delete the pool
	 *	item given.  Admin user can delete anything.
	 * @param string $username The name of the user requesting to delete
	 * @param integer $itemid The ID of the item that is to be deleted.
	 * @return bool Returns true if the pool item can be deleted, false otherwise.
	 */
	function canDeletePoolItem ($username, $itemid)
	{
				
		if (app()->isAdminLoggedIn())
		{
			// Admin can delete anything.
			return true;
		}		
		
		if (!$username)
		{			
			// If not admin, must specify username
			return false;
		}
		
		$query = 'SELECT COUNT(item_id) FROM sh_item_pool WHERE username="'.$username.'" AND item_id='.$itemid;
		$res1 = app()->getDatabase()->GetAll($query);
		
		if ($res1 == false)
		{
			return false;
		}
		else 
		{
			// this item does belong to this user which means he or she
			//	can delete ite themselves.
			return (count($res1) > 0);
		}
	}
	
     
    /**
     * Determines whether or not the given user can download the pool
     *    item given.  Admin user can download anything.
     * @param string $username The name of the user requesting to download
     * @param integer $itemid The ID of the item that is to be downloaded.
     * @return bool Returns true if the pool item can be downloaded, false otherwise.
     */
    function canDownloadPoolItem ($username, $itemid)
    {
        if (app()->isAdminLoggedIn())
        {
            // Admin can delete anything.
            return true;
        }        
        
        if ($username == '')
        {            
            // If not admin, must specify username
            return false;
        }
        
        $poolItem = new shPoolItem;
        if ($poolItem->loadId($itemid))
        {
            if (($poolItem->get(PROP_USERNAME) == app()->getLoggedInUsername()) ||
                $poolItem->isSharedWithUser(app()->getLoggedInUsername()))
            {
                return true;
            }
        }
        
        return false;
    }
    
    
    /**
    * @desc Determines if the given company/group (domain) can view
    *       the given category. If the group id is 0 then this just
    *       checks to see if the company is in permissions list.
    * @param int $catid The Id of the category we're trying to view.
    * @param int $cid The Id of the copmany
    * @param int $gid The Id of the group (or 0 to ignore)
    * @param bool $includeEveryoneClass If this is set to false then 
    *               the function will return false even if the "Everyone" 
    *               classification is specified in the list of permissions for
    *               the document but the specific company/group given is
    *               not.
    * @return bool Returns true if the category can be viewed, false otherwise.
    */
    static public function canDomainViewCategory($catid,$cid,$gid,$includeEveryoneClass=true)
    {
        // now see if we can find a matching company/group for that category.  A value of
        //     0 (zero) in the database for a company/group id means that it will work with any 
        //    group or any company.  For example, company_id = 10 and group_id = 0 means that 
        //    all groups in company 10 have access. When both are 0, that means that everyone
        //    has access to that item. 
        
        $query = '';
        if ($gid > 0)
        {
            $query = "SELECT COUNT(*) AS _count FROM sh_category_access_map WHERE " .
                    "(group_id=$gid".($includeEveryoneClass?" OR group_id=0":"").") AND ".
                    "(company_id=$cid".($includeEveryoneClass?" OR company_id=0":"").") AND " .
                    "category_id=$catid";
        }
        else
        {
            $query = "SELECT COUNT(*) AS _count FROM sh_category_access_map WHERE " .
                    "(company_id=$cid".($includeEveryoneClass?" OR company_id=0":"").") AND " .
                    "category_id=$catid";
        }
             
        app()->flushCacheIfInvalid($query);                   
        $res2 = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
        if ($res2 !== false)
        {
            $found = $res2[0]['_count'];
            return ($found > 0);
        }
        else
        {
            app()->addErrorObject ($res2,ERROR_TYPE_SERIOUS);
        }        
        
        return false;
    }	
    /**
    * @desc Determines if the given company/group (domain) can view
    *       the given document. If the group id is 0 then this just
    *       checks to see if the company is in permissions list.
    * @param int $itemid The Id of the item we're trying to view.
    * @param int $cid The Id of the copmany
    * @param int $gid The Id of the group (0 to ignore)
    * @return bool Returns true if the document can be viewed, false otherwise.
    */
    static public function canDomainViewDocument($itemid,$cid,$gid)
    {
        // now see if we can find a matching company/group for that item.  A value of
        //     0 (zero) in the database for a company/group id means that it will work with any 
        //    group or any company.  For example, company_id = 10 and group_id = 0 means that 
        //    all groups in company 10 have access. When both are 0, that means that everyone
        //    has access to that item. 
        
        $doc = new shItemDocument();
        if ($doc->loadId($itemid))
        {
        	return $doc->doesDomainHaveAccess($cid,$gid);
        }        
        
        return false;
    }
    
    static public function canAddCategoriesToCategory($username,$categoryId)
    {
        $user = null;
        if (is_object($username))
        {
            $user = $username;
        }
        else
        {
    	    $user = new shUser;
    	    $user->set(PROP_USERNAME,$username);
    	    if (!$user->load($username))
    	    {
    		    return false;
    	    }
        }
    	    	
    	$pgroup = $user->getPermissions()->getPermissionGroup();
    	if ($pgroup == USER_TYPE_ADMIN)
    	{
    		return true;
    	}
    	if ($pgroup == USER_TYPE_DOMAIN_GROUP_ADMIN)
    	{
    		// if you can edit the category then you can add new categories to it.
    		return $user->getPermissions()->isAllowed(URLACTION_MODCAT, $categoryId);
    	}
    	else if ($pgroup == USER_TYPE_DOMAIN_ADMIN)
    	{
    		return shAuthorize::canDomainViewCategory(
    			$categoryId,
    			$user->getCompany()->get(PROP_ID),
    			$user->getGroup()->get(PROP_ID));
    	}
    	
    	return false;
    }
    
    static public function canAddDocumentsToCategory($username,$categoryId)
    {
        $user = null;
        if (is_object($username))
        {
            $user = $username;
        }
        else
        {
            $user = new shUser;
            $user->set(PROP_USERNAME,$username);
            if (!$user->load($username))
            {
                return false;
            }
        }
    
        $pgroup = $user->getPermissions()->getPermissionGroup();
    	if ($pgroup == USER_TYPE_ADMIN)
    	{
    		return true;
    	}
    	if ($pgroup == USER_TYPE_DOMAIN_GROUP_ADMIN)
    	{
    		return $user->getPermissions()->isAllowed(URLACTION_MODCAT, $categoryId);
    	}
    	else if ($pgroup == USER_TYPE_DOMAIN_ADMIN)
    	{
    		return shAuthorize::canDomainViewCategory(
    			$categoryId,
    			$user->getCompany()->get(PROP_ID),
    			$user->getGroup()->get(PROP_ID));
    	}    	
    	
    	return false;
    }
    
    /**
     * Determines if the given user can edit the given category
     * @param string $username The name of the user to check
     * @param int $catid The ID of the category to check for access
     * @return bool Returns true if editing is allowed, false otherwise. 
     */
    static public function canEditCategory($username,$catid)
    {
        $user = null;
        if (is_object($username))
        {
            $user = $username;
        }
        else
        {
            $user = new shUser;
            $user->set(PROP_USERNAME,$username);
            if (!$user->load($username))
            {
                return false;
            }
        }
    	
    	$permissions = $user->getPermissions();    	
    	if ($permissions->allowGlobalEditCategory)
    	{
    		return true;
    	}
    	else
    	{    		  
   			$userCompanyId = $user->getCompany()->get(PROP_ID);
   			$userGroupId = $user->getGroup()->get(PROP_ID);
   			$cat = new shCategory;
   			$cat->set(PROP_ID,$catid);
   		
   			if ($cat->load())
   			{
   				$permGroup = $permissions->getPermissionGroup();
	   			if ($permGroup == USER_TYPE_DOMAIN_ADMIN)
	    		{
    				if ($cat->doesCompanyHaveAccess($userCompanyId))
    				{
    					return true;
    				}
	    		}
	    		
	   			if ($permGroup == USER_TYPE_DOMAIN_GROUP_ADMIN)
	    		{	    			
	    			if ($cat->doesDomainHaveAccess($userCompanyId,$userGroupId))
    				{
    					return true;
    				}
	    		}
    		}
    	}
    	
    	return false;
    }
  
    /**
     * Determines if the given user can edit the given document
     * @param string $username The name of the user to check
     * @param int $itemid The ID of the document to check for access
     * @return bool Returns true if editing is allowed, false otherwise. 
     */
    static public function canEditDocument($username,$itemid)
    {
    	$user = app()->getLoggedInUserObject();
    	if (!$user)
    	{
    		return false;
    	}
    	
    	$permissions = $user->getPermissions();
    	if ($permissions->allowGlobalEditDocument)
    	{
    		return true;
    	}
    	else
    	{
   			$userCompanyId = $user->getCompany()->get(PROP_ID);
   			$userGroupId = $user->getGroup()->get(PROP_ID);
   			$doc = new shItemDocument;
   			$doc->set(PROP_ID,$itemid);
   		
   			if ($doc->load())
   			{
	   			if ($permissions->allowDomainEditDocument)
	    		{
    				if ($doc->doesCompanyHaveAccess($userCompanyId))
    				{
    					return true;
    				}
	    		}
	   			if ($permissions->allowDomainGroupEditDocument)
	    		{
    				if ($doc->doesDomainHaveAccess($userCompanyId,$userGroupId))
    				{
    					return true;
    				}
	    		}
    		}
    	}
    	
    	return false;
    }

	/**
	 * Determines whether or not the given user can view the given item
	 * @param string $username The username of the user to check against.
	 * @param int $itemid The ID of the item attempting to be viewed.
	 * @return bool Returns true if the item can be viewed, false otherwise.
	 */
	static public function canViewCategory ($username, $catid)
	{
				
		if (app()->isAdminLoggedIn() || $catid == 0)
		{
			// Admin can view anything and any registered user can view the 
			//	root of the library.
			return true;
		}
		
		$res1 = false;
        $user = null;
		if ($username == '')
		{
			// Not a logged in user so simulate a non-registered
			//	user object bysetting res1 to the company id and 
			//	group id of the non-registered user.
			$res1 = array(array('company_id'=>COMPANY_UNREGISTERED,'group_id'=>GROUP_ALL));
		}
		else
		{
			$query = 'SELECT group_id,company_id FROM sh_user_access_map WHERE username="'.$username.'"';
            app()->flushCacheIfInvalid($query);
			$res1 = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
            
            $user = new shUser();
            $user->loadId($username);
		}
						
		if ($res1 !== false)
		{	           
			if (count($res1) == 1)
			{		                
				$cid = $res1[0]['company_id'];
				$gid = $res1[0]['group_id'];

                if ($user)
                {
                    $permGroup = $user->getPermissions()->getPermissionGroup();
                    if ($permGroup == USER_TYPE_DOMAIN_ADMIN)
                    {
                        // Domain admins need not worry about the group, just the org.
                        return shAuthorize::canDomainViewCategory($catid,$cid,0,true);
                    }
                }
                				
                // Use the org id and group id to determine if the category is permitted.
                return shAuthorize::canDomainViewCategory($catid,$cid,$gid,true);
			}
			else
			{
				app()->addError (str('STR_ERR_VALIDATE_USER'),ERROR_TYPE_SERIOUS);
			}
		}
		else
		{
			app()->addErrorObject ($res1,ERROR_TYPE_SERIOUS);
		}
		
		return false;
	}
	
	/**
	 * Determines whether or not the given user can view the given item
	 * @param string $username The username of the user to check against.
	 * @param int $itemid The ID of the item attempting to be viewed.
	 * @return bool Returns true if the item can be viewed, false otherwise.
	 */
	static public function canViewDocument ($username, $itemid)
	{        		
		if (app()->isAdminLoggedIn())
		{
			// Admin can view anything.
			return true;
		}
		
		$res1 = false;
		if ($username == '')
		{
			
			// Not a logged in user so simulate a non-registered
			//	user object bysetting res1 to the company id and 
			//	group id of the non-registered user.
			$res1 = array(array('company_id'=>COMPANY_UNREGISTERED,'group_id'=>GROUP_ALL));
		}
		else
		{
			$query = 'SELECT group_id,company_id FROM sh_user_access_map WHERE username="'.$username.'"';
            app()->flushCacheIfInvalid($query);
			$res1 = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		}
			
		if ($res1 !== false)
		{	
			if (count($res1) == 1)
			{		
				$cid = $res1[0]['company_id'];
				$gid = $res1[0]['group_id'];
				
                return shAuthorize::canDomainViewDocument($itemid,$cid,$gid,true);
			}
			else
			{
				app()->addError (str('STR_ERR_VALIDATE_USER'),ERROR_TYPE_SERIOUS);
			}
		}
		else
		{
			app()->addErrorObject ($res1,ERROR_TYPE_SERIOUS);
		}
		
		return false;
	}
}
?>
