<?php 
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shFileInfo
{
	/** internal types (such as DOCTYPE_IMAGE or DOCTYPE_WORD )*/
	public $type;
	
	/** The name of the file (no extension, no path) */
	public $name;
	
	/** The extension of the file (if any) */
	public $extension;
	
	/** The path to the file (no filename) */
	public $path;
	
	/** The size of the file in bytes */
	public $size;
	
	/** The last time the file was modified (as unix timestamp) */
	public $lastModifiedTime;

	/** The creation time for the file (as unix timestamp) */
	public $creationTime;	

	/** The last time the file was accessed (as unix timestamp) */
	public $lastAccessTime;
}

/**
 * This class provides a convenient way for analyzing a set of documents on disk.
 */
class shFileAnalysis
{	
	/** The array of shFileInfo objects generated during analysis */
	public $files = array();
	
	/** The total size (in bytes) of all the files that were analyzed */
	public $totalSizeInBytes = 0;
	
	/** The breakdown of file types in an associative array keyed on the filetype */
	public $fileTypes = array();
	
	/**
	 * Read information about the files associated with the given document
	 * collection.  
	 *	@param iwObjectCollection $documentCollection The collection
	 *			 of documents to analyze
	 *	@return bool Returns true if the analysis was successful, false otherwise.
	 */
	public function analyze(iwObjectCollection $documentCollection)
	{
		if ($documentCollection == null)
		{
			return false;
		}
		
		$objects = &$documentCollection->items();
		foreach ($objects as $ob)
		{
			if ($ob instanceof shItemDocument)
			{
                $fileInfo = storage()->getDocumentInfo($ob);               
				if ($fileInfo)
				{				
					$this->files[] = &$fileInfo;
					$this->totalSizeInBytes += $fileInfo->size;
                    
                    if ($fileInfo->type == null)
                    {
                        $fileInfo->type = new shFileTypeBinary();
                    }
                    
                    if (!isset ($this->fileTypes[$fileInfo->type]))
                    {                        
                        $this->fileTypes[$fileInfo->type] = 0;
                    }
                    $this->fileTypes[$fileInfo->type] += 1;
				}
			}
		}		
		return true;
	} 
		
	/**
	 * Determines if the contents of the library plus the given byte count
	 * 	adds up to more than the maximum allowable storage capacity as setup in the
	 *	settings.
	 *	@param int $toFitInBytes The number of bytes to fit with the contents of this collection
	 *	@return bool Returns true if there is space, false otherwise.
	 */ 
	static public function isFileBoxSpaceAvailable ($toFitInBytes, $maxSpaceInMegaBytes=MAX_STORAGE_IN_MB)
	{
		if (!app()->isAdminLoggedIn())
		{		
			$docs = new shPoolItemCollection;
			$docs->loadUserPool(false);

			$a = new shFileAnalysis;
			$a->analyze ($docs);
			if (($a->totalSizeInBytes + $toFitInBytes) > ($maxSpaceInMegaBytes*1024)*1024)
			{
				return false;
			}
		}
		return true;
	}
	/**
	 * Determines if the contents of the library plus the given byte count
	 * 	adds up to more than the maximum allowable storage capacity as setup in the
	 *	settings.
	 *	@param int $toFitInBytes The number of bytes to fit with the contents of this collection
	 *	@return bool Returns true if there is space, false otherwise.
	 */ 
	static public function isLibrarySpaceAvailable ($toFitInBytes, $maxSpaceInMegaBytes=MAX_STORAGE_IN_MB)
	{
		$docs = new shItemDocumentCollection;
		$docs->load (new iwParameterCollection);
		$a = new shFileAnalysis;
		$a->analyze ($docs);
		if (($a->totalSizeInBytes + $toFitInBytes) > ($maxSpaceInMegaBytes*1024)*1024)
		{
			return false;
		}

		return true;
	}
}

?>
