<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


/**
 * The item bin (also known as the Briefcase) is used 
 *	as a convenient place to store references to items 
 *	for the life of the browser session.  You can do things
 *	like download all at once, delete all, edit all, etc.  The
 *	item bin is simply an array of shItem-based objects.  Clearing
 *	out the bin is simply a matter of passing an empty array to
 *	the @c setItemBin method.
 */
class shItemBin 
{
	/**
	 * This stores the item id array given in the php session.  This can 
	 *	be an empty array which means that the 
	 * @param shItemBin The bin object.
	 */
	public function setItemBin ($bin)
	{
		$_SESSION['__itembin'] = $bin;
	}
	
	/**
	 * Retrieves the currently stored array of item ids from the 
	 * 	session variable.  If there is no session variable yet, one
	 *	is created then returned.
	 *	@return array The array of shItem-derived objects.
	 */
	static public function getItemBin ()
	{
		if (!isset ($_SESSION['__itembin']) || !is_array($_SESSION['__itembin']))
		{
			$_SESSION['__itembin'] = array ();
		}
		
		return $_SESSION['__itembin'];
	}
	
	/**
	 * Gets the number of ids in the item bin so far.
	 *	@param int The number of items in the bin.
	 */
	static public function getCount()
	{
		return count(shItemBin::getItemBin());
	}
		
	function createZip ()
	{
		// Download item bin
		$username = app()->getLoggedInUserObject()->get(PROP_USERNAME);
		$zipfile = 'cache/documents_'.$username.'.zip';	
		$zip = new Archive_Zip($zipfile);
		
		$files = array ();
		// Copy all the relevant documents to a temp location
		foreach (shItemBin::getItemBin() as $itemId)
		{
			$item = new shItemDocument;
			$item->set (PROP_ID, $itemId);
			if ($item->load())
			{
				// Copy to temporary location
                $path = storage()->retrieveDocument($item);
				$dest = 'cache/temp/'.$item->get(PROP_ID).'_'.$item->get(PROP_ORIGINAL_FILENAME);
				if (copy ($path,$dest))
				{
					$files[] = $dest;								
				}	
                
                storage()->removeTempDocument($item,$path);
			}
		}
		
		$zip->create ($files,array('remove_all_path'=>true));
		foreach ($files as $file)
		{						
			// Delete temporary files.
			unlink ($file);	
		}	

		return $zipfile;
	}
	
	/**
	 * Email the contents of the item bin as a zip file to the given
	 * 	email destination.  The from address will be the administrator's
	 *	address.
	 */
	function emailBin ($email)
	{
		$user = app()->getLoggedInUserObject();
		$username = $user->get(PROP_EMAIL);
		
		$zipfile = shItemBin::createZip();
		if ($zipfile != '')
		{		
			
			$subject = str('STR_EMAIL_SUBJ_BRIEFCASE',$user->getFullName());
			$msg = str('STR_EMAIL_BODY_BRIEFCASE_HDR',settings()->company_name);
			
			foreach (shItemBin::getItemBin() as $itemId)
			{
				$item = new shItemDocument;
				$item->set (PROP_ID, $itemId);
				if ($item->load())
				{			
					$msg .= str('STR_EMAIL_BODY_BRIEFCASE_CONTENT',
						$item->get(PROP_NAME),$item->get(PROP_ORIGINAL_FILENAME),
						$item->get(PROP_DESCRIPTION));
				}
			}			
			return app()->sendMail($username, $email, $subject, $msg, array($zipfile));
		}

        return false;
	}
	
	/**
	 * Send a zip file directly to the browser which contains
	 *	all the documents in the bin file.
	 */
	function downloadZip ()
	{
		$zipfile = shItemBin::createZip();
		if ($zipfile == '')
		{
			$this->addError (str('STR_ZIP_CREATEFAILED'), ERROR_TYPE_SERIOUS);
		}
		
		header('Content-Description: File Transfer');
		header('Content-Type: application/force-download');
		header('Content-Length: ' . filesize($zipfile));
		header('Content-Disposition: attachment; filename=' . basename($zipfile));
		echo readfile ($zipfile);
		
		unlink ($zipfile);
		
		exit ();
	}
	
	/**
	 * Determines if the item with the given ID can be found 
	 *	in the bin.
	 *	@return bool Returns true if the item is in the bin, false otherwise
	 */
	function isItemInBin ($idToFind)
	{
		$bin = shItemBin::getItemBin();
		foreach ($bin as $id)
		{		
			if ($id == $idToFind)
			{
				
				return true;
			}
		}		
		
		return false;
	}

	/**
	 * Removes the item with the given id from the bin
	 *	@param int $toFind The item to search for.
	 */
	function removeItemFromBin ($toFind)
	{
		$final = array();
		$bin = shItemBin::getItemBin();
		foreach ($bin as $id)
		{
			if ($id != $toFind)
			{
				$final[] = $id;
			}
		}
		
		shItemBin::setItemBin ($final);
	}
	
	/**
	 * Clears out all items in the item bin.
	 */
	function clearItemBin ()
	{
		shItemBin::setItemBin(array());
	}
	
	/**
	 * Adds the item with the given id to the bin
	 *	@param int $id The id of the item
	 */
	function addItemToBin ($id)
	{
		$bin = shItemBin::getItemBin();
		if (!shItemBin::isItemInBin($id))
		{
			$bin[] = $id;
		}
		
		shItemBin::setItemBin ($bin);
	}	
}

?>
