<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shMultiSubmitForm extends iwForm
{
    protected $redirectLink = ''; 
    
	function getFormAction()
	{
		// this is not usually used because this form has no
		//	front-end.
		return '';	
	}
	
	function getFormType()
	{
		return FORMTYPE_MULTI;	
	}
		
	/**
	 * This will return an associative array of data that should 
	 * be applied to whatever template system is being used.
	 * @return array An associative array of data
	 */
	function render ()
	{	
		return '';	
	}
		
	/**
	 * Override to implement form submit for this object
	 * @return bool Returns true if the form submission was successful 
	 */
	protected function _submit ()
	{	
		$this->processTrashSubmit();
		$this->processAttributeSubmit();
		$this->processItemBinSubmit();
		$this->processItemPoolSubmit();
		$this->processCategorySubmit();
		$this->processGroupSubmit();
		$this->processCompanySubmit();
		$this->processUserSubmit();
		
		return true;
	}
	
    public function getRedirectLink()
    {
        if ($this->redirectLink != '')
        {
            return $this->redirectLink;
        }
        else
        {
            return '';    
        }
    }
        
	/**
	 * Handles whenever the user presses a button that will change
	 * the status of the session-global item bin. If one of the
	 * submit values is present in the post then it will expect
	 * that a form variable of type array named itembin is present.
	 * If not, this function won't do anything
	 */
	function processTrashSubmit ()
	{		
        $redirectToTrashViewPage = false; 
		if (ps(FORMACTION_HIDDEN) == FORMACTION_DELETESELECTEDPERMANENTLY)
		{
			// Deleting items permanelty
			$itemIds = ps (shTrashCollection::getClassName());
			$actions = new shActions();		
			if (is_array ($itemIds))
			{
				foreach ($itemIds as $idtype)
				{
					list($id,$type) = split(',',$idtype);					
					$actions->doDeleteObject($type,$id,true);
				}
			}
            $redirectToTrashViewPage = true;
		}
		else if (ps (FORMACTION_HIDDEN) == FORMACTION_DELETEALLTRASHPERMANENTLY)
		{
			$coll = new shTrashCollection();
			$coll->emptyTrash();
            $redirectToTrashViewPage = true;
		}
		else if (ps (FORMACTION_HIDDEN) == FORMACTION_RESTORESELECTED)
		{
			// Deleting items permanelty
			$itemIds = ps (shTrashCollection::getClassName());
			if (is_array ($itemIds))
			{
				foreach ($itemIds as $idtype)
				{
					list($id,$type) = split(',',$idtype);
                    
                    if (class_exists($type))
                    {
                        $item = new $type;
                        if ($item->loadId($id))
                        {
                            $item->restore();
                        }
                    }
				}
			}
            $redirectToTrashViewPage = true;
		}
		else if (ps (FORMACTION_HIDDEN) == FORMACTION_RESTOREALLTRASH)
		{
			$coll = new shTrashCollection();
			$coll->restoreTrash();
            $redirectToTrashViewPage = true;
		}                       
        
        if ($redirectToTrashViewPage)
        {
            $this->redirectLink = sprintf('index.php?%s=%s',URLARG_ACTION,URLACTION_VIEWTRASH);        
        }
	}
	
	function processAttributeSubmit()
	{
		if (ps(FORMACTION_DELETEATTRIBUTE) != '')
		{			
			$selected = ps (shAttributeCollection::getClassName());
			if (is_array ($selected))
			{
				foreach ($selected as $id)
				{
					$att = new shAttribute;
					$att->set(PROP_ID, $id);
					$att->delete();
				}
				
				$this->addNotification (sprintf (str('STR_ATTRIB_MULTI_TRASHED'), count($selected)));
			}
		}		               	
	}
		
	function processCategorySubmit()
	{
		if (ps(FORMACTION_DELETECATEGORY) != '')
		{
			$selected = ps (shCategoryCollection::getClassName());
			if (is_array ($selected))
			{
				foreach ($selected as $id)
				{
					$cat = new shCategory;
					$cat->set(PROP_ID, $id);
					$cat->delete();
				}
				
				$this->addNotification (sprintf (str('STR_CAT_MULTI_TRASHED'), count($selected)));
			}
            
            $this->redirectLink = 'index.php';
		}			
	}
	
	function processGroupSubmit()
	{
		if (ps(FORMACTION_DELETEGROUP) != '')
		{
			$selected = ps (shGroupCollection::getClassName());
			if (is_array ($selected))
			{
				foreach ($selected as $id)
				{
					$cat = new shGroup;
					$cat->set(PROP_ID, $id);
					$cat->delete();
				}
				
				$this->addNotification (sprintf (str('STR_GROUP_MULTI_TRASHED'), count($selected)));
			}
            
            $this->redirectLink = 'index.php';
		}			
	}	

	function processCompanySubmit()
	{
		if (ps(FORMACTION_DELETECOMPANY) != '')
		{
			$selected = ps (shCompanyCollection::getClassName());
			if (is_array ($selected))
			{
				foreach ($selected as $id)
				{
					$cat = new shCompany;
					$cat->set(PROP_ID, $id);
					$cat->delete();
				}
				
				$this->addNotification (sprintf (str('STR_COMPANY_MULTI_TRASHED'), count($selected)));
			}
            
            $this->redirectLink = 'index.php';
		}			
	}	
	

	function processUserSubmit()
	{
		if (ps(FORMACTION_DELETEUSER) != '')
		{
			$selected = ps (shUserCollection::getClassName());
			if (is_array ($selected))
			{
				foreach ($selected as $username)
				{
					$cat = new shUser;
					$cat->set(PROP_USERNAME, $username);
					$cat->delete();
				}
				
				$this->addNotification (sprintf (str('STR_USER_MULTI_TRASHED'), count($selected)));
			}
            
            $this->redirectLink = 'index.php';
		}			
	}
		
	/**
	 * Handles whenever the user presses a button that will change
	 * the status of the session-global item bin. If one of the
	 * submit values is present in the post then it will expect
	 * that a form variable of type array named itembin is present.
	 * If not, this function won't do anything
	 */
	function processItemBinSubmit ()
	{                
		if (ps(FORMACTION_ADDBIN) != '')
		{
			// Adding items to bin
			$itemIds = ps (shItemDocumentCollection::getClassName());
			if (is_array ($itemIds))
			{
				foreach ($itemIds as $id)
				{
					if (!shItemBin::isItemInBin($id))
					{
						shItemBin::addItemToBin ($id);
					}
				}
			}
            
            $this->redirectLink = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWBIN;
		}
		else if (ps(FORMACTION_DELETEDOC) != '')
		{           
			$itemids = ps(shItemDocumentCollection::getClassName());
			if (is_array ($itemids))
			{
				foreach ($itemids as $id)
				{
					$item = new shItemDocument;
					$item->set(PROP_ID, $id);
					$item->delete();
				}
				
				$this->addNotification (sprintf (str('STR_DOC_MULTI_TRASHED'), count($itemids)));
				
			}			
            
            $this->redirectLink = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWBIN;
		}
		else if (ps (FORMACTION_REMOVEFROMBIN) != '')
		{
			$itemids = ps (shItemDocumentCollection::getClassName());
			if (is_array($itemids))
			{
				foreach ($itemids as $id)
				{
					shItemBin::removeItemFromBin($id);
				}
			}
            
            $this->redirectLink = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWBIN;
		}
		else if (ps (FORMACTION_EMAILBIN) != '')
		{
			$destination = ps(FORM_EMAIL_DESTINATION);
			if ($destination != '')
			{
				if (shItemBin::emailBin ($destination))
                {
                    app()->addGlobalNotification(str('STR_BRIEFCASE_EMAIL_SENT'));
                }
                else
                {
                    app()->addGlobalNotification(str('STR_BRIEFCASE_EMAIL_NOT_SENT'));
                }
			}
            
            $this->redirectLink = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWBIN;
		}		
		else if (ps (FORMACTION_CLEARBIN) != '')
		{
			shItemBin::clearItemBin();
            $this->redirectLink = 'index.php';
		}
		else if (ps (FORMACTION_DOWNLOADBIN) != '')
		{
		}
		else if (ps (FORMACTION_DELETEBIN) != '')
		{
			foreach (shItemBin::getItemBin() as $id)
			{
				$item = new shItemDocument;
				$item->set(PROP_ID,$id);
				$item->delete();
			}

			shItemBin::clearItemBin();
            
            $this->redirectLink = 'index.php';
		}
		else if (ps (FORMACTION_EDITBIN) != '')
		{
			$form = new shFormMultiDocument();
			$form->setTargetIds(shItemBin::getItemBin());

			if (ps (FORMACTION_EDITBIN) == FORM_SUBMIT)
			{
				// Try submitting the change
				$form->submitForm();
			}
			else
			{
				app()->getActivePage()->addContent($form->render());
			}		
		}
	}	
	
	function processItemPoolSubmit ()
	{
		if (ps('deleteSelectedFromPool') != '')
		{
			// Adding items to bin
			$itemIds = ps ('shPoolItem');
			if (is_array ($itemIds))
			{
				$problems = array();
				foreach ($itemIds as $id)
				{
					$item = new shPoolItem();
					$item->set(PROP_ID, $id);
					if (!$item->load())
					{
						$problems[] = sprintf(str('STR_FILEBOX_NOTFOUND_ID',$id));
					}
					else
					{
						$item->enableTrueDeletion(true);
						if (!$item->delete())
						{
							$problems[] = sprintf(str('STR_FILEBOX_MULTI_DELETE_FAILED',$id,$item->get(PROP_NAME)));
						}
					}
				}
			}
            
            $this->redirectLink = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWPOOL;
		}
		if (ps('addSelectedToLibrary') != '')
		{
			if (app()->isAdminLoggedIn())
			{
				// Adding items to bin
				$itemIds = ps ('shPoolItem');
				if (is_array ($itemIds))
				{
					$a = new shFileAnalysis;
					$coll = new shItemDocumentCollection;
					$coll->load (new iwParameterCollection);
					$a->analyze ($coll);
					$spaceUsed = $a->totalSizeInBytes;
					
					$problems = array();
					foreach ($itemIds as $id)
					{
						$item = new shPoolItem();
						$item->set(PROP_ID, $id);
						if (!$item->load())
						{
							$problems[] = sprintf(str('STR_FILEBOX_NOTFOUND_ID'), $id);
						}
						else
						{
							$size = storage()->getFileSize($item);
							if ($spaceUsed + $size > ((MAX_STORAGE_IN_MB*1024)*1024))
							{
								$problems[] = str('STR_FILEBOX_NOSPACE');
								break;
							}
							
							// Make a library item out of it.
							$lib = new shItemDocument;
							$lib->set(PROP_NAME, $item->get(PROP_NAME));
							$lib->set(PROP_DESCRIPTION, $item->get(PROP_DESCRIPTION));
							$lib->addCompanyGroup(ps(FORM_FILTER_COMPANY),ps(FORM_FILTER_GROUP));
							$lib->addCategory (ps(FORM_FILTER_CATEGORY));
							$lib->set(PROP_STATUS, ITEM_STATUS_ACTIVE);
							$lib->loadDocument(
									typeman()->determineNexusType($item->get(PROP_ORIGINAL_FILENAME)),
									$item->get(PROP_ORIGINAL_FILENAME),
									storage()->getStorageLocation($item));
	
							// Add the item to the database
							if ($lib->add ())
							{
								$lib->storeFileToFinalLocation(false);
							}
							else
							{
								$problems[] = sprintf(str('STR_FILEBOX_MULTI_ADD_FAILED'),$item->get(PROP_NAME));
							}
						}
					}
	
					if (count($problems) > 0)
					{
						$this->addError (sprintf(str('STR_MULTIDOC_COPYLIBFAILED'),join('<br>',$problems)), ERROR_TYPE_SERIOUS);
					}
					else
					{
						$this->addNotification (str('STR_MULTIDOC_COPYLIB'), ERROR_TYPE_SERIOUS);
					}
				}
				else
				{
					$this->addError(str('STR_NOSEL'), ERROR_TYPE_WARNING);
				}
                
                $this->redirectLink = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWPOOL;
			}
			else
			{
				$this->addError(str('STR_NOPERMISSION'), ERROR_TYPE_SERIOUS);
                $this->redirectLink = 'index.php';
			}                        
            
            
		}
	}
	
	/**
	 * We're going to always return true here and let 
	 * the submit method handle authorization since there 
	 * are so many different permutations.
	 */
	public function authorize()
	{
		return true;
	}
}
?>
