<?php
/**
 * The purpose of this class is to provide a simple mechanism 
 * for doing anything from simple to complex searches without having
 * to worry about queries.  Most search functions return a collection
 * object with all the objects returned from the search.   
 *   BUG: This will return an entire list of found objects so
 *			if the result set is large it can have an ...unfortunate...
 *			effect on script execution time.
 */ 
class shSearch
{
	/**
	 *  This will do a keyword search on all text-based fields in a document record 
	 *	and return the document collection object containing all matching documents
	 * 	@param array $keywords An array of keywords to search for
	 * 	@return shItemDocumentCollection The document collection object containing
	 *			all items found
	 */
	public function doDocumentKeywordSearch($keywords)
	{
		$params = new iwParameterCollection;
		$p = new iwParameterArray(FILTER_KEYWORDS);
		$p->setArray ($keywords);
		$params->add ($p);

        $p = new iwParameter(FILTER_SORTBY,'item_title');
        $params->add($p);
        $p = new iwParameter(FILTER_SORTDIR,'asc');
        $params->add($p);
		
        if (!app()->isAdminLoggedIn())
        {
            $p2 = new iwParameterArray(FILTER_USER,app()->getLoggedInUsername());
            $params->add($p2);           
        }
		
		$docs = new shItemDocumentCollection;
		$docs->load ($params);		
		
		return $docs;
	}
    public function doCategoryKeywordSearch($keywords)
    {
        $params = new iwParameterCollection;
        $p = new iwParameterArray(FILTER_KEYWORDS);
        $p->setArray ($keywords);
        $params->add ($p);

        $p = new iwParameter(FILTER_SORTBY,'name');
        $params->add($p);
        $p = new iwParameter(FILTER_SORTDIR,'asc');
        $params->add($p);
        
        if (!app()->isAdminLoggedIn())
        {
            $p2 = new iwParameterArray(FILTER_USER,app()->getLoggedInUsername());
            $params->add($p2);           
        }
        
        
        $docs = new shCategoryCollection;
        $docs->load ($params);        
        
        return $docs;
    }
}

?>