<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

class shUsageReport extends iwObject
{
	protected $fieldList;	
	protected $title;

    function __construct ()
    {
    	$this->fieldList = array 
    		(
    			'User'=>PROP_USERNAME,
    			'Date'=>'mod_date',    			
    			'Action'=>'action'
    		);

    	$this->_addProperty (PROP_START_DATE, '');
    	$this->_addProperty (PROP_END_DATE, '');
    	$this->_addProperty (PROP_GROUP_BY, REPORT_GROUP_REGION);    	
    }
    	    
	public function getTitle()
	{
		if ($this->get(PROP_GROUP_BY) == REPORT_GROUP_COMPANY)
		{
			return 'Usage Report by Organization/Group';
			
		}
		else if ($this->get(PROP_GROUP_BY) == REPORT_GROUP_REGION)
		{
			return 'Usage Report by Region';
			
		}
	}
		
    public function getStatistics ()
    {
    	    	
    	$array = array ();
    	$array['active_users'] = array(
						'icon'=>app()->getTemplateFile('images/user_icon_small.jpg'),
						'name'=>'Users', 
						'link'=>'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWUSER,
						'query'=>'select COUNT(*) AS total from iw_users',
						'value'=>0);
						
    	$array['active_docs'] = array(
						'icon'=>app()->getTemplateFile('images/doc_icon_small.jpg'),
						'name'=>'Documents', 
						'link'=>'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWDOC,
						'query'=>'select COUNT(*) AS total from sh_items WHERE item_status='.ITEM_STATUS_ACTIVE,
						'value'=>0);
						
    	$array['active_companies'] = array(
						'icon'=>app()->getTemplateFile('images/company_icon_small.jpg'),
						'name'=>'Organizations', 
						'link'=>'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWCOMPANY,
						'query'=>'select COUNT(*) AS total from sh_companies WHERE status='.COMPANY_STATUS_ACTIVE,
						'value'=>0);
						
    	$array['active_groups'] = array(
						'icon'=>app()->getTemplateFile('images/group_icon_small.jpg'),
						'name'=>'Groups', 
						'link'=>'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWGROUP,
						'query'=>'select COUNT(*) AS total from  sh_groups WHERE status='.GROUP_STATUS_ACTIVE,
						'value'=>0);
						
    	$array['active_categories'] = array(
						'icon'=>app()->getTemplateFile('images/category_icon_small.jpg'),
						'name'=>'Categories', 
						'link'=>'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWCAT,
						'query'=>'select COUNT(*) AS total from  sh_categories WHERE status='.CATEGORY_STATUS_ACTIVE,
						'value'=>0);
						
		$array['active_tags'] = array(
						'icon'=>app()->getTemplateFile('images/attrib_icon_small.jpg'),
						'name'=>'Tags', 
						'link'=>'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWATTRIB,
						'query'=>'select COUNT(*) AS total from sh_attributes WHERE status='.ATTRIBUTE_STATUS_ACTIVE,
						'value'=>0);
    	
    	$final = array();
    	foreach ($array as $key=>$details)
    	{
            app()->flushCacheIfInvalid($details['query']);
    		$res = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$details['query']);
    		if ($res !== false)
    		{
    			$details['value'] = $res[0]['total'];
    			$final[$key] = $details;
    		}
    		else 
    		{
    			app()->addError (sprintf(str('STR_REPORT_STATFAILED'),$details['name'],$res->message));
    		}
    	}
    	
    	return $final;
    }
    
    /**
     * Generate report based on current settings.
     *	@return string Returns the html for the report.
     */
    public function generateReport ()
    {
    	$collection = new shModCollection;
    	
    	$params = new iwParameterCollection();
		$p = new iwParameterArray(FILTER_MODIFY_DATE);
    	$p->addItem ('start',$this->get(PROP_START_DATE));
    	$p->addItem ('end',$this->get(PROP_END_DATE));
    	$params->add($p);
    	
    	// Load all changes during the time period given
    	$collection->load ($params);
    	    
    	// Now group as requested.
    	$html = '';
    	if ($this->get(PROP_GROUP_BY) == REPORT_GROUP_COMPANY)
    	{
			$html = $this->groupByCompanyThenGroup($collection);
    	}
    	else if ($this->get(PROP_GROUP_BY) == REPORT_GROUP_REGION)
    	{
    		$html = $this->groupByRegion($collection);
    	}
    	
    	return $html;
    }    

    protected function getRangeString ()
    {
		$start = strtotime ($this->get(PROP_START_DATE));
		$end = strtotime ($this->get(PROP_END_DATE));
		$start = date (settings()->date_format_short,$start);
		$end = date (settings()->date_format_short, $end); 

		return 'From '.$start.' to '.$end;	
    }
    
    protected function groupByRegion ($collection)
    {
		$regionGrouping = array();
		foreach ($collection->items() as $mod)
		{
			$user = $mod->user;
			if ($user != null) 
			{							
				$city = $user->get(PROP_CITY);
				$state = $user->get(PROP_STATE);
				
				if ($city == '' && $state != '') 
				{
					$region = $state;	
				}
				else if ($city != '' && $state == '')
				{
					$region = $city;
				}
				else if ($city == '' && $state == '')
				{
					$region = '[None Given]';
				}
				else 
				{
					$region = $city.','.$state;
				}
				
				$region = trim($region);				
				$regionGrouping[$region][] = $mod;
			}
		}
		
		// Sort by region
		ksort ($regionGrouping, SORT_STRING);
		
		// ... then by modification date in descending order.
		foreach ($regionGrouping as &$modArray)
		{
			// Sort the array by the modication date (descending);
			$arraySorter = new iwObjectArraySort($modArray,'mod_date',1,null,0,SORT_DATE);    				
			$modArray = $arraySorter->objArr;			
		}
		
		
		// Generate the report
		$html = '<h2>'.$this->getTitle();
		$html .= '<div class="subtext">'.$this->getRangeString().'</div>';
		$html .= '</h2>';
		$html .= '<table class="report" width="100%" cellpadding="2" cellspacing="0">';
		foreach ($regionGrouping as $region=>$modList)
		{			
			$html .= '<tr><td class="reportPrimaryGrouping" colspan="'.count($this->fieldList).'">'.$region.'</td></tr>';
		
			$other = false;
			foreach ($modList as $mod)
			{				
				$html .= '<tr bgcolor="'.($other?'#D6E7F7':'#ffffff').'">';
				foreach ($this->fieldList as $fieldName=>$property)
				{		
						
					$propertyText = $this->transformPropertyForDisplay($mod,$property);		
					$html .= '<td class="reportCell">'.$propertyText.'</td>';						
				}					
				$html .= '</tr>';
				
				$other = !$other;
			}
		}
		
		$html .= '</table>';
		
		return $html;
		
    }
    
    protected function groupByCompanyThenGroup ($collection)
    {	
		$companyGrouping = array();
		foreach ($collection->items() as $mod)
		{
			$user = $mod->user;
			if ($user != null) {
				
				$user->loadRelatedObjects();
				$company = $user->getCompany();
				if ($company != null)
				{
					$group = $user->getGroup();
					if ($group != null)
					{						
						$companyName = $company->get(PROP_NAME);
						$groupName = $group->get(PROP_NAME);
						
						if ($companyName == '' || $groupName == '')
						{
							continue;
						}
						else 
						{
							if (!isset ($companyGrouping[$companyName]))
							{
								$companyGrouping[$companyName] = array();
							}
							
							if (!isset ($companyGrouping[$companyName][$groupName]))
							{
								$companyGrouping[$companyName][$groupName] = array();
							}
							    						
							
							$companyGrouping[$companyName][$groupName][] = $mod;
						}
					}
				}
			}    			
		}
	
		// Sort the company array by company name
		ksort ($companyGrouping, SORT_STRING);
	
		// Now sort each of the companies' group list by group name;
		foreach ($companyGrouping as &$groupList)
		{
			ksort ($groupList, SORT_STRING);
			
			// Now sort each of the groups mod entries by date descending
			foreach ($groupList as &$modList)
			{
				// Sort the array by the modication date (descending);
				$arraySorter = new iwObjectArraySort($modList,'mod_date',1,null,0,SORT_DATE);    				
				$arraySorter->sortType = SORT_DATE;
				$modList = $arraySorter->objArr;
			}    				    			
		}    	
		
		$html = '<h3>'.$this->getTitle().'</h3>';
		$html .= '<h4>'.$this->getRangeString().'</h4>';
		$html .= '<table class="report" width="100%" cellpadding="2" cellspacing="0">';
		foreach ($companyGrouping as $companyName=>$groupList)
		{
			$html .= '<tr><td class="reportPrimaryGrouping" colspan="'.count($this->fieldList).'">'.$companyName.'</td></tr>';
			foreach ($groupList as $groupName=>$logItemList)
			{
				$html .= '<tr><td class="reportSecondaryGrouping" colspan="'.count($this->fieldList).'">'.$groupName.'</td></tr>';
				foreach ($logItemList as $mod)
				{
					$html .= '<tr>';
					foreach ($this->fieldList as $fieldName=>$property)
					{				
						$propertyText = $this->transformPropertyForDisplay($mod,$property);		
						$html .= '<td class="reportCell">'.$propertyText.'</td>';						
					}					
					$html .= '</tr>';
				}
			}
		}
		$html .= '</table>';
		
		return $html;
    }
    
	/**  Checks if the data stored in this object is valid.
     *	All this does is verify that there is valid data in the *required*
     *	fields.
     *	@param bool $checkKey If this is true, then the unique key field for the object is checked for validity as well
     *	@return bool True, if all required fields are valid.
	 */
	public function requiredFieldsValid ($checkKey = false) {

    	$startDate = $this->get(PROP_START_DATE);
    	$endDate = $this->get(PROP_END_DATE);
    	
		$start = strtotime ($startDate);
		$end = strtotime ($endDate);
		
		if ($end >= $start )	
		{
			return true;
		}
		
		return false;
	}

	protected function transformPropertyForDisplay ($item,$prop)
	{
		if ($prop == 'mod_date')
		{
			return iwUtilities::dateTimeFormat ($item->get($prop), false);
		}
		else if ($prop == PROP_USERNAME)
		{
			$link = '<a href="index.php?a=view_mod&filter_user='.$item->get($prop).'">%s</a>';
			return sprintf ($link,$item->get($prop));			
		}
		else if ($prop == 'client_info')
		{
			$bc = new iwBrowserCap ($item->get($prop));
			return $bc->property ('long_name').' '.$bc->property('version').' ('.$bc->property('platform').' '.$bc->property('os').')';			
		}
		else if ($prop == 'action')
		{
			$name = '';
			$type = '';
			
			switch ($item->get('target_type'))
			{
				case OBJECT_TYPE_COMPANY:
					$company = new shCompany;
					$company->set(PROP_ID, $item->get('target_id'));
					if ($company->load())
					{
						$name = $company->get(PROP_NAME);
					}
					
					$type = $company->getTypeName(false);
					break;
					
				case OBJECT_TYPE_GROUP:
					$group = new shGroup;
					$group->set(PROP_ID, $item->get('target_id'));
					if ($group->load())
					{
						$name = $group->get(PROP_NAME);
					}					
					$type = $group->getTypeName(false);					
					break;
				case OBJECT_TYPE_ITEM:
					$it = new shItemDocument;
					$it->set(PROP_ID, $item->get('target_id'));
					if ($it->load())
					{
						$name = $it->get(PROP_NAME);
					}					
					$type = $it->getTypeName(false);
					break;
				case OBJECT_TYPE_CATEGORY:
					$cat = new shCategory;
					$cat->set(PROP_ID, $item->get('target_id'));
					if ($cat->load())
					{
						$name = $cat->get(PROP_NAME);
					}					
					$type = $cat->getTypeName(false);
					break;
				case OBJECT_TYPE_SHARE_USER:
					$user = new shUser;
					$user->set(PROP_ID, $item->get('target_id'));
					if ($user->load())
					{
						$name = $user->get(PROP_USERNAME);
					}					
					$type = $user->getTypeName(false);
					break;
			}
			
			if ($type == '' && $name == '')
			{
				return $item->get('action');
			}
			else if ($type != '' && $name == '')
			{
				$link = '<a href="index.php?a=view_mod&filter_object_type='.$item->get('target_type').'&filter_object_id='.$item->get('target_id').'">%s</a>';
				return sprintf ($link,$item->get('action'). ' '.$type.' (with ID '.$item->get('target_id').')');
			}
			else
			{
				$link = '<a href="index.php?a=view_mod&filter_object_type='.$item->get('target_type').'&filter_object_id='.$item->get('target_id').'">%s</a>';
				return sprintf ($link,$item->get('action').' '.$type.' "'.$name.'"');
			}				
		}
	}
	
}
