<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * This class is used for the various user notifications that are sent out for various
 * reasons.  It is responsible for determining who is to be notified for various
 * events and will send those notifications as necessary
 */
class shUserNotification extends iwObject
{
	/**
	 * Adds the given user as a recipient of notifications whenever the given category
	 * is updated with a new document or an existing one is modified.
	 * @param shCategory $category The category being registered.
	 * @param string $type The type of notification to register for (NOTIFY_CHANGE_TYPE_UPDATE or NOTIFY_CHANGE_TYPE_ADD)
	 * @param string $username The username of the user to register
	 * @return bool Returns true if the registration was successful.
	 */
	function registerForCategoryNotification ($catId, $type, $username)
	{
		$username = app()->getLoggedInUsername();
		
		$query = 'insert into sh_category_user_notification_map '.
					'(category_id,username,notify_type,change_type) VALUES '.
					'('.$catId.',\''.$username.'\', \'email\', \''.$type.'\')';
			
		$ret = app()->getDatabase()->query($query);				
		if ($ret)
		{
			return true;
		}		
		else
		{
			$this->addErrorObject ($ret,ERROR_TYPE_SERIOUS);
			return false;
		}					
	}	
	
	function removeCategoryNotification ($catId, $type, $username)
	{
		$username = app()->getLoggedInUsername();
		
		$query = "delete from sh_category_user_notification_map WHERE ".
						 "category_id={$catId} AND ".
						 "username='$username' AND ".
						 "change_type='$type'";
						 			
		$ret = app()->getDatabase()->query($query);				
		
		if ($ret)
		{
			return true;
		}		
		else
		{
			$this->addErrorObject ($ret,ERROR_TYPE_SERIOUS);
			return false;
		}					
	}	
	
	public function isRegisteredForCategoryNotification ($catId, $type)
	{
		$user = app()->getLoggedInUserObject();
		$username = $user->get(PROP_USERNAME);
		
		$query = "select count(*) from sh_category_user_notification_map ".
						"where username='$username' and category_id=$catId ".
						"AND change_type='$type'";
						
        app()->flushCacheIfInvalid($query);
		$count = app()->getDatabase()->CacheGetOne(DB_CACHE_EXPIRE,$query);
		if ( ($count))	
		{
			return ($count > 0);
		}
		
		return false;
	}

	/**
	 * Sends notifications to anyone registered for notifications on this
	 * document.  
	 * @param shItemDocument $item The document object that was added
	 */
	public function notifyAdd ($item)
	{
		$emails = $this->getCategoryEmailList($item, NOTIFY_CHANGE_TYPE_ADD);
		
		$subject = 
		'Nexus Document Add Notification: '. $item->get(PROP_NAME);
				
		$msg = str('STR_EMAIL_NOTIFY_ADD1');
			
		$msg .= str('STR_EMAIL_NOTIFY_ADD2',$item->get(PROP_NAME),$item->get(PROP_CREATION_DATE));
			
		$link = IW_BASE_URL."/index.php?".URLARG_ACTION."=".URLACTION_VIEWDOC."&".URLARG_ID."=".$item->get(PROP_ID);
		$msg .= str('STR_EMAIL_NOTIFY_ADD3',$link);
		
		foreach ($emails as $emailInfo)
		{
			$email = $emailInfo['email'];
			$user = $emailInfo['username'];
			
			if (shAuthorize::canViewDocument($user, $item->get(PROP_ID)))
			{
				app()->sendMail (settings()->admin_email, $email, $subject, $msg, array());
			}
		}
	}
	
	/**
	 * Sends notifications to anyone registered for notifications on this
	 * document.  
	 * @param shItemDocument $item The document object that was added
	 */
	public function notifyUpdate ($item)
	{
		$emails = $this->getCategoryEmailList($item, NOTIFY_CHANGE_TYPE_UPDATE);
		
		$subject = 
		'Nexus Document Update Notification: '. $item->get(PROP_NAME);
		
		$link = IW_BASE_URL."/index.php?".URLARG_ACTION."=".URLACTION_VIEWDOC."&".URLARG_ID."=".$item->get(PROP_ID);
		$msg = str('STR_EMAIL_NOTIFY_UPDATE', $item->get(PROP_NAME), iwUtilities::dateTimeFormat($item->get(PROP_MODIFIED_DATE), true), $link);
		
		foreach ($emails as $emailInfo)
		{
			$email = $emailInfo['email'];
			$user = $emailInfo['username'];
			
			if (shAuthorize::canViewDocument($user, $item->get(PROP_ID)))
			{
				app()->sendMail (settings()->admin_email, $email, $subject, $msg, array());
			}
		}
	}
		
	/**
	 * Retrieves the emails of those people who have registered to be notified when
	 *	this document is added/updated in the library
	 */
	protected function getCategoryEmailList ($item, $type)
	{
		$catIds = array();
		$categories = $item->getParentCategories();
		if ($categories->count() > 0)
		{
			foreach ($categories->items() as $cat)
			{
				array_push($catIds,$cat->get(PROP_ID));
			}
		}
		
		if (count($catIds) == 0)
		{
			array_push($catIds,0);
		}
		
		$emails = array();
		$cats = join (',',$catIds);
		$query = 'select sh_category_user_notification_map.username, iw_users.email '.
					'from sh_category_user_notification_map INNER JOIN iw_users ON '.
					'iw_users.username = sh_category_user_notification_map.username '.
					'where category_id IN ('.$cats.') AND change_type=\''.$type.'\'';

        app()->flushCacheIfInvalid($query);
		$results = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($results !== false)	
		{
			foreach ($results as $row)
			{
				array_push($emails,array('username'=>$row['username'],'email'=>$row['email']));
			}
			
			return $emails;
		}

		return array();
	}
	
	/**
	 * Sends an email to the given company/group pair
	 *  @param int $cid The company id
	 *  @param int $gid The group id
	 *  @param string $subject The subject of the email
	 *  @param string $body The body of the email
	 */
	public function sendMailToCompanyGroup ($cid, $gid, $subject, $body)
	{
		// Only administrators can do this.
		if (app()->isAdminLoggedIn())
		{
			$group = new shGroup;
			$company = new shCompany;
			if (!$group->loadId($gid))
			{
				$this->addError (str('STR_MAIL_GROUP_NOTFOUND'), ERROR_TYPE_SERIOUS);
				return;
			}
			
			if (!$company->loadId($cid))
			{
				$this->addError (str('STR_MAIL_GROUP_NOTFOUND'), ERROR_TYPE_SERIOUS);
				return;
			}
						
			$pair = new shCompanyGroupPair();
			$pair->setGroup($group);
			$pair->setCompany($company);
			$users = $pair->getUsers();
			
			$failedCount = 0;
			if (count($users) == 0)
			{
				$this->addError (sprintf(str('STR_MAIL_NOUSERS'),$group->get(PROP_NAME),$company->get(PROP_NAME)), ERROR_TYPE_WARNING);
				return;			
			}
			
			foreach ($users as $user)
			{				
				$prefixSubject = str('STR_EMAIL_GROUPMSG');

				$prefixBody = str('STR_EMAIL_GROUPMSG_BODY', app()->getLoggedInUsername(), 
					$pair->getCompany()->get(PROP_NAME),$pair->getGroup()->get(PROP_NAME));
								
				$success = 
					app()->sendMail (settings()->admin_email, 
										$user->get(PROP_EMAIL), 
										$prefixSubject.$subject, 
										$prefixBody.$body, array());
										
				if (!$success)
				{
					$failedCount++;
				}
			}			
			               
			if ($failedCount > 0)
			{                
				$this->addError(str('STR_EMAIL_GROUPMSG_ERR',$failedCount,count($users)), ERROR_TYPE_SERIOUS);
			}
			else
			{
				$this->addNotification(str('STR_MAIL_SENT_ALL'));
			}
		}
	}
	
	/**
	 * Sends an email to the given user
	 *  @param int $username The username to send to
	 *  @param string $subject The subject of the email
	 *  @param string $body The body of the email
	 */
	public function sendMailToUser ($username, $subject, $body)
	{
		// Only administrators can do this.
		if (app()->isUserLoggedIn())
		{
			$user = new shUser;
			$user->set(PROP_USERNAME, $username);
			if ($user->load())
			{
				$prefixSubject = str('STR_EMAIL_USERMSG');
				
				$prefixBody = str('STR_EMAIL_USERMSG_BODY', app()->getLoggedInUsername(), $username);
												
				$success = 
					app()->sendMail (settings()->admin_email, 
									$user->get(PROP_EMAIL), 
									$prefixSubject.$subject, 
									$prefixBody.$body, array());
									
				if ($success)
				{
					$this->addNotification("Email was sent successfully");											
				}
			}
			else
			{
				$this->addError(sprintf(str('STR_MAIL_USER_NOTFOUND'),$username), ERROR_TYPE_SERIOUS);				
			}
		}
		else
		{
			$this->addError(str('STR_NOPERMISSION'), ERROR_TYPE_SERIOUS);
		}
	}	
}
?>