<?php

class shUploaderAjax extends shUploader
{
    protected $_startingFile = null;
    protected $_uploadFinishedHandler = 'onUploadFinished';
    
    /**
    * @desc Use this to prepopulate the markup with information about an existing file. 
    *       This is useful when this uploader is being displayed in form that represents
    *       an existing document
    * @param shUploadDocumentProperties $fileInfo The information about the file    
    */
    public function setStartingFile(shUploadDocumentProperties $fileInfo)
    {
        $this->_startingFile = $fileInfo;
    }
    
    /**
    * @desc Set to the name of the javascript function that should be called after the upload
    *       has been completed.
    * @param string $handler The javascript function name to call    
    */
    public function setJsUploadFinishedHandler($handler)
    {
        $this->_uploadFinishedHandler = $handler;
    }
    
	public function getMarkup()
	{        
        $smarty = app()->getSmartyObject();
        $smarty->assign('max_document_size',iwUtilities::byteFormat(DOCUMENT_MAX_FILESIZE)); 
        
        if ($this->_startingFile)
        {
            $smarty->assign('doc_filename',$this->_startingFile->originalFilename); 
            $smarty->assign('doc_size',iwUtilities::byteFormat($this->_startingFile->fileSize)); 
            $smarty->assign('doc_binary',true);             
        }
        
        $smarty->assign('upload_success_callback', $this->_uploadFinishedHandler);
        $smarty->assign('form_action',$this->createBaseTargetUrl());
                                                                 
		return smarty_templ_fetch('controls/control.ajax.uploaddoc.tpl.html', $smarty);
	}
	
	public function handleFiles()
	{	
        $json = new Services_JSON();
        
        uploads()->clearUserRecentFiles();
        uploads()->clearUserPurgatory();
        
		foreach ($_FILES as $file)
		{
            $info = new shUploadDocumentProperties;
		 	$info->localFilePath = $file['tmp_name'];
			$info->originalFilename = $file['name'];
            $info->mimeType = $file['type'];
			$info->parentCategoryId = $this->getCategory();
			$info->domainsWithAccess = array();
			$info->uploadLocation = $this->getUploadLocation();
			$info->description = '';
			$info->name = $file['name'];
			                        
			uploads()->addFile($info);
		}
        
        uploads()->doUpload();
        $files = uploads()->getOutputFiles();  
        
        $results = array();

        if (is_array($files) && count($files) > 0)
        {
            $file = $files[0];
            if ($file->uploadResult == UPLOAD_RES_SUCCESS)
            {
                $results['message'] = 'The file '.$file->originalFilename.' was uploaded successfully';
                $results['filesize'] = iwUtilities::byteFormat($file->fileSize);
                $results['filename'] = $file->originalFilename;
                $results['filetype'] = $file->mimeType; 
            }
            else
            {
                $results['message'] = 'The file could not be uploaded';
            }
        }
        else
        {
            $results['message'] = 'The file was not posted to the server correctly.'; 
        }
    
        print($json->encode($results));                
	}
}

?>