<?php
class FileLockForm extends iwForm
{
    public function getFormAction()
    {
        return 'filelock_settimeout';    
    }
        
    function getFormType()
    {
        return 'ftfilelock';    
    }
    
    public function getTitle()
    {
        return 'File Lock Settings';
    }
            
    public function getFields()
    {
        $fields = array();
        
        $doc = new shItemDocument();
        if (!$doc->loadId($this->getFormOrUrlId()))
        {
             return array();
        }
               
        $expirationInfo = filelock_get_expiration_info($doc->get(PROP_ID));
    
        if (!$expirationInfo)
        {
            $defaultInterval = settings()->__get(GLOBAL_SETTING_NAME);
            $time = mktime(0,0,0,date('m'),date('d')+$defaultInterval,date('Y'));
        }
        else
        {
            $time = strtotime($expirationInfo['expiration_date']);
        }
        $timeString = iwUtilities::dateFormat($time,false);


        $r = $this->getData();
        $fields[] = new shFormField(FIELD_TYPE_HIDDEN,g(PROP_ID),PROP_ID);
        $fields[] = new shFormField(FIELD_TYPE_STATIC,$doc->get(PROP_NAME).'<br/>'.$doc->get(PROP_DESCRIPTION),'document',"Document Info");
        $fields[] = new shFormField(FIELD_TYPE_TEXT,$timeString,'timeout_date',"Expiration Date");
        $fields[] = new shFormField(FIELD_TYPE_TEXTAREA,$r['comment'],'comment',"Comments (displayed near file prior to expiration)");
        
        if (is_array($expirationInfo) && count($expirationInfo) > 0)
        {
            $fields[] = new shFormField(FIELD_TYPE_CHECKBOX,0,'remove_expiration','Click checkbox to remove expiration for file');
        }
                
        return $fields;    
    }
    
    public function getRedirectLink()
    {
        $id = $this->getFormOrUrlId();
        if ($id != 0)
        {
            return '?'.URLARG_ACTION.'='.URLACTION_VIEWDOC.
                    '&'.URLARG_ID.'='.$id;
        }
        
        return '';        
    }
    
    /**
     * If the ID parameter is in the url then that group
     * is loaded and stored in the data field of the class
     * otherwise a new object is created and set.
     * @return bool Returns true unless an invalid id is given
     *                 in the url id parameter. 
     */
    protected function loadData()
    {
        // Check the cache first.    
        if ($this->recallFormData())
        {
            return true;
        }
        
        $result = filelock_get_expiration_info($this->getUrlIdField());
        
        if ($result)
        {
            $this->setData($result);            
        }
        
        return true;
    }
    
    /**
     * Override to implement form submit for this object
     * @return bool Returns true if the form submission was successful 
     */
    function _submit ()
    {                
        $r = $this->getData();
        if (!is_array($r))
        {
            $r = array();
        }
        
        $remove_expiration = ps('remove_expiration')?1:0;
                                                       
        $timeout_date = ps('timeout_date');
        $r['document_id'] = ps(PROP_ID);
        $time = strtotime($timeout_date);
        
        if ($time === false)
        {
            $this->addError ("The given expiration date is invalid");        
            return false;
        }
        
        $r['expiration_date'] = iwUtilities::databaseDateFormat($time);
        $r['comment'] = ps('comment');
        
        if ($remove_expiration)
        {
            $result = db()->Execute("delete from sh_filelock_files where document_id={$r['document_id']}");
            if (!$result)
            {
                $this->addError('Unable to delete expiration info from database');
                return false;
            }      
            else
            {
                $this->addNotification('Successfully removed expiration info for file');
                return true;
            }      
        }
                        
        $this->setData($r);
        $this->rememberFormData($r);
        
        $dbTime = db()->DBTimeStamp($time);
        $count = db()->GetOne('select count(*) from sh_filelock_files where document_id='.$r['document_id']);
        if ($count)
        {
            // update the existing entry
            $result = db()->Execute("update sh_filelock_files set 
                    expiration_date={$dbTime}
                    ,comment='{$r['comment']}' where document_id={$r['document_id']}");
                    
            if (!$result)
            {
                $this->addError('Unable to update expiration info for file');
                return false;
            }
            else
            {
                $this->addNotification('Successfully update expiration info for file');
                return true;
            }
        }
        else
        {
            // add a new entry
            $result = db()->Execute("insert into sh_filelock_files 
                    (document_id,expiration_date,comment) values 
                    ({$r['document_id']},{$dbTime},'{$r['comment']}')");
                        
            if (!$result)
            {
                $this->addError('Unable to add expiration info for file');
                return false;
            }
            else
            {
                $this->addNotification('Successfully added expiration info for file');
                return true;
            }
        }
                        
                
        return false;        
    }

    function authorize ()
    {
        $id = g(URLARG_ID);
        if (!$id) 
        {
            $id = ps(PROP_ID);
        }
        
        $user = app()->getLoggedInUserObject();
        return $user->getPermissions()->isAllowed(URLACTION_MODCAT,$id);
    }

    /**
     * Returns the results of the form submission.  The results value is an
     * an empty array if the form submit failed.  Otherwise, it contains an
     * associative array:
     * 
     * 'name' => group name
     * 'id' => group id
     *
     * @return array The array described above. 
     */
    function getResults()
    {
        if ($this->_results)
        {
            return array('name'=>$this->_results->get(PROP_NAME), 
                     'id'=>$this->_results->get(PROP_ID));
        }
        
        return array();
    }    
    
}
?>
